/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/

package jonelo.NumericalChameleon.Clusters;
import java.math.*;
import java.util.*;
import java.text.*;
import jonelo.NumericalChameleon.*;
import jonelo.sugar.util.*;

public class ClusterCalendarYear extends ClusterObject {

    public final static int INTERNAL=93;
    public final static String CODE="CALENDARYEAR";

    private final static int
      // source
      cYEAR=               0,

      // target
      cLEAPYEAR=           0,
      cCENTURY=            1,
      cMILLENNIUM=         2,

      // USA
      usNEWYEAR=           3,
      usMARTINLUTHER=      4,
      usVALENTINESDAY=     5,
      usPRESIDENTSDAY=     6,
      usDAYLIGHTBEGIN=     7,
      usMEMORIALDAY=       8,
      usINDEPENDENCEDAY=   9,
      usLABORDAY=         10,
      usCOLUMBUSDAY=      11,
      usDAYLIGHTEND=      12,
      usHALLOWEEN=        13,
      usVETERANSDAY=      14,
      usTHANKSGIVINGDAY=  15,
      usCHRISTMASDAY=     16,

      usELECTIONDAY=      17,
      usCHRISTMASEVE=     18,
      usNEWYEARSEVE=      19,
      usMOTHERSDAY=       20,
      usEASTERDAY=        21,
      usSAINTPATRICKSDAY= 22,
      usLINCOLNSBIRTHDAY= 23,
      usGROUNDHOGDAY=     24,
      usFLAGDAY=          25,
      usSECRETARYSDAY=    26,
      usFATHERSDAY=       27,
      usTAXDAY=           28,

      // GERMANY
      deNEUJAHR=          29,
      deHL3KOENIGE=       30,
      deVALENTINSTAG=     31,
      deROSENMONTAG=      32,
      deFASTNACHT=        33,
      deASCHERMITTWOCH=   34,
      deSOMMERZEIT=       35,
      dePALMSONNTAG=      36,
      deGRUENDONNERSTAG=  37,
      deKARFREITAG=       38,
      deOSTERSONNTAG=     39,
      deOSTERMONTAG=      40,
      deWEISSERSONNTAG=   41,
      deMAIFEIERTAG=      42,
      deMUTTERTAG=        43,
      deCHRISTI=          44,
      dePFINGSTSONNTAG=   45,
      dePFINGSTMONTAG=    46,
      deGEDENKTAG=        47,
      deFRONLEICHNAM=     48,
      deFRIEDENSFEST=     49,
      deMARIAHIMMELFAHRT= 50,
      deTAGDERDTEINHEIT=  51,
      deERNTEDANKFEST=    52,
      deWINTERZEIT=       53,
      deREFORMATIONSTAG=  54,
      deALLERHEILIGEN=    55,
      deALLERSEELEN=      56,
      deMARTINSTAG=       57,
      deVOLKSTRAUERTAG=   58,
      deBUSSUNDBETTAG=    59,
      deTOTENSONNTAG=     60,
      deNIKOLAUS=         61,
      deMARIAEEMFAENGNIS= 62,
      deADVENT1=          63,
      deADVENT2=          64,
      deADVENT3=          65,
      deADVENT4=          66,
      deHEILIGABEND=      67,
      deWEIHNACHTEN1=     68,
      deWEIHNACHTEN2=     69,
      deSILVESTER=        70,

      // SWITZERLAND
      chNEUJAHR=          71,
      chBERCHTOLDSTAG=    72,
      chVALENTINSTAG=     73,
      chASCHERMITTWOCH=   74,
      chSOMMERZEIT=       75,
      chPALMSONNTAG=      76,
      chOSTERSONNTAG=     77,
      chCHRISTI=          78,
      chMUTTERTAG=        79,
      chPFINGSTSONNTAG=   80,
      chNATIONALSCHWEIZ=  81,
      chEIDGENOESSISCHER= 82,
      chWINTERZEIT=       83,
      chALLERHEILIGEN=    84,
      chTOTENSONNTAG=     85,
      chADVENT1=          86,
      chADVENT2=          87,
      chADVENT3=          88,
      chADVENT4=          89,
      chHEILIGABEND=      90,
      chWEIHNACHTEN1=     91,
      chSILVESTER=        92
      ;




    public static String
      NOT_DEFINED = "---",
      YEAR = "Year",
      LEAPYEAR = "Leapyear",
      CENTURY = "Century",
      MILLENNIUM= "Millennium",

      INVALID = "Source is invalid",
      YES = "yes",
      NO = "no";

    private Vector vs,vt;

    public ClusterCalendarYear() {
        // source
        vs=new Vector();
        vs.add(new NcRecord(YEAR,"int"));

        // target
        // international
        vt=new Vector();
        vt.add(new NcRecord(LEAPYEAR,"int"));
        vt.add(new NcRecord(CENTURY,"int"));
        vt.add(new NcRecord(MILLENNIUM,"int"));

        // USA
        vt.add(new NcRecord("New Year's Day [legal holiday]","us"));
        vt.add(new NcRecord("Martin Luther King Day [legal holiday]","us"));
        vt.add(new NcRecord("Valentine's Day","us"));
        vt.add(new NcRecord("President's Day [legal holiday]","us"));
        vt.add(new NcRecord("Begin Daylight Saving Time (DST), +1h","us"));
        vt.add(new NcRecord("Memorial Day [legal holiday]","us"));
        vt.add(new NcRecord("Independence Day [legal holiday]","us"));
        vt.add(new NcRecord("Labor Day [legal holiday]","us"));
        vt.add(new NcRecord("Columbus Day [legal holiday]","us"));
        vt.add(new NcRecord("End Daylight Saving Time (DST), -1h","us"));
        vt.add(new NcRecord("Halloween","us"));
        vt.add(new NcRecord("Veteran's Day [legal holiday]","us"));
        vt.add(new NcRecord("Thanksgiving Day [legal holiday]","us"));
        vt.add(new NcRecord("Christmas Day [legal holiday]","us"));

        // verteilen
        vt.add(new NcRecord("Election Day","us"));
        vt.add(new NcRecord("Christmas Eve","us"));
        vt.add(new NcRecord("New Year's Eve","us"));
        vt.add(new NcRecord("Mother's Day","us"));
        vt.add(new NcRecord("Easter Day","us"));
        vt.add(new NcRecord("Saint Patrick's Day","us"));
        vt.add(new NcRecord("Lincoln's Birthday","us"));
        vt.add(new NcRecord("Groundhog Day","us"));
        vt.add(new NcRecord("Flag Day","us"));
        vt.add(new NcRecord("Administrative Professionals Day (Secretary's Day)","us"));
        vt.add(new NcRecord("Father's Day","us"));
        vt.add(new NcRecord("Tax Day","us"));

        // Germany
        // http://www.bmi.bund.de/services/lexikon/lexikon.jsp?key=F&hit=Feiertage
        vt.add(new NcRecord("Neujahrstag [gesetzlich]","de"));
        vt.add(new NcRecord("Hl. Drei Knige [BW,BY,ST]","de"));
        vt.add(new NcRecord("Valentinstag","de"));
        vt.add(new NcRecord("Rosenmontag","de"));
        vt.add(new NcRecord("Fastnacht (Faschingsdienstag)","de"));
        vt.add(new NcRecord("Aschermittwoch","de"));
        vt.add(new NcRecord("Beginn der Sommerzeit (Uhren 1 h vorstellen)","de"));
        vt.add(new NcRecord("Palmsonntag","de"));
        vt.add(new NcRecord("Grndonnerstag","de"));
        vt.add(new NcRecord("Karfreitag [gesetzlich]","de"));
        vt.add(new NcRecord("Ostersonntag (Ostern) [gesetzlich]","de"));
        vt.add(new NcRecord("Ostermontag [gesetzlich]","de"));
        vt.add(new NcRecord("Weier Sonntag","de"));
        vt.add(new NcRecord("Maifeiertag (Tag der Arbeit) [gesetzlich, seit 1934]","de"));
        vt.add(new NcRecord("Muttertag","de"));
        vt.add(new NcRecord("Christi Himmelfahrt [gesetzlich]","de"));
        vt.add(new NcRecord("Pfingstsonntag (Pfingsten) [gesetzlich]","de"));
        vt.add(new NcRecord("Pfingstmontag [gesetzlich]","de"));
        vt.add(new NcRecord("Nationaler Gedenktag [1953-1990]","de",false));
        vt.add(new NcRecord("Fronleichnam [BW,BY,HE,NW,RP,SL,SN teilw.,TH teilw.]","de"));
        vt.add(new NcRecord("Friedensfest [gesetzlich nur in Augsburg]","de"));
        vt.add(new NcRecord("Mari Himmelfahrt [SL,BY kath.]","de"));
        vt.add(new NcRecord("Tag der deutschen Einheit [gesetzlich seit 1990]","de"));
        vt.add(new NcRecord("Erntedankfest","de"));
        vt.add(new NcRecord("Ende der Sommerzeit (Uhren 1 h zurckstellen)","de"));
        vt.add(new NcRecord("Reformationstag [gesetzlich in BB, MV, SN, ST, TH]","de"));
        vt.add(new NcRecord("Allerheiligen [gesetzlich in BW, BY, NW, RP, SL]","de"));
        vt.add(new NcRecord("Allerseelen","de"));
        vt.add(new NcRecord("Martinstag","de"));
        vt.add(new NcRecord("Volkstrauertag [seit 1952]","de"));
        vt.add(new NcRecord("Bu- und Bettag [gesetzlich in SA, vor 1995 bundeseinheitlich]","de"));
        vt.add(new NcRecord("Totensonntag","de"));
        vt.add(new NcRecord("Nikolaus","de"));
        vt.add(new NcRecord("Mari Empfngnis","de"));
        vt.add(new NcRecord("1. Advent","de"));
        vt.add(new NcRecord("2. Advent","de"));
        vt.add(new NcRecord("3. Advent","de"));
        vt.add(new NcRecord("4. Advent","de"));
        vt.add(new NcRecord("Heiligabend","de"));
        vt.add(new NcRecord("1. Weihnachtsfeiertag (Weihnachten) [gesetzlich]","de"));
        vt.add(new NcRecord("2. Weihnachtsfeiertag [gesetzlich]","de"));
        vt.add(new NcRecord("Silvester","de"));

        // Schweiz
        vt.add(new NcRecord("Neujahr [gesetzlich]","ch"));
        vt.add(new NcRecord("Berchtoldstag (Bchtelistag) [nicht in allen Kantonen]","ch"));
        vt.add(new NcRecord("Valentinstag","ch"));
        vt.add(new NcRecord("Aschermittwoch","ch"));
        vt.add(new NcRecord("Beginn der Sommerzeit (Uhren 1 h vorstellen)","ch"));
        vt.add(new NcRecord("Palmsonntag","ch"));
        vt.add(new NcRecord("Ostersonntag (Ostern) [gesetzlich]","ch"));
        vt.add(new NcRecord("Auffahrt [gesetzlich]","ch"));
        vt.add(new NcRecord("Muttertag","ch"));
        vt.add(new NcRecord("Pfingstsonntag (Pfingsten) [gesetzlich]","ch"));
        vt.add(new NcRecord("Nationalfeiertag Schweiz (Bundesfeier) [gesetzlich], seit 1993 gesamtschweizerisch","ch"));
        vt.add(new NcRecord("Eidgenssischer Dank-, Buss- und Bettag","ch"));
        vt.add(new NcRecord("Ende der Sommerzeit (Uhren 1 h zurckstellen)","ch"));
        vt.add(new NcRecord("Allerheiligen","ch"));
        vt.add(new NcRecord("Totensonntag","ch"));
        vt.add(new NcRecord("1. Advent","ch"));
        vt.add(new NcRecord("2. Advent","ch"));
        vt.add(new NcRecord("3. Advent","ch"));
        vt.add(new NcRecord("4. Advent","ch"));
        vt.add(new NcRecord("Heiliger Abend","ch"));
        vt.add(new NcRecord("Weihnachten [gesetzlich]","ch"));
        vt.add(new NcRecord("Silvester","ch"));

    }

    public Vector getSourceEntries()  {
        return vs;
    }

    public Vector getTargetEntries() {
        return vt;
    }

    public int getSourceDefault() {
        return 0;
    }

    public int getTargetDefault() {
        return 0;
    }

    public boolean isOneway() {
        return true;
    }
    /** get Output
     * @param s soure
     * @param t tareget
     * @throws Exception if there is a problem
     * @return String
     */
    public String getOutput(int s, int t) throws Exception
    {
        ExtendedGregorianCalendar gc = new ExtendedGregorianCalendar();
        int year = 0;

        switch (s) {
            // international
            case cYEAR:   try {
                            year = Integer.parseInt(input);
                            if ((year <=0 ) || (year > 9999)) throw new Exception(INVALID);
                          } catch (Exception e) { throw new Exception(INVALID); }
                          break;
        }

        DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);

        int c=0;
        switch (t) {

            case cLEAPYEAR:           gc.set(year,0,1);
                                      return gc.isLeapYear(year) ? YES:NO;

            case cCENTURY:            int major = year / 100;
                                      int minor = year % 100;
                                      if (minor > 0) major++;
                                      return major+".";

            case cMILLENNIUM:         major = year / 1000;
                                      minor = year % 1000;
                                      if (minor > 0) major++;
                                      return major+".";

            // January
            case deNEUJAHR:
            case chNEUJAHR:
            case usNEWYEAR:           gc.set(year, Calendar.JANUARY, 1);
                                      break;

            case chBERCHTOLDSTAG:     gc.set(year, Calendar.JANUARY, 2);
                                      break;

            case deHL3KOENIGE:        gc.set(year, Calendar.JANUARY, 6);
                                      break;


            case usMARTINLUTHER:      if (year >= 1984) {
                                        gc.setNthDow(
                                          ExtendedGregorianCalendar.THIRD,
                                          ExtendedGregorianCalendar.MONDAY,
                                          ExtendedGregorianCalendar.JANUARY,
                                          year);
                                        break;
                                      } else return "nicht definiert";
            // February
            case usGROUNDHOGDAY:      gc.set(year, Calendar.FEBRUARY, 2);
                                      break;

            case usLINCOLNSBIRTHDAY:  gc.set(year, Calendar.FEBRUARY, 12);
                                      break;

            case deVALENTINSTAG:
            case chVALENTINSTAG:
            case usVALENTINESDAY:     gc.set(year, Calendar.FEBRUARY, 14);
                                      break;

            case usPRESIDENTSDAY:     gc.setNthDow(
                                        ExtendedGregorianCalendar.THIRD,
                                        ExtendedGregorianCalendar.MONDAY,
                                        ExtendedGregorianCalendar.FEBRUARY,
                                        year);
                                      break;

            case deROSENMONTAG:       gc.setEaster(year);
                                      gc.add(Calendar.DATE, -48);
                                      break;

            case deFASTNACHT:         gc.setEaster(year);
                                      gc.add(Calendar.DATE, -47);
                                      break;

            case deASCHERMITTWOCH:
            case chASCHERMITTWOCH:    gc.setEaster(year);
                                      gc.add(Calendar.DATE, -46);
                                      break;

            case usSAINTPATRICKSDAY:  gc.set(year, Calendar.MARCH, 17);
                                      break;

            case deSOMMERZEIT:        try {
                                          gc.setDSTstart(year, TimeZone.getTimeZone("Europe/Berlin"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case chSOMMERZEIT:        try {
                                          gc.setDSTstart(year, TimeZone.getTimeZone("Europe/Zurich"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case usDAYLIGHTBEGIN:     try {
                                          // Zeitzone New York stellvertretend fr die gesamte USA
                                          gc.setDSTstart(year, TimeZone.getTimeZone("America/New_York"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case usTAXDAY:            // http://www.calendar-updates.com/Holidays/US/tax.htm
                                      // Income taxes are usually due on April 15th from all employed Americans.  If April 15th falls on a Saturday or a Sunday, the due date is extended to the next non-holiday weekday.
                                      gc.set(year, Calendar.APRIL, 15);
                                      if (gc.get(Calendar.DAY_OF_WEEK)==Calendar.SATURDAY)
                                      gc.add(Calendar.DATE,+2);
                                      else
                                      if (gc.get(Calendar.DAY_OF_WEEK)==Calendar.SUNDAY)
                                      gc.add(Calendar.DATE,+1);
                                      break;

            case usSECRETARYSDAY:     // Administrative Professionals Day
                                      // = Observed annually the Wednesday of Administrative Professionals Week
                                      // Administrative Professionals Week
                                      // = Observed annually during the last FULL week in April, standard Sunday through Saturday calendar week.
                                      if (year >= 1952) {
                                        gc.setNthDow(
                                          ExtendedGregorianCalendar.LAST,
                                          ExtendedGregorianCalendar.SATURDAY,
                                          ExtendedGregorianCalendar.APRIL,
                                          year);
                                        gc.add(Calendar.DATE, -3); // this is Wednesday
                                        break;
                                      } else return "nicht definiert";

            case dePALMSONNTAG:
            case chPALMSONNTAG:       gc.setEaster(year);
                                      gc.add(Calendar.DATE, -7);
                                      break;

            case deGRUENDONNERSTAG:   gc.setEaster(year);
                                      gc.add(Calendar.DATE, -3);
                                      break;

            case deKARFREITAG:        gc.setEaster(year);
                                      gc.add(Calendar.DATE, -2);
                                      break;

            case deOSTERSONNTAG:
            case chOSTERSONNTAG:
            case usEASTERDAY:         gc.setEaster(year);
                                      break;

            case deOSTERMONTAG:       gc.setEaster(year);
                                      gc.add(Calendar.DATE, +1);
                                      break;

            case deWEISSERSONNTAG:    gc.setEaster(year);
                                      gc.add(Calendar.DATE, +7);
                                      break;

            // May
            case deMAIFEIERTAG:       if (year >= 1934) {
                                       gc.set(year,Calendar.MAY,1);
                                       break;
                                      } else
                                      return "nicht definiert";

            case deMUTTERTAG:
            case chMUTTERTAG:
            case usMOTHERSDAY:       gc.setNthDow(
                                        ExtendedGregorianCalendar.SECOND,
                                        ExtendedGregorianCalendar.SUNDAY,
                                        ExtendedGregorianCalendar.MAY,
                                        year);
                                      break;

            case usMEMORIALDAY:      gc.setNthDow(
                                        ExtendedGregorianCalendar.LAST,
                                        ExtendedGregorianCalendar.MONDAY,
                                        ExtendedGregorianCalendar.MAY,
                                        year);
                                      break;

            case deCHRISTI:
            case chCHRISTI:           gc.setEaster(year);
                                      gc.add(Calendar.DATE, +39);
                                      break;

            case dePFINGSTSONNTAG:
            case chPFINGSTSONNTAG:    gc.setEaster(year);
                                      gc.add(Calendar.DATE, +49);
                                      break;

            case dePFINGSTMONTAG:     gc.setEaster(year);
                                      gc.add(Calendar.DATE, +50);
                                      break;

            case usFLAGDAY:           gc.set(year, Calendar.JUNE, 14);
                                      break;

            case usFATHERSDAY:        // http://www.holidays.net/father/story.htm
                                      // in 1966 President Lyndon Johnson signed a presidential proclamation declaring the 3rd Sunday of June as Father's Day.
                                      gc.setNthDow(
                                        ExtendedGregorianCalendar.THIRD,
                                        ExtendedGregorianCalendar.SUNDAY,
                                        ExtendedGregorianCalendar.JUNE,
                                        year);
                                      break;

            case deGEDENKTAG:         if ((year >= 1953) && (year <= 1990))
                                      {
                                        gc.set(year,Calendar.JUNE,17);
                                        break;
                                      } else return "nicht definiert";

            case deFRONLEICHNAM:      gc.setEaster(year);
                                      gc.add(Calendar.DATE, +60);
                                      break;

            case usINDEPENDENCEDAY:   gc.set(year, Calendar.JULY, 4);
                                      break;

            case chNATIONALSCHWEIZ:   if (year >= 1899) {
                                         gc.set(year, Calendar.AUGUST, 1);
                                         break;
                                      } else return "nicht definiert";

            case deFRIEDENSFEST:      gc.set(year, Calendar.AUGUST, 8); // http://www.bmi.bund.de/Bild/prmedienoriginal_3457/Uebersicht_der_gesetzlichen_Feiertage_in_Deutschland.gif
                                      break;

            case deMARIAHIMMELFAHRT:  gc.set(year, Calendar.AUGUST, 15);
                                      break;

            // SEPTEMBER
            case usLABORDAY:          gc.setNthDow(
                                        ExtendedGregorianCalendar.FIRST,
                                        ExtendedGregorianCalendar.MONDAY,
                                        ExtendedGregorianCalendar.SEPTEMBER,
                                        year);
                                        break;

            case chEIDGENOESSISCHER:  if (year >= 1832) {
                                      gc.setNthDow(
                                        ExtendedGregorianCalendar.THIRD,
                                        ExtendedGregorianCalendar.SUNDAY,
                                        ExtendedGregorianCalendar.SEPTEMBER,
                                        year);
                                        break;
                                      } else
                                      return "nicht definiert";

            // OCTOBER
            case deTAGDERDTEINHEIT:   gc.set(year, Calendar.OCTOBER, 3);
                                      break;

            case deERNTEDANKFEST:     gc.setNthDow(
                                        ExtendedGregorianCalendar.FIRST,
                                        ExtendedGregorianCalendar.SUNDAY,
                                        ExtendedGregorianCalendar.OCTOBER,
                                        year);
                                      break;

            case usCOLUMBUSDAY:       gc.setNthDow(
                                        ExtendedGregorianCalendar.SECOND,
                                        ExtendedGregorianCalendar.MONDAY,
                                        ExtendedGregorianCalendar.OCTOBER,
                                        year);
                                      break;

            case deWINTERZEIT:        try {
                                          gc.setDSTend(year, TimeZone.getTimeZone("Europe/Berlin"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case chWINTERZEIT:        try {
                                          gc.setDSTend(year, TimeZone.getTimeZone("Europe/Zurich"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case usDAYLIGHTEND:       try {
                                          // Zeitzone New York stellvertretend fr die gesamte USA
                                          gc.setDSTend(year, TimeZone.getTimeZone("America/New_York"));
                                      } catch (Exception e){
                                          return NOT_DEFINED;
                                      }
                                      break;

            case deREFORMATIONSTAG:
            case usHALLOWEEN:         gc.set(year, Calendar.OCTOBER, 31);
                                      break;

            // NOVEMBER
            case deALLERHEILIGEN:
            case chALLERHEILIGEN:     gc.set(year,Calendar.NOVEMBER, 1);
                                      break;

            case deALLERSEELEN:       gc.set(year,Calendar.NOVEMBER, 2);
                                      break;



            case usELECTIONDAY:       gc.setNthDow( // http://www.fec.gov/pages/faqvdayeprocedures.htm
                                        ExtendedGregorianCalendar.FIRST,
                                        ExtendedGregorianCalendar.TUESDAY,
                                        ExtendedGregorianCalendar.NOVEMBER,
                                        year);
                                      break;

            case deMARTINSTAG:
            case usVETERANSDAY:       gc.set(year,Calendar.NOVEMBER, 11);
                                      break;

            case deVOLKSTRAUERTAG:    if (year >= 1952) {
                                        gc.setAdvent(year);
                                        gc.add(Calendar.DATE, -14);
                                        break;
                                      } else return "nicht definiert";

            case deBUSSUNDBETTAG:     gc.setAdvent(year);
                                      gc.add(Calendar.DATE, -11);
                                      break;

            case deTOTENSONNTAG:      if (year >= 1816) {
                                        gc.setAdvent(year);
                                        gc.add(Calendar.DATE, -7);
                                        break;
                                      } else
                                      return "nicht definiert";
            case chTOTENSONNTAG:      gc.setAdvent(year);
                                      gc.add(Calendar.DATE, -7);
                                      break;

            case deNIKOLAUS:          gc.set(year, Calendar.DECEMBER, 6);
                                      break;

            case deMARIAEEMFAENGNIS:  gc.set(year, Calendar.DECEMBER, 8);
                                      break;

            case deADVENT1:
            case chADVENT1:           gc.setAdvent(year);
                                      break;

            case deADVENT2:
            case chADVENT2:           gc.setAdvent(year);
                                      gc.add(Calendar.DATE, +7);
                                      break;

            case deADVENT3:
            case chADVENT3:           gc.setAdvent(year);
                                      gc.add(Calendar.DATE, +14);
                                      break;

            case deADVENT4:
            case chADVENT4:           gc.setAdvent(year);
                                      gc.add(Calendar.DATE, +21);
                                      break;

            case usTHANKSGIVINGDAY:   gc.setNthDow(
                                        ExtendedGregorianCalendar.FOURTH,
                                        ExtendedGregorianCalendar.THURSDAY,
                                        ExtendedGregorianCalendar.NOVEMBER,
                                        year);
                                      break;

            case deHEILIGABEND:
            case chHEILIGABEND:
            case usCHRISTMASEVE:      gc.set(year, Calendar.DECEMBER, 24);
                                      break;

            case deWEIHNACHTEN1:
            case chWEIHNACHTEN1:
            case usCHRISTMASDAY:      gc.set(year, Calendar.DECEMBER, 25);
                                      break;

            case deWEIHNACHTEN2:      gc.set(year, Calendar.DECEMBER, 26);
                                      break;

            case deSILVESTER:
            case chSILVESTER:
            case usNEWYEARSEVE:       gc.set(year, Calendar.DECEMBER, 31);
                                      break;

        }
        return df.format(gc.getTime());
    }


    public String getInitialValue() {
       // return the current year
       SimpleDateFormat formatter = new SimpleDateFormat("yyyy");
       return formatter.format(new Date());
    }

    public String getTransferValue(int s) throws Exception {
        return input;
    }

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception {
        int value = Integer.parseInt(input);
        int add = Integer.parseInt(plus.toString());
        value+=add;
        setInput(Integer.toString(value));
        return getInput();
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
        setInput(big.toString());
        return getInput();
    }

}
