/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import java.util.*;
import java.math.*;
import java.io.*;
import jonelo.NumericalChameleon.*;
import jonelo.sugar.math.*;
import jonelo.sugar.util.*;

public class ClusterGeneric extends ClusterObject {

    public static String INVALID="source is invalid";
    private Vector v;
    private boolean logic;

    /** Creates new ClusterGeneric */
    public ClusterGeneric(String name, boolean logic) {
        this.logic = logic;
        v = readTextFromJar("/data/units/"+name+".list");
    }

    public Vector getSourceEntries()  {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public boolean isSpecialDefaultValue() {
        return false;
    }

    public boolean isScientificSupported() {
        return true;
    }

    public String getOutput(int s, int t) throws Exception {
        BigDecimal x=null;

        try {
            x = new BigDecimal(input);
        } catch (Exception e) { throw new Exception(INVALID); }

        if (s!=t) {
            if (logic) {
                // anything to target
                x = x.multiply( ((NcRecord)v.elementAt(s)).getFactor() );
                x = x.divide( ((NcRecord)v.elementAt(t)).getFactor(),getPrecision()+2,BigDecimal.ROUND_HALF_UP);
            } else {
                // target to anything
                x = x.divide(((NcRecord)v.elementAt(s)).getFactor(),getPrecision()+2,BigDecimal.ROUND_HALF_UP);
                x = x.multiply(((NcRecord)v.elementAt(t)).getFactor());
            }
        }
        
       if (getScientific()) {
            if (Main.supportForJava5)
            return GeneralMath.decimal2Scientific(x.toPlainString(),getPrecision()); else
            return GeneralMath.decimal2Scientific(x.toString(),getPrecision());
        } else {
            if (Main.supportForJava5)
            return x.setScale(getPrecision(),BigDecimal.ROUND_HALF_UP).toPlainString(); else
            return x.setScale(getPrecision(),BigDecimal.ROUND_HALF_UP).toString();
        }     
    }



    private final Vector readTextFromJar(String filename) {
        Vector v = null;
        try {
            InputStream is = getClass().getResourceAsStream(filename);
            BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
            BigDecimal bigdecimal = null;
            BigDecimal fraction1 = null, fraction2 = null;
            StringTokenizer st = null;
            String description = null;
            v = new Vector();
            String thisLine;
            int i=0;
            while ((thisLine = br.readLine()) != null) {
                if (!thisLine.startsWith("#") && !thisLine.equals("")) {

                    st = new StringTokenizer(thisLine,":");
                    description = GeneralString.decodeEncodedUnicode(st.nextToken());

                    String strzahl = st.nextToken();

                    int pos = strzahl.indexOf("/");
                    if (pos == -1) // no fraction
                    {
                        bigdecimal = new BigDecimal(strzahl);
                    } else {
                        fraction1 = new BigDecimal(strzahl.substring(0,pos));
                        fraction2 = new BigDecimal(strzahl.substring(pos+1,strzahl.length()));
                        bigdecimal = fraction1.divide( fraction2, 1000, BigDecimal.ROUND_HALF_UP);
                    }
                    String flag = null;
                    if (st.hasMoreTokens()) flag = st.nextToken().toLowerCase();
                    NcRecord ncr = new NcRecord(description, bigdecimal, flag);
                    String temp="";
                    if (st.hasMoreTokens()) {
                        temp = st.nextToken();
                        if (temp.equals("false")) ncr.setActive(false);

                        if (st.hasMoreTokens()) {
                            temp= st.nextToken();
                            if (temp.equals("source")) setSourceDefault(i);
                            if (temp.equals("target")) setTargetDefault(i);
                            if (temp.equals("source,target") || temp.equals("target,source")) {
                                setSourceDefault(i);
                                setTargetDefault(i);
                            }
                        }
                    }
                    v.add(ncr);
                    i++;
                }
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return v;
    }

    public String addValue(BigDecimal plus, int sourceindex) throws Exception {
        if ((input==null) || (input.equals(""))) input="0";
        BigDecimal big = null;
        try {
            big = new BigDecimal(getInput());
        } catch (Exception e) {
            throw new Exception(INVALID);
        }
        big=big.add(plus);
        return big.toString();
    }

    public String getTransferValue(int s) throws Exception {
        return getOutput(s,s);
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
        setInput(big.toString());
        return input;
    }

}
