/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import java.math.*;
import jonelo.sugar.math.*;
import java.util.*;
import jonelo.NumericalChameleon.*;
public class ClusterTemperature extends ClusterObject {

    public final static int INTERNAL=5;
    public final static String CODE="TEMPERATURE";

    private final static int
      CKELVIN = 0,
      CCELSIUS = 1,
      CFAHRENHEIT = 2,
      CRANKINE = 3,
      CREAUMUR = 4;

    public static String
      KELVIN = "Kelvin (K)",
      CENTIGRADE = "Centigrade (C)",
      FAHRENHEIT = "Fahrenheit (F)",
      RANKINE = "Rankine (R)",
      REAUMUR = "Raumur (Re)",
      INVALID = "Source is invalid";

   public final static int
     PRECISION=5; // additional precision for internal

   private Vector v;
   private final static
      BigDecimal _5 = new BigDecimal("5"),
                 _4 = new BigDecimal("4"),
                 _9 = new BigDecimal("9"),
                _32 = new BigDecimal("32"),
               _273 = new BigDecimal("273.15");

    /** Creates new ClusterTemperature */
    public ClusterTemperature() {
        v=new Vector();
        v.add(new NcRecord(KELVIN,"siu"));
        v.add(new NcRecord(CENTIGRADE,"int"));
        v.add(new NcRecord(FAHRENHEIT,"eng"));
        v.add(new NcRecord(RANKINE,"eng"));
        v.add(new NcRecord(REAUMUR,"fr",false));
    }

    public Vector getSourceEntries()  {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public int getSourceDefault() {
        return CKELVIN;
    }

    public int getTargetDefault() {
        return CCELSIUS;
    }

    // defaultValue is transferValue
    // is there any special transverValue ?
    public boolean isSpecialDefaultValue() {
        return false;
    }

    public boolean isScientificSupported() {
        return true;
    }

    public int getPreferredPrecision()  {
        return 12;
    }

    /** get Output
     * @param s soure
     * @param t tareget
     * @throws Exception if there is a problem
     * @return String
     */
    public String getOutput(int s, int t) throws Exception
    {
        BigDecimal x=new BigDecimal("0");

        try {
          x = new BigDecimal(input);
        } catch (Exception e) { throw new Exception(INVALID); }

        if (s==t) return x.setScale(getPrecision(),BigDecimal.ROUND_HALF_UP).toString();

        switch (s) // anything to kelvin
        {
            case CKELVIN: break;
            case CCELSIUS: x = x.add(_273); break;
            case CFAHRENHEIT: x = (_5.divide(_9,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP )).multiply
                                 (x.subtract(_32)) .add(_273); break;
            case CREAUMUR: x = (_5.divide(_4,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP)).multiply(x).add(_273); break;
            case CRANKINE: x = (_5.divide(_9,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP)).multiply(x); break;
        }

        switch (t) // kelvin to anything
        {
            case CKELVIN: break;
            case CCELSIUS: x = x.subtract(_273); break;
            case CFAHRENHEIT: x = (_9.divide(_5,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP)).multiply (x.subtract(_273)).add (_32); break;
            case CREAUMUR: x = (_4.divide(_5,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP)).multiply (x.subtract(_273)); break;
            case CRANKINE: x = (_9.divide(_5,getPrecision()+PRECISION,BigDecimal.ROUND_HALF_UP)).multiply(x); break;
        }

       if (getScientific()) {
            if (Main.supportForJava5)
            return GeneralMath.decimal2Scientific(x.toPlainString(),getPrecision()); else
            return GeneralMath.decimal2Scientific(x.toString(),getPrecision());
        } else {
            if (Main.supportForJava5)
            return x.setScale(getPrecision(),BigDecimal.ROUND_HALF_UP).toPlainString(); else
            return x.setScale(getPrecision(),BigDecimal.ROUND_HALF_UP).toString();
        }     
    }


    public String addValue(BigDecimal plus, int sourceindex) throws Exception {
        if ((input==null) || (input.equals(""))) input="0";
        BigDecimal big = new BigDecimal(getInput());
        big=big.add(plus);
        return big.toString();
    }

    public String getTransferValue(int s) throws Exception {
        return getOutput(s,s);
    }

    public String setValue(BigDecimal big, int sourceindex) throws Exception {
        setInput(big.toString());
        return input;
    }

}

