/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.math.*;
import java.awt.datatransfer.*;
import java.io.*;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;
import javax.swing.plaf.metal.DefaultMetalTheme;
import com.jgoodies.plaf.plastic.*;
import com.jgoodies.plaf.plastic.theme.*;
import jonelo.sugar.gui.SplashScreen;
import jonelo.sugar.gui.AboutDialog;
import jonelo.sugar.util.Counter;
import jonelo.sugar.util.GeneralProgram;
import jonelo.sugar.gui.GeneralGUI;
import com.Ostermiller.util.Browser;
import jonelo.sugar.util.GeneralString;
import jonelo.sugar.gui.PropertiesMetalTheme;
import javax.help.*;
import jonelo.NumericalChameleon.ExchangeRates.*;
import jonelo.NumericalChameleon.Clusters.*;
import jonelo.sugar.io.GeneralIO;
import jonelo.sugar.args.*;
import jonelo.sugar.io.ExampleFileFilter;
import jonelo.sugar.io.TeeStream;
import jonelo.sugar.util.ExtendedGregorianCalendar;
import jonelo.NumericalChameleon.RomanNumber.*;
import jonelo.NumericalChameleon.SpokenNumbers.SpokenNumber;
import net.sourceforge.mlf.metouia.*;
import jonelo.NumericalChameleon.MultipleNotice.*;
import jonelo.NumericalChameleon.PropsDialog.*;
import jonelo.NumericalChameleon.CalendarChooser.*;
import jonelo.NumericalChameleon.PhoneticAlphabetDialog.*;
import jonelo.NumericalChameleon.Find.*;
import java.text.*;
import jonelo.NumericalChameleon.Bookmarks.*;
import javax.swing.undo.*;
import javax.swing.text.*;
import javax.swing.event.*;

public class Main extends javax.swing.JFrame {

    private static String
    PROGRAMNAME="NumericalChameleon",
    VERSION = jonelo.NumericalChameleon.Version.number,
    PATCHLEVEL = jonelo.NumericalChameleon.Version.patchlevel,
    PROGRAM = PROGRAMNAME+" "+VERSION;

    private final static
    Color LILA = new Color(0x9999CC),
    GREEN = new Color(153,255,153);
    private final static String
    ERROR_GUI_PRECISION = "Error.GUI.Precision",
    ERROR_GUI_LOOKANDFEEL = "Error.GUI.LookAndFeel",
    ERROR_DIRNOTFOUND = "Error.DirNotFound",
    ERROR_NOTADIR = "Error.NotADirectory",
    ERROR_READDIR = "Error.CanNotReadDirectory",

    MESSAGE_JAVACHECK = "Checking Java VM version ... ",
    MESSAGE_GUI_PROMPT = "Message.GUI.Prompt",
    MESSAGE_OK = "OK.";

    public final static String
    IMAGE_DIR = "/data/images",
    MAC_OS_X = "Mac OS X";

    private static NcRecordCellRenderer ncRecordCellRenderer = null;
    private static SectionRecordCellRenderer sectionRecordCellRenderer = null;

    private HelpSet mainHS = null, tableHS = null;
    private HelpBroker mainHB = null, tableHB = null;

    private final static String
    DIR_USER_PROPERTIES = ".NumericalChameleon",
    FILE_USER_PROPERTIES = DIR_USER_PROPERTIES+"/.nc.properties",
    FILE_SYSTEM_PROPERTIES = ".nc.properties",
    FILE_BOOKMARKS = DIR_USER_PROPERTIES+"/.nc.bookmarks";

    private ClusterObject clusterObject;
    private String transferValue;
    private boolean precisionerror;
    private Vector groupAllCategories;
    private ResourceBundle rb;
    public static ResourceBundle iso3166;

    private com.incors.plaf.kunststoff.KunststoffLookAndFeel klaf = null;
    private net.sourceforge.mlf.metouia.MetouiaLookAndFeel mlnf = null;
    private net.beeger.squareness.SquarenessLookAndFeel slnf = null;
    private com.jgoodies.plaf.plastic.PlasticXPLookAndFeel plnf = null;
    private DefaultMetalTheme defaultMetalTheme = null;
    private MetalTheme oceanTheme = null;

    private Vector themesVector = null, groupsVector=null;
    private javax.swing.JScrollPane themesScrollPane, groupsScrollPane;
    private javax.swing.JScrollPane kunststoffThemesScrollPane;
    private javax.swing.JScrollPane metouiaThemesScrollPane;
    private javax.swing.JScrollPane bookmarkScrollPane;

    private javax.swing.JList themesList, groupsList;
    private javax.swing.JList kunststoffThemesList;
    private javax.swing.JList metouiaThemesList;
    private javax.swing.JList bookmarkList;

    private Properties systemProps;
    private Properties userProps;

    private boolean first=true;
    private boolean debug=false;
    private boolean logfile=false;
    private boolean jnlp=false;

    //private int exchange_rates_pos=0, color_pos=0, calendar_pos=0;
    private String exchange_rates_description=null;
    private boolean helpGenerated=false, tablesGenerated=false;
    private int view;

    private final static BigDecimal MINUS_ONE=new BigDecimal("-1");
    private final static BigDecimal PLUS_ONE=new BigDecimal("1");

    private MultipleNoticeCluster multipleNoticeCluster=null;
    private final static String DEFAULT_NOTICE_FORM = "$SVALUE [$SUNIT] = $TVALUE [$TUNIT]";
    private String notice_form = DEFAULT_NOTICE_FORM;
    private JFileChooser fc_save, fc_load;
    private ExampleFileFilter utf16be, utf16le, utf16, utf8, html, iso8859, vcs;

    // must be public
    public static Hashtable groupsHashtable, categoriesHashtable;
    private Vector groupCurrentSelection;

    private int lastTypeBoxSelectedIndex = -1;
    private Bookmarks bookmarks=null;

    private final UndoManager historyUndo = new UndoManager();
    private final UndoManager sourceUndo = new UndoManager();

    private String HTML_HEADER=
    "<html>\n<head>\n  <meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">\n  <title>Generated by NumericalChameleon "+jonelo.NumericalChameleon.Version.number+"</title>\n  <meta name=\"author\" content=\"NumericalChameleon "+jonelo.NumericalChameleon.Version.number+"\">\n</head>\n<body>\n\n<table BORDER CELLSPACING=0 CELLPADDING=3>\n",
    HTML_FOOTER=
    "</table>\n\n</body>\n</html>";

    DefaultComboBoxModel categoryModel;
    private static boolean isOrientationRightToLeft=false;
    public static boolean supportForJava5=false;


    // MAIN & CONSTRUCTOR
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * the main routine for Numerical Chameleon
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new Main(args);
    }

    // Constructor
    public Main(String[] args) {
        javaCheck();

        // params
        Arguments arguments = new Arguments(args, "-rates|-filter");
        if (arguments.listArguments().size() >= 1) {
            String arg_rates=null;
            String arg_filter=null;
            if (arguments.receivedArgument("-verbose")) {
                debug=true;
                CurrencyConverter.setDebug(debug);
                ExchangeRatesUpdateDialog.setDebug(debug);
            }

            if (arguments.receivedArgument("-logfile")) {
                logfile=true;
            }

            if (arguments.receivedArgument("-reset")) {
               // let's remove the user properties
               System.out.print("Removing user properties ... ");
               File f = new File(System.getProperty("user.home")+File.separator+FILE_USER_PROPERTIES);

               try {
                 if (f.exists()) f.delete();
                 System.out.println("OK.");
               } catch (Exception e) {
                  System.err.println(e);
               }
            }

            if (arguments.receivedArgument("-jnlp_no_permissions")) {
                // we detect jnlp (Java Webstart)
                debug=false;
                jnlp=true;
            }

            if (arguments.receivedArgument("-rates") &&
                arguments.receivedArgument("-filter")
            ) {
                userProps = new Properties();
                iso3166 = ResourceBundle.getBundle("data.lang.iso3166");
                try {
                    arg_rates = arguments.getStringValueOf("-rates");
                    arg_filter = arguments.getStringValueOf("-filter");
                    loadSystemProperties();
                    JTextArea textArea = new JTextArea();
                    Counter counter = new Counter();
                    ExchangeRates exchangeRates = new ExchangeRates();
                    CurrencyConverter cc = new CurrencyConverter("/data/rates/"+arg_filter+".filter", arg_rates, counter, textArea, exchangeRates);

                    if (exchangeRates.success()) {
                        try {
                            exchangeRates.writeToDisk();
                            systemProps.setProperty("rates_date",exchangeRates.getDate());
                            systemProps.setProperty("rates_logic",exchangeRates.getLogic() ? "true":"false");
                            saveSystemProperties();
                        } catch (Exception e){ System.err.println(e); }
                    }
                    if (debug) System.out.println("update finished.");
                    System.exit(0);
                } catch (Exception e) {
                    System.err.println(e);
                    System.exit(0);
                }
            } else
                if (arguments.receivedArgument("-help") ||
                arguments.receivedArgument("-h")) {
                    System.out.println("Version:    "+PROGRAM);
                    System.out.println("Patchlevel: "+PATCHLEVEL);
                    System.out.println("Launch:     java -jar nc.jar");
                    System.out.println("Options:    [-rates <exchange rates> -filter <exchange rate filter>]");
                    System.out.println("            [-h|-help]");
                    System.out.println("            [-verbose]");
                    System.out.println("            [-logfile]");
                    System.out.println("            [-reset]");
                    //System.out.println("            [-jnlp_no_permissions]");  // for Java Web Start
                    System.exit(0);
                }
        }
        // start the GUI
        if (GeneralProgram.isSupportFor("1.5")) {
           try {
             oceanTheme = (MetalTheme)sun.awt.AppContext.getAppContext().get("currentMetalTheme");
           } catch (Exception e) {
             System.err.println(e);
           }
        }
        init(true);
    }


    private void init(boolean splash) {
        if (logfile) {
          try {
            // Tee standard output
            PrintStream out = new PrintStream(new FileOutputStream("nc"+jonelo.NumericalChameleon.Version.number+"_out.log"));
            PrintStream tee = new TeeStream(System.out, out);
            System.setOut(tee);

            // Tee standard error
            PrintStream err = new PrintStream(new FileOutputStream("nc"+jonelo.NumericalChameleon.Version.number+"_err.log"));
            tee = new TeeStream(System.err, err);
            System.setErr(tee);
          } catch (FileNotFoundException e) {
          }
        }

        if (debug) {
          System.out.println("application.name="+PROGRAMNAME);
          System.out.println("application.version="+VERSION);
          System.out.println("application.patch.level="+PATCHLEVEL);
          Properties debugProperties = System.getProperties();

          // this statement truncates long lines, so let's code it manually
          // debugProperties.list(System.out);
          for (Enumeration e = debugProperties.keys(); e.hasMoreElements() ;) {
            String o = (String)e.nextElement();
            System.out.println(o+"="+debugProperties.getProperty(o));
          }

          System.out.println("system.lnf="+UIManager.getSystemLookAndFeelClassName());
          System.out.println("OK.");
        }

        // Mac OS X support
        // http://developer.apple.com/documentation/Java/Conceptual/Java141Development/Virtual_Machine/index.html
        // if (System.getProperty("mrj.version") != null)
        if (System.getProperty("os.name").equals(MAC_OS_X))
        {
          // for 1.4
          System.setProperty("com.apple.mrj.application.apple.menu.about.name","NumericalChameleon");
          System.setProperty("apple.laf.useScreenMenuBar","true");
          System.setProperty("apple.awt.showGrowBox","true");
          // System.setProperty("apple.awt.brushMetalLook","true");
          // for 1.3
          System.setProperty("com.apple.macos.useScreenMenuBar","true");
          System.setProperty("com.apple.mrj.application.growbox.intrudes","false");
          System.setProperty("com.apple.mrj.application.live-resize","true");
        }

        lastTypeBoxSelectedIndex=-1;
        tablesGenerated=false;
        SplashScreen splashScreen = null;
        if (!splash) {
            setVisible(false);
            remove(converterPanel);
            // The application has been restarted, so it's ok to request the GC at this time!
            System.gc();
        }

        if (splash)
            splashScreen = new SplashScreen(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/logo.jpg")));
        else splashScreen=null;

        if (splash) splashScreen.showStatus("Loading system properties ...");
        loadSystemProperties();

        if (splash) splashScreen.showStatus("Loading user properties ...");
        loadUserProperties();

        ComponentOrientation ce = ComponentOrientation.getOrientation(Locale.getDefault());
        isOrientationRightToLeft=!ce.isLeftToRight();

        // tooltip user property is set
        if (!userProps.getProperty("tooltips","").equals("")) {
            // disable tooltips only if property set to "false"
            setToolTips(!userProps.getProperty("tooltips","false").equals("false"));
        }
        if (splash) splashScreen.showStatus("Loading language resource bundle ...");
        loadResourceBundle();
        iso3166 = ResourceBundle.getBundle("data.lang.iso3166");

        // supported since Java 1.4
        if (splash && GeneralProgram.isSupportFor("1.4") &&
            // but only Java based Look and Feels
            ( userProps.getProperty("lnf","").equals("kunststoff") ||
              userProps.getProperty("lnf","").equals("metal") ||
              userProps.getProperty("lnf","").equals("metal_ocean") ||
              userProps.getProperty("lnf","").equals("plastic") ||
              userProps.getProperty("lnf","").equals("metal_default") ||
              userProps.getProperty("lnf","").equals("metouia") ||
              userProps.getProperty("lnf","").equals("squareness")
            )
          ) {
            String decoration = userProps.getProperty("decoration","native");
            if (decoration.equals("java")) {
                setUndecorated(true);
                getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            }
        }

        if (splash)
            splashScreen.showStatus(rb.getString("SplashScreen.InitThemes"));
        initThemes();

        if (splash) splashScreen.showStatus(rb.getString("SplashScreen.CreateGUI"));
        initComponents();



        Browser.init();
        categoryModel = new DefaultComboBoxModel();
        setRenderers();

        setUndoRedoFor(historyTextArea, historyUndo);
        setUndoRedoFor(sourceTextArea, sourceUndo);

        //if (multipleNoticeCluster==null)
            multipleNoticeCluster=new MultipleNoticeCluster();

        if (splash) splashScreen.showStatus(rb.getString("SplashScreen.LoadingFavorites"));
        loadBookmarks();

        if (splash) {
          addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
          });
        }

        if (jnlp) {
            // do some moifications
            // disable "update exchange rates"
            updateExchangeRatesMenuItem.setEnabled(false);
            saveButton.setEnabled(false);
            loadMenuItem.setEnabled(false);
            saveMenuItem.setEnabled(false);
            historyMenuItemLoad.setEnabled(false);
            historyMenuItemSave.setEnabled(false);
        }

        // Look and Feels
        lafButtonGroup.add(aquaRadioButtonMenuItem);
        lafButtonGroup.add(gtkRadioButtonMenuItem);
        lafButtonGroup.add(oceanRadioButtonMenuItem);
        lafButtonGroup.add(swingRadioButtonMenuItem);
        lafButtonGroup.add(motifRadioButtonMenuItem);
        lafButtonGroup.add(windowsRadioButtonMenuItem);
        lafButtonGroup.add(macRadioButtonMenuItem);
        lafButtonGroup.add(systemRadioButtonMenuItem);

        // Look and Feels (enable and disable)
        motifRadioButtonMenuItem.setEnabled(GeneralGUI.isLookAndFeelAvailable(GeneralGUI.LAF_MOTIF));
        windowsRadioButtonMenuItem.setEnabled(GeneralGUI.isLookAndFeelAvailable(GeneralGUI.LAF_WINDOWS));
        macRadioButtonMenuItem.setEnabled(GeneralGUI.isLookAndFeelAvailable(GeneralGUI.LAF_MAC));
        aquaRadioButtonMenuItem.setEnabled(GeneralGUI.isLookAndFeelAvailable(GeneralGUI.LAF_AQUA));
        gtkRadioButtonMenuItem.setEnabled(GeneralGUI.isLookAndFeelAvailable(GeneralGUI.LAF_GTK));
        plasticMenuItem.setEnabled(GeneralProgram.isSupportFor("1.4"));
        squarenessMenuItem.setEnabled(GeneralProgram.isSupportFor("1.4"));
        oceanRadioButtonMenuItem.setEnabled(GeneralProgram.isSupportFor("1.5"));

        if (userProps.getProperty("lnf","").equals("motif")) motifRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("windows")) windowsRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("mac")) macRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("aqua")) aquaRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("gtk")) gtkRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("metal_ocean")) oceanRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("lnf","").equals("system")) systemRadioButtonMenuItem.setSelected(true);

        // Internationalization
        langButtonGroup.add(enRadioButtonMenuItem);
        langButtonGroup.add(usRadioButtonMenuItem);
        langButtonGroup.add(deRadioButtonMenuItem);
        langButtonGroup.add(frRadioButtonMenuItem);
        langButtonGroup.add(ptRadioButtonMenuItem);
        langButtonGroup.add(esRadioButtonMenuItem);
        langButtonGroup.add(elRadioButtonMenuItem);
        langButtonGroup.add(heRadioButtonMenuItem);

        if (userProps.getProperty("language").equals("en")&& userProps.getProperty("country").equals("GB")) enRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("en")) usRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("de")) deRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("fr")) frRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("pt")) ptRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("es")) esRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("el")) elRadioButtonMenuItem.setSelected(true); else
        if (userProps.getProperty("language").equals("he")) heRadioButtonMenuItem.setSelected(true); else
           enRadioButtonMenuItem.setSelected(true);

        Font font = historyTextArea.getFont();
        try {
            historyTextArea.setFont(font.deriveFont( Float.valueOf((String)userProps.get("fontsize")).floatValue()) );
        } catch (Exception e) { System.err.println(e.toString()); }

        if (splash) splashScreen.showStatus(rb.getString("SplashScreen.SettingView"));
        frameButtonGroup.add(luxuryRadioButtonMenuItem);
        frameButtonGroup.add(standardRadioButtonMenuItem);
        frameButtonGroup.add(economicalRadioButtonMenuItem);
        frameButtonGroup.add(miserlyRadioButtonMenuItem);
        frameButtonGroup.add(maximumRadioButtonMenuItem);
        view=0;
        try {
            view = Integer.valueOf((String)userProps.get("view")).intValue();
            if ((view < -1) || (view > 3)) view=0;
        } catch (Exception e) { System.err.println(e.toString()); }

        switch (view) {
            case -1: maximumRadioButtonMenuItem.setSelected(true); break;
            case 0: luxuryRadioButtonMenuItem.setSelected(true); break;
            case 1: standardRadioButtonMenuItem.setSelected(true); break;
            case 2: economicalRadioButtonMenuItem.setSelected(true); break;
            case 3: miserlyRadioButtonMenuItem.setSelected(true); break;
        }

        // context sensitive help
        // CSH.setHelpIDString(categoryBox,"categories");

        if (splash)
            splashScreen.showStatus(rb.getString("SplashScreen.PrepareThemesList"));
        prepareThemesList();

        if (splash)
            splashScreen.showStatus(rb.getString("SplashScreen.PrepareGroupsList"));
        groupsHashtable=new Hashtable();
        groupCurrentSelection=new Vector();
        prepareGroupsList();

        if (splash)
            splashScreen.showStatus(rb.getString("SplashScreen.RegisterCategories"));
        register();

        if (splash)
            splashScreen.showStatus(rb.getString("SplashScreen.Starting"));

        first=false;
        luxuryFrame(view);
        try {
            setLocation(Integer.parseInt(userProps.getProperty("locationX")),
                        Integer.parseInt(userProps.getProperty("locationY")));
            if (splash)
            {
            setSize(Integer.parseInt(userProps.getProperty("width")),
                    Integer.parseInt(userProps.getProperty("heigth")));
            }

        } catch (Exception e) {
            GeneralGUI.centerComponent(this);
        }

        if (splash) {
            splashScreen.close();
            splashScreen=null;
        }

        if (isOrientationRightToLeft()) {
          GeneralGUI.applyOrientation(ncMenuBar, ce); //ComponentOrientation.RIGHT_TO_LEFT);
          GeneralGUI.applyOrientation(converterPanel, ce);
          GeneralGUI.applyOrientation(sourcePopupMenu, ce);
          GeneralGUI.applyOrientation(targetPopupMenu, ce);
          GeneralGUI.applyOrientation(historyPopupMenu, ce);
        }

        repaint(); // repaint problem on Solaris
        show();
        toFront();
    }

    public static boolean isOrientationRightToLeft() {
        return isOrientationRightToLeft;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // METHODS
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    private void setUndoRedoFor(JTextArea textArea, final UndoManager manager) {
        Document doc = textArea.getDocument();
        doc.addUndoableEditListener(new UndoableEditListener() {
            public void undoableEditHappened(UndoableEditEvent evt) {
                manager.addEdit(evt.getEdit());
            }
        });
        textArea.getActionMap().put("Undo",
        new AbstractAction("Undo") {
            public void actionPerformed(ActionEvent evt) {
                undo(manager);
            }
        });
        textArea.getInputMap().put(KeyStroke.getKeyStroke("control Z"), "Undo");

        textArea.getActionMap().put("Redo",
        new AbstractAction("Redo") {
            public void actionPerformed(ActionEvent evt) {
                redo(manager);
            }
        });
        textArea.getInputMap().put(KeyStroke.getKeyStroke("control Y"), "Redo");
    }

    private void undo(final UndoManager manager) {
        try {
           if (manager.canUndo()) {
               manager.undo();
           }
        } catch (CannotUndoException e) {
        }
    }

    private void redo(final UndoManager manager) {
        try {
            if (manager.canRedo()) {
               manager.redo();
            }
        } catch (CannotRedoException e) {
        }
    }

    private void loadBookmarks() {
        // load Bookmarks

        bookmarks = new Bookmarks(System.getProperty("user.home")+File.separator+FILE_BOOKMARKS);
        try {
            bookmarks.read();
        } catch (IOException ioe) {
            System.err.println(ioe);
        }
        bookmarkList = new JList(bookmarks);


        if (bookmarks.size() < 10)
            bookmarkList.setVisibleRowCount(bookmarks.size()); else
                bookmarkList.setVisibleRowCount(10);


        bookmarkScrollPane = new JScrollPane();
        bookmarkList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
                int index = bookmarkList.locationToIndex(me.getPoint());
                bookmarkListValueChanged(index);
            }
        });
        bookmarkScrollPane.setViewportView(bookmarkList);

        favoritesMenu.add(bookmarkScrollPane);
        // end-bookmarks
    }

    private void setToolTips(boolean status) {
        ToolTipManager.sharedInstance().setEnabled(status);
    }

    // checks if the Java VM is sufficient
    private void javaCheck() {
        System.out.print(MESSAGE_JAVACHECK);
        GeneralProgram.requiresMinimumJavaVersion("1.3");
        supportForJava5 = GeneralProgram.isSupportFor("1.5");
        System.out.println(MESSAGE_OK);
    }


    // visit the bookmark
    private void bookmarkListValueChanged(int index) {

        Bookmark bookmark = (Bookmark)bookmarks.get(index);
        // is code not in the whiteList ?
        if (!groupsHashtable.containsKey(bookmark.getCategory()))
            // reset the filter
            clickOnFilterCategoryIndicator();

        // search for the category
        int found = findCategoryWithCode(bookmark.getCategory());
        if (found == -1) {
            // not found!
            JOptionPane.showMessageDialog(this,rb.getString("Message.CategoryNotFound"), rb.getString("Message.CategoryNotFound"), JOptionPane.ERROR_MESSAGE);
        } else {
            favoritesMenu.setPopupMenuVisible(false);
            navigationMenu.setPopupMenuVisible(false);
            SwingUtilities.updateComponentTreeUI(getRootPane());

            // it's not the current Code
            if (categoryBox.getSelectedIndex()!=found) {
                categoryBox.setSelectedIndex(found);
                categoryBoxItemStateChanged();
            }

            found = findUnitExact(sourceBox,bookmark.getSourceDescription(),0,true); // country bercksichtigen!!!
            if (found > -1) sourceBox.setSelectedIndex(found);

            found = findUnitExact(targetBox,bookmark.getTargetDescription(),0,true);
            if (found > -1) targetBox.setSelectedIndex(found);

            sourceTextArea.setText(bookmark.getValue());
            process();
        }
    }

    private void loadSystemProperties() {
        if (debug) System.out.print("Loading system properties ... ");
        systemProps = new Properties();

        systemProps.setProperty("rates_date","not defined");
        systemProps.setProperty("rates_logic","false");
        systemProps.setProperty("rates_preferred","oanda.com.filter");

        try{
            systemProps.load(getClass().getResourceAsStream("/data/"+FILE_SYSTEM_PROPERTIES));
            if (debug) System.out.print("data"+File.separator+FILE_SYSTEM_PROPERTIES);
            if (debug) System.out.println(" OK.");
        }  catch (Exception e) {
            System.err.println(e);
        }
    }

    private void loadUserProperties() {
        if (debug) System.out.print("Loading user properties ... ");
        userProps = new Properties();

        // set defaults
        Locale defaultLocale = Locale.getDefault();

        String deflang = defaultLocale.getLanguage(),
        defcountry = null,
        defhelp = null;

        // supported languages
        if (!( deflang.equals("en") || deflang.equals("de") || deflang.equals("fr") ||
        deflang.equals("pt") || deflang.equals("es") || deflang.equals("iw") || deflang.equals("el") )) {
            deflang="en";
            defcountry="US";
        } else {
            defcountry = defaultLocale.getCountry();
        }

        // supported help files
        if (!( deflang.equals("en") || deflang.equals("de") )) {
            defhelp="en";
        } else {
            defhelp = deflang;
        }

        // set default properties
        userProps.setProperty("view","0");
        userProps.setProperty("language",deflang);
        //userProps.setProperty("iso3166",deflang);
        userProps.setProperty("help",deflang);
        userProps.setProperty("country",defcountry);
        userProps.setProperty("theme","Default.theme");
        userProps.setProperty("fontsize","12.0");
        userProps.setProperty("props.version","1");

        try{
            File f = new File(System.getProperty("user.home")+File.separator+".NumericalChameleon");
            if (!f.exists()) {
                f.mkdir();
            }

            // load user properties
            try {
                userProps.load(new FileInputStream(System.getProperty("user.home")+File.separator+FILE_USER_PROPERTIES));
            } catch (FileNotFoundException fnfe) {
                System.out.println(fnfe);
            }

            // props.version 1 is obsoleted, upgrade to props.version 2
            if (userProps.getProperty("props.version").equals("1")) {
                // begin bugfix (for NC 1.5.0 users only);
                // NC will not start if you are not english, german, french or spain
                // therefore overwrite the settings with defaults again
                userProps.setProperty("language",deflang);
                userProps.setProperty("categories",deflang);
                // userProps.setProperty("iso3166",deflang);
                userProps.setProperty("help",deflang);
                userProps.setProperty("country",defcountry);
                // end bugfix

                // begin bugfix (for NC 1.5.0+ users)
                // problems with Kunststoff LnF on Mac OS X
                if (System.getProperty("os.name").equals(MAC_OS_X))
                    userProps.setProperty("lnf","system");
                // end bugfix

                // now we have props.version 2
                userProps.setProperty("props.version","2");
            }

            if (debug) System.out.print(System.getProperty("user.home")+File.separator+FILE_USER_PROPERTIES);
            Locale.setDefault(new Locale(userProps.getProperty("language"),userProps.getProperty("country")));
            if (debug) System.out.println(" OK.");
        }  catch (Exception e) {
            System.err.println(e);
        }
    }

    private void generatingHelp() throws Exception {
        if (debug) System.out.print("Loading help ... ");

        ClassLoader cl = Main.class.getClassLoader();
        java.net.URL url = HelpSet.findHelpSet(cl,"help/"+userProps.getProperty("help","en")+"/index.hs");
        mainHS = new HelpSet(cl, url);
        mainHB = mainHS.createHelpBroker();

        //mainHB.enableHelpKey(getRootPane(), "description", null);

        mainHB.setSize(new Dimension(800,600));
        java.awt.Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
        mainHB.setLocation(new Point(
        (dimension.width - mainHB.getSize().width) / 2,
        (dimension.height - mainHB.getSize().height) / 2 ));
        if (debug) System.out.println("OK.");
    }

    private void generatingTables() throws Exception {
        if (debug) System.out.print("Loading tables ... ");

        ClassLoader cl = Main.class.getClassLoader();
        java.net.URL url = HelpSet.findHelpSet(cl,"help/"+userProps.getProperty("help","en")+"/index.tables.hs");
        tableHS = new HelpSet(cl, url);
        tableHB = tableHS.createHelpBroker();

        //tableHB.enableHelpKey(getRootPane(), "description", null);

        tableHB.setSize(new Dimension(800,600));
        java.awt.Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();

        tableHB.setLocation(new Point(
        (dimension.width - tableHB.getSize().width) / 2,
        (dimension.height - tableHB.getSize().height) / 2 ));
        if (debug) System.out.println("OK.");
    }

    private void showAboutDialog() {

        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        AboutDialog dialog = new AboutDialog(this,false);
        dialog.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/logo.jpg")));
        if (GeneralProgram.isSupportFor("1.4") && isUndecorated()) {
            dialog.setUndecorated(true);
            dialog.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
        }
        try {
            dialog.setLicense(readFromJar("/core/docs/license.html"));
        } catch (Exception e) { dialog.setLicense(GeneralString.message(rb.getString("Error.FileDoesNotExist"),"license.html")); }
        try {
            dialog.setThanks(readFromJar("/core/docs/thanks.html"));
        } catch (Exception e) { dialog.setThanks(GeneralString.message(rb.getString("Error.FileDoesNotExist"),"thanks.html")); }
        dialog.setTitle(rb.getString("GUI.Menu.Help.About"));
        dialog.setAboutTab(rb.getString("GUI.Menu.Help.About"));
        dialog.setLicenseTab(rb.getString("GUI.General.License"));
        dialog.setThanksTab(rb.getString("GUI.General.Thanks"));
        dialog.create();
        GeneralGUI.setLocationRelativeTo(dialog,this);
        dialog.show();
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        dialog=null;
        sourceTextArea.requestFocus();
    }

    private void loadResourceBundle() {
        if (debug) System.out.print("Loading resource bundle ... ");
        try {
            rb = ResourceBundle.getBundle("data.lang.resource");

            // modify the OptionPane, because it's localized by the JVM only for the top 10 languages
            UIManager.put("OptionPane.yesButtonText", rb.getString("GUI.General.Yes"));
            UIManager.put("OptionPane.noButtonText", rb.getString("GUI.General.No"));
            UIManager.put("OptionPane.cancelButtonText", rb.getString("GUI.General.Cancel"));
            UIManager.put("OptionPane.okButtonText", rb.getString("GUI.General.OK"));

            AboutDialog.OK                        = rb.getString("GUI.General.OK");
            AboutDialog.THANKS                    = rb.getString("GUI.General.Thanks");
            AboutDialog.LICENSE                   = rb.getString("GUI.General.License");
            AboutDialog.ABOUT                     = rb.getString("GUI.Menu.Help.About");

            // localization: set static constants
            ClusterGeneric.INVALID                = rb.getString("Moduls.sourceInvalid");
            ExtendedGregorianCalendar.ERROR_1583  = rb.getString("Message.GUI.Year1583");

            ClusterBases35.DECIMAL                = rb.getString("Moduls.decimalName");
            ClusterBases35.INVALID                = rb.getString("Moduls.sourceInvalid");
            ClusterBases35.RADIX                  = rb.getString("Radix.radix");
            ClusterBases35.DUAL                   = rb.getString("Radix.dual");
            ClusterBases35.OCTAL                  = rb.getString("Radix.octal");
            ClusterBases35.HEX                    = rb.getString("Radix.hex");

            ClusterRoman.DECIMAL                  = rb.getString("Moduls.decimalName");
            ClusterRoman.ROMAN                    = rb.getString("Roman.roman");
            ClusterRoman.INVALID                  = rb.getString("Roman.decimalInvalid");
            RomanNumber.ERROR_RANGE               = rb.getString("Roman.decimalRange");
            RomanNumber.ERROR_INVALIDCHAR         = rb.getString("Roman.invalidChar");
            RomanNumber.ERROR_ISEMPTY             = rb.getString("Roman.isEmpty");
            RomanNumber.ERROR_CHARONLYONCE        = rb.getString("Roman.charOnlyOnce");
            RomanNumber.ERROR_CHAR3TIMES          = rb.getString("Roman.char3times");

            ClusterTemperature.INVALID            = rb.getString("Moduls.sourceInvalid");
            ClusterTemperature.KELVIN             = rb.getString("Temperature.kelvin");
            ClusterTemperature.CENTIGRADE         = rb.getString("Temperature.centigrade");
            ClusterTemperature.FAHRENHEIT         = rb.getString("Temperature.fahrenheit");
            ClusterTemperature.RANKINE            = rb.getString("Temperature.rankine");
            ClusterTemperature.REAUMUR            = rb.getString("Temperature.reaumur");

            ClusterCalendar.INVALID               = rb.getString("Moduls.sourceInvalid");
            ClusterCalendar.DATE                  = rb.getString("Calendar.date");
            ClusterCalendar.DAY_OF_WEEK           = rb.getString("Calendar.dayOfWeek");
            ClusterCalendar.DAY_OF_YEAR           = rb.getString("Calendar.dayOfYear");
            ClusterCalendar.WEEK_OF_MONTH         = rb.getString("Calendar.weekOfMonth");
            ClusterCalendar.WEEK_OF_YEAR          = rb.getString("Calendar.weekOfYear");
            ClusterCalendar.LEAPYEAR              = rb.getString("Calendar.leapyear");
            ClusterCalendar.MONTH                 = rb.getString("Calendar.month");
            ClusterCalendar.AGE                   = rb.getString("Calendar.age");
            ClusterCalendar.AGE_IN_DAYS           = rb.getString("Calendar.ageInDays");
            ClusterCalendar.YES                   = rb.getString("GUI.General.Yes");
            ClusterCalendar.NO                    = rb.getString("GUI.General.No");
            ClusterCalendar.DAYS_IN_MONTH         = rb.getString("Calendar.daysInMonth");

            ClusterCalendarYear.INVALID           = rb.getString("Moduls.sourceInvalid");
            ClusterCalendarYear.YES               = rb.getString("GUI.General.Yes");
            ClusterCalendarYear.NO                = rb.getString("GUI.General.No");
            ClusterCalendarYear.LEAPYEAR          = rb.getString("Calendar.leapyear");
            ClusterCalendarYear.CENTURY           = rb.getString("Calendar.century");
            ClusterCalendarYear.MILLENNIUM        = rb.getString("Calendar.millennium");
            ClusterCalendarYear.YEAR              = rb.getString("Calendar.year");

            ClusterSpokenNumbers.INVALID          = rb.getString("Moduls.sourceInvalid");
            ClusterSpokenNumbers.DECIMAL          = rb.getString("Moduls.decimalName");
            ClusterSpokenNumbers.languageUS       = rb.getString("Language.us");
            ClusterSpokenNumbers.languageUK       = rb.getString("Language.gb");
            ClusterSpokenNumbers.languageFR       = rb.getString("Language.fr");
            ClusterSpokenNumbers.languageES       = rb.getString("Language.es");
            ClusterSpokenNumbers.languageDE       = rb.getString("Language.de");
            ClusterSpokenNumbers.languageIT       = rb.getString("Language.it");
            ClusterSpokenNumbers.languageFI       = rb.getString("Language.fi");
            ClusterSpokenNumbers.languageIN_hindi = rb.getString("Language.in_hindi");
            ClusterSpokenNumbers.languageIN_tamil = rb.getString("Language.in_tamil");
            ClusterSpokenNumbers.languageHU       = rb.getString("Language.hu");
            ClusterSpokenNumbers.languageSE       = rb.getString("Language.se");
            ClusterSpokenNumbers.languageTR       = rb.getString("Language.tr");
            ClusterSpokenNumbers.languagePL       = rb.getString("Language.pl");
            ClusterSpokenNumbers.languagePT       = rb.getString("Language.pt");
            ClusterSpokenNumbers.languageBR       = rb.getString("Language.br");
            ClusterSpokenNumbers.languageRO       = rb.getString("Language.ro");
            ClusterSpokenNumbers.languageRU       = rb.getString("Language.ru");
            ClusterSpokenNumbers.languageJP       = rb.getString("Language.jp");
            ClusterSpokenNumbers.languageESP      = rb.getString("Language.esp");
            ClusterSpokenNumbers.languageLAT      = rb.getString("Language.lat");
            ClusterSpokenNumbers.languageKLINGON  = rb.getString("Language.klingon");
            ClusterSpokenNumbers.languageMORSE    = rb.getString("Language.morsecode");
            ClusterSpokenNumbers.languageNL       = rb.getString("Language.nl");

            ClusterUnicode.INVALID                = rb.getString("Moduls.sourceInvalid");
            ClusterUnicode.DECIMAL                = rb.getString("Moduls.decimalName");
            ClusterUnicode.HEX                    = rb.getString("Radix.hex");

            SpokenNumber.ERROR_TOOBIG             = rb.getString("SpokenNumber.toobig");
            SpokenNumber.ERROR_TOOSMALL           = rb.getString("SpokenNumber.toosmall");

            ClusterAngle.INVALID                  = rb.getString("Moduls.sourceInvalid");
            ClusterAngle.DEGREES                  = rb.getString("Angle.degrees");
            ClusterAngle.GON                      = rb.getString("Angle.gon");
            ClusterAngle.RADIANS                  = rb.getString("Angle.radians");

            ClusterColor.INVALID                  = rb.getString("Moduls.sourceInvalid");
            ClusterColor.DECIMAL                  = rb.getString("Moduls.decimalName");

            ClusterAlphaPhone.ALPHA               = rb.getString("AlphaPhone.alphaName");
            ClusterAlphaPhone.NUM                 = rb.getString("AlphaPhone.numName");


            PhoneticAlphabet.AMERICAN_NAME        = rb.getString("Language.us");
            PhoneticAlphabet.BRITISH_NAME         = rb.getString("Language.gb");
            PhoneticAlphabet.FRENCH_NAME          = rb.getString("Language.fr");
            PhoneticAlphabet.SPAIN_NAME           = rb.getString("Language.es");
            PhoneticAlphabet.ITALIAN_NAME         = rb.getString("Language.it");
            PhoneticAlphabet.GERMAN_NAME          = rb.getString("Language.de");
            PhoneticAlphabet.PORTUGUESE_NAME      = rb.getString("Language.pt");
            PhoneticAlphabet.DANISH_NAME          = rb.getString("Language.dk");
            PhoneticAlphabet.DUTCH_NAME           = rb.getString("Language.nl");
            PhoneticAlphabet.SWEDISH_NAME         = rb.getString("Language.se");
            PhoneticAlphabet.HEBREW_NAME          = rb.getString("Language.il");

            PhoneticAlphabet.INTERNATIONAL_NAME   = rb.getString("PhoneticAlphabet.International");
            PhoneticAlphabet.NATO_NAME            = rb.getString("PhoneticAlphabet.NATO");
            PhoneticAlphabet.MORSE_NAME           = rb.getString("PhoneticAlphabet.Morse");

            if (debug) System.out.println("OK.");
        }
        catch(MissingResourceException ex) {
            error(ex.getMessage()+"\nExit.");
            System.exit(1);
        }
    }

    /**
     * prints an error at the command line and at the GUI
     */
    private void error(String error) {
        System.err.println(error);
        JOptionPane.showMessageDialog(this,error,rb.getString("Error.errorname"),JOptionPane.ERROR_MESSAGE);
    }

    public void initThemes() {
        if (debug) System.out.print("Init themes ... ");

        defaultMetalTheme = new DefaultMetalTheme();
        String lnf = userProps.getProperty("lnf","kunststoff");
        // bugfix for Java Webstart, because Kunststoff LnF has problems with
        // JavaWebstart (e.g. JavaHelp is not shown, Colorchooser does not work correctly, etc.)
        // Metouia has also wired problems
        // so let's start with Standard Metal
        if (jnlp) lnf="metal";

        String theme = userProps.getProperty("theme","Default.theme");
        if (lnf.equals("kunststoff")) {
            setKunststoffTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+theme)));
        } else
        if (lnf.equals("metal")) {
             setMetalTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+theme)));
        } else
         if (lnf.equals("metouia")) {
             setMetouiaTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+theme)));
        } else
        if (GeneralProgram.isSupportFor("1.4") && lnf.equals("plastic")) {
             setPlasticLnf();
        } else
        if (lnf.equals("mac")) setLaf(GeneralGUI.LAF_MAC); else
        if (lnf.equals("motif")) setLaf(GeneralGUI.LAF_MOTIF); else
        if (lnf.equals("windows")) setLaf(GeneralGUI.LAF_WINDOWS); else
        if (lnf.equals("metal_default")) setMetalTheme(defaultMetalTheme); else
        if (lnf.equals("metal_ocean"))  setMetalTheme(oceanTheme); else
        if (lnf.equals("squareness")) setSquarenessLnf(); else
        if (lnf.equals("system")) setLaf(UIManager.getSystemLookAndFeelClassName());

        if (debug) System.out.println("OK.");
    }

    /**
     * set Renderers to all important GUI components
     **/
    public void setRenderers() {
        if (sectionRecordCellRenderer==null)
            sectionRecordCellRenderer = new SectionRecordCellRenderer();

        GeneralGUI.applyOrientation(sectionRecordCellRenderer,
          isOrientationRightToLeft ? ComponentOrientation.RIGHT_TO_LEFT : ComponentOrientation.LEFT_TO_RIGHT);

        categoryBox.setRenderer(sectionRecordCellRenderer);

        if (ncRecordCellRenderer==null)
            ncRecordCellRenderer = new NcRecordCellRenderer();
        GeneralGUI.applyOrientation(ncRecordCellRenderer,
          isOrientationRightToLeft ? ComponentOrientation.RIGHT_TO_LEFT : ComponentOrientation.LEFT_TO_RIGHT);

        sourceBox.setRenderer(ncRecordCellRenderer);
        targetBox.setRenderer(ncRecordCellRenderer);
    }

    public void register() {
        if (debug) System.out.print("Loading categories.list ... ");
        try {
            readSectionsFromJar();
            if (debug) System.out.println("OK.");
        } catch (Exception e) {
            error("data/lists/categories.list not found. Exit.");
            System.exit(1);
        }

        fillCategoryModel(groupAllCategories);
        //categoryBox.setModel(getComboBoxModel(index));

        transferValue ="";
        categoryBoxItemStateChanged();
    }

    private void fillCategoryModel(Vector v) {

        DefaultComboBoxModel catModel = new DefaultComboBoxModel();
        for (int i=0; i < v.size(); i++) {
            catModel.addElement(categoriesHashtable.get(v.get(i)));
        }
        categoryBox.setModel(catModel);
/*
        Vector x = new Vector();
        SectionRecord sr = new SectionRecord();
        sr.setName("anzahl");
        x.add(sr);
        categoryBox.setModel(getComboBoxModel(x));
        categoryModel.removeAllElements();
        for (int i=0; i < v.size(); i++) {
          categoryModel.addElement(categoriesHashtable.get(v.get(i)));
        }
        categoryBox.setModel(categoryModel);
 */
        if (!groupsHashtable.isEmpty()) {
            // disable menu items if categories are not available (due to filtering)
            sourceMenuItemColor.setEnabled(groupsHashtable.containsKey("COLOR"));
            sourceMenuItemCalendar.setEnabled(groupsHashtable.containsKey("CALENDAR"));
        } else {
            sourceMenuItemColor.setEnabled(true);
            sourceMenuItemCalendar.setEnabled(true);
        }
    }


    private void prepareThemesList() {
        if (debug) System.out.print("Loading custom Look&Feels ... ");

        //if (home.charAt(home.length()-1)!=File.separatorChar)  home += File.separator;
        try {
            String myHome = "../data/themes/";
            if (jnlp) {
                themesVector = readFromJarInVector("/data/themes/javawebstart.list");
            } else {
                themesVector = GeneralIO.ls(myHome,".theme");
            }

            if (themesVector==null) throw new Exception("No themes found in "+myHome);

            // Java-Themes
            themesList = new JList(themesVector);
            if (themesVector.size() < 10)
                themesList.setVisibleRowCount(themesVector.size()); else
                    themesList.setVisibleRowCount(10);

            themesScrollPane = new JScrollPane();
            themesList.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent me) {
                    int index = themesList.locationToIndex(me.getPoint());
                    themesListValueChanged(index);
                }
            });
            themesScrollPane.setViewportView(themesList);
            themesMenu.add(themesScrollPane);
            // end Java-Themes

            // Kunststoff-Themes
            kunststoffThemesList = new JList(themesVector);
            if (themesVector.size() < 10)
                kunststoffThemesList.setVisibleRowCount(themesVector.size()); else
                    kunststoffThemesList.setVisibleRowCount(10);

            kunststoffThemesScrollPane = new JScrollPane();
            kunststoffThemesList.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent me) {
                    int index = kunststoffThemesList.locationToIndex(me.getPoint());
                    kunststoffThemesListValueChanged(index);
                }
            });
            kunststoffThemesScrollPane.setViewportView(kunststoffThemesList);
            kunststoffMenu.add(kunststoffThemesScrollPane);
            // end Kunststoff-Themes

            // Metouia-Themes
            metouiaThemesList = new JList(themesVector);
            if (themesVector.size() < 10)
                metouiaThemesList.setVisibleRowCount(themesVector.size()); else
                    metouiaThemesList.setVisibleRowCount(10);

            metouiaThemesScrollPane = new JScrollPane();
            metouiaThemesList.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent me) {
                    int index = metouiaThemesList.locationToIndex(me.getPoint());
                    metouiaThemesListValueChanged(index);
                }
            });
            metouiaThemesScrollPane.setViewportView(metouiaThemesList);
            metouiaMenu.add(metouiaThemesScrollPane);
            // end Metouia-Themes

            if (debug) System.out.println("OK.");
        }  catch (Exception e) {
            System.err.println("ERROR: "+e.getMessage());
        }
    }

    private void prepareGroupsList() {
        if (debug) System.out.print("Loading group filter ... ");

        try {
            String myHome = "../data/groups/";
            if (jnlp) {
                groupsVector = readFromJarInVector("/data/groups/javawebstart.list");
            } else {
                groupsVector = GeneralIO.ls(myHome,".group",false);
            }

            if (groupsVector==null) throw new Exception("No group files found in "+myHome);
            // groupsVector.insertElementAt("all", 0);

            // search for .group files in the home directory
            myHome = System.getProperty("user.home")+File.separator+DIR_USER_PROPERTIES+File.separator;
            Vector v2 = GeneralIO.ls(myHome,".group",false);
            if (v2!=null)
                groupsVector.addAll(v2);

            // Group files
            groupsList = new JList(groupsVector);
            if (groupsVector.size() < 10)
                groupsList.setVisibleRowCount(groupsVector.size()); else
                    groupsList.setVisibleRowCount(10);

            groupsScrollPane = new JScrollPane();
            groupsList.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent me) {
                    int index = groupsList.locationToIndex(me.getPoint());
                    groupsListValueChanged(index);
                }
            });
            groupsScrollPane.setViewportView(groupsList);
            groupsMenu.add(groupsScrollPane);
            // end Java-Themes

            if (debug) System.out.println("OK.");
        }  catch (Exception e) {
            System.err.println("ERROR: "+e.getMessage());
        }
    }

    private void themesListValueChanged(int index) {
        try {
            themesMenu.setPopupMenuVisible(false);
            LaFMenu.setPopupMenuVisible(false);
            configMenu.setPopupMenuVisible(false);
            setMetalTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+themesVector.get(index))));
            userProps.setProperty("theme",(String)themesVector.get(index));
            userProps.setProperty("lnf","metal");

            saveUserProperties();
        } catch (Exception e) {
            error("ERROR: theme corrupted.");
        }
    }

    private void groupsListValueChanged(int index) {
        try {
            groupsMenu.setPopupMenuVisible(false);
            navigationMenu.setPopupMenuVisible(false);
            navigationMenu.setSelected(false);

            InputStream is = getClass().getResourceAsStream("/data/groups/"+groupsVector.get(index)+".group");
            BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
            String thisLine;

            groupCurrentSelection.clear(); // remain the order in the file
            groupsHashtable.clear(); // for an indexed access

            while ((thisLine = br.readLine()) != null) {
                groupsHashtable.put(thisLine,"");
                groupCurrentSelection.add(thisLine);
            }
            br.close();
            is.close();
            filterLabel.setToolTipText(GeneralString.message(rb.getString("GUI.Area.NoFilter"),(String)groupsVector.get(index)));
            filterLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/data/images/groups/"+groupsVector.get(index)+".png")));
            filterLabel.setVisible(true);
            filterCatIndicatorLabel.setToolTipText(filterLabel.getToolTipText());
            filterCatIndicatorLabel.setVisible(true);
            deactivateFilterCategoriesMenuItem.setEnabled(true);
            fillCategoryModel(groupCurrentSelection);

            SwingUtilities.updateComponentTreeUI(getRootPane());
            categoryBoxItemStateChanged();

            // reload categories
            // load group file and add entries to a hashtable
            //userProps.setProperty("group",(String)gruopsVector.get(index));
            //saveUserProperties();
        } catch (Exception e) {
            System.err.println(e);
            error("ERROR: group file corrupted.");
        }
    }


    private void kunststoffThemesListValueChanged(int index) {
        try {
            kunststoffMenu.setPopupMenuVisible(false);
            LaFMenu.setPopupMenuVisible(false);
            configMenu.setPopupMenuVisible(false);
            setKunststoffTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+themesVector.get(index))));
            userProps.setProperty("theme",(String)themesVector.get(index));
            userProps.setProperty("lnf","kunststoff");
            saveUserProperties();
        } catch (Exception e) {
            error("ERROR: theme corrupted.");
        }
    }

    private void metouiaThemesListValueChanged(int index) {
        try {
            metouiaMenu.setPopupMenuVisible(false);
            LaFMenu.setPopupMenuVisible(false);
            configMenu.setPopupMenuVisible(false);
            setMetouiaTheme(new PropertiesMetalTheme(getClass().getResourceAsStream("/data/themes/"+themesVector.get(index))));
            userProps.setProperty("theme",(String)themesVector.get(index));
            userProps.setProperty("lnf","metouia");
            saveUserProperties();
        } catch (Exception e) {
            error("ERROR: theme corrupted.");
        }
    }

    public void squarenessLnfPressed() {
        setSquarenessLnf();
        userProps.setProperty("lnf","squareness");
        saveUserProperties();
    }

    public void plasticLnfPressed() {
        setPlasticLnf();
        userProps.setProperty("lnf","plastic");
        saveUserProperties();
    }

    private void setKunststoffTheme(MetalTheme theme) {
        if (klaf==null) klaf=new com.incors.plaf.kunststoff.KunststoffLookAndFeel();
        try {
            klaf.setCurrentTheme(theme);
            UIManager.setLookAndFeel(klaf);
            updateLaF();
        } catch (Exception e) {}
    }

    private void setMetalTheme(MetalTheme theme) {
        try {
            MetalLookAndFeel.setCurrentTheme(theme);
            UIManager.setLookAndFeel(GeneralGUI.LAF_METAL);
            updateLaF();
        } catch (Exception e) {}
    }

    private void setMetouiaTheme(MetalTheme theme) {
        if (mlnf==null) mlnf = new net.sourceforge.mlf.metouia.MetouiaLookAndFeel();
        try {
            UIManager.setLookAndFeel(mlnf);
            mlnf.setCurrentTheme(theme);
            UIManager.setLookAndFeel(mlnf);
            updateLaF();
        } catch (Exception e) {}
    }

    private void setPlasticLnf() {
        if (plnf==null) plnf = new com.jgoodies.plaf.plastic.PlasticXPLookAndFeel();
        try {
            PlasticLookAndFeel.setMyCurrentTheme(new DesertBlue());
            UIManager.setLookAndFeel(plnf);
            updateLaF();
        } catch (Exception e) {}
    }

    private void setSquarenessLnf() {
        if (slnf==null) slnf = new net.beeger.squareness.SquarenessLookAndFeel();
        try {
            UIManager.setLookAndFeel(slnf);
            updateLaF();
        } catch (Exception e) {}
    }

    private void updateLaF() {
        SwingUtilities.updateComponentTreeUI(getRootPane());
        if (view > -1) this.pack();
    }

    private String readFromJar(String filename) throws Exception {
        InputStream is = getClass().getResourceAsStream(filename);
        BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
        String thisLine;
        StringBuffer sb = new StringBuffer();
        while ((thisLine = br.readLine()) != null) {
            sb.append(thisLine);
            sb.append("\n");
        }
        br.close();
        return sb.toString();
    }

    private Vector readFromJarInVector(String filename) throws Exception {
        Vector v = new Vector();
        InputStream is = getClass().getResourceAsStream(filename);
        BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
        String thisLine;
        StringBuffer sb = new StringBuffer();
        while ((thisLine = br.readLine()) != null) {
            v.add(thisLine);
        }
        br.close();
        return v;
    }


    private String getPropertyDefault(Properties properties, String code, String defaultString) {
        if (properties.containsKey(code))
        return properties.getProperty(code); else
        return defaultString;

    }
    /**
     * categoriesHashtable will be filled with all Categories
     * (key=category-code, value=SectionRecord)
     * groupAllCategories Vector containing all category-codes
     */
    private void readSectionsFromJar() throws Exception{

        categoriesHashtable = new Hashtable();
        groupAllCategories = new Vector();

        // read the category file
        Properties cat = new Properties();
        Properties catDefault = new Properties();

        try {
            InputStream is = getClass().getResourceAsStream(
               "/data/lang/categories.properties");
            catDefault.load(is);
            is.close();
        } catch (Exception e) {
            System.out.println(e);
        }

        try{

            String lang = userProps.getProperty("language","en");
            InputStream is = getClass().getResourceAsStream(
                    "/data/lang/categories_"+
                    lang+"_"+userProps.getProperty("country","")+".properties");

            if (is==null) {
                is = getClass().getResourceAsStream("/data/lang/categories_"+lang+".properties");
                if (is==null) {
                    is = getClass().getResourceAsStream("/data/lang/categories.properties");
                }
            }
            cat.load(is);
            is.close();
            if (debug) System.out.println(" OK.");
        }  catch (Exception e) {
            System.err.println(e);
        }

        InputStream is = getClass().getResourceAsStream("/data/lists/categories.list");
        BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
        BigDecimal bigdecimal = null;
        StringTokenizer st = null;
        String description = null;

        String thisLine;
        String code;
        //        color_pos=-1;
        //        calendar_pos=-1;

        while ((thisLine = br.readLine()) != null) {
            if (!thisLine.startsWith("#") && !thisLine.equals("")) {

                st = new StringTokenizer(thisLine,":");
                // code
                code = st.nextToken();
                // is code in the white list? (groupsHashtable)
                if (groupsHashtable.isEmpty() || groupsHashtable.containsKey(code)) {

                    SectionRecord sr = new SectionRecord();
                    sr.setName(code);
                    sr.setIcon(IMAGE_DIR+"/sections/"+code.toLowerCase()+".png");

                    // description
                    // Category names without a translation should be read from the default list and if
                    // not in the default list display them as "NOT TRANSLATED".
                    description = cat.getProperty("Category."+code,getPropertyDefault(catDefault,"Category."+code,"NOT TRANSLATED"));

                    // modify description if exchange rates
                    if (code.equals("exchange_rates")) {
                        exchange_rates_description=description;
                        description = GeneralString.replaceString(description,"$RATES_DATE", systemProps.getProperty("rates_date","n/a"));
                    }

                    sr.setString(description);

                    // figures
                    boolean cont = false;
                    String figures = "-1";
                    if (st.hasMoreTokens()) {
                        figures = st.nextToken();
                        cont = true;
                    } else figures="12";
                    if (figures.equals("-")) figures="-1";
                    int fig = -1;
                    try {
                        fig = Integer.parseInt(figures);
                    } catch (Exception e) {}
                    sr.setFigures(fig);

                    // Logic
                    boolean b = true;
                    if (cont && st.hasMoreTokens()) {
                        try {
                            String token = st.nextToken();
                            if (code.equals("exchange_rates")) {
                                token = GeneralString.replaceString(token,"$RATES_LOGIC", systemProps.getProperty("rates_logic","false"));
                            }
                            b = (token.equals("true") || token.equals("1"));

                        } catch (Exception e) {}
                        //cont = true;
                    }
                    sr.setLogic(b);
                    groupAllCategories.add(code);
                    categoriesHashtable.put(code,sr);
                }
            }
        }
        br.close();
    }

    /**
     * categoryBox - the item state changed
     */
    private void categoryBoxItemStateChanged() {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        SectionRecord sr = (SectionRecord)categoryBox.getSelectedItem();
        if (sr==null) sr = (SectionRecord)categoryBox.getItemAt(0);
        String code = sr.getName();
        int precision = sr.getFigures();
        boolean logic = sr.getLogic();

        if (code.equals(ClusterBases35.CODE))
            clusterObject = new ClusterBases35(); else
        if (code.equals(ClusterRoman.CODE))
            clusterObject = new ClusterRoman(); else
        if (code.equals(ClusterSpokenNumbers.CODE)) {
            clusterObject = new ClusterSpokenNumbers();
            targetMenuItemPlay.setEnabled(true);
            targetMenuItemStopPlaying.setEnabled(true);
            targetPopupMenuItemPlay.setEnabled(true);
            targetPopupMenuItemStopPlaying.setEnabled(true);
/*
            swapButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/sound24.png")));
            swapButton.setMnemonic('p');
            swapButton.setText(rb.getString("GUI.Area.Main.Play.Button"));
            swapButton.setToolTipText(null);
 */
            cardLayout.show(cardPanel,"soundCard");
        } else
        if (code.equals(ClusterTemperature.CODE))
            clusterObject = new ClusterTemperature(); else
        if (code.equals(ClusterAngle.CODE))
            clusterObject = new ClusterAngle(); else
        if (code.equals(ClusterUnicode.CODE))
            clusterObject = new ClusterUnicode(); else
        if (code.equals(ClusterColor.CODE)) {
            clusterObject = new ClusterColor();
            cardLayout.show(cardPanel,"colCard");
        } else
        if (code.equals(ClusterCalendar.CODE)) {
            clusterObject = new ClusterCalendar();
            cardLayout.show(cardPanel,"calCard");
        } else
        if (code.equals(ClusterCalendarYear.CODE))
            clusterObject = new ClusterCalendarYear(); else
        if (code.equals(ClusterTimezones.CODE))
            clusterObject = new ClusterTimezones(); else
        if (code.equals(ClusterAlphaPhone.CODE))
            clusterObject = new ClusterAlphaPhone(); else
        if (code.equals(ClusterDialCode.CODE))
            clusterObject = new ClusterDialCode(); else {
            // we read data from files
            clusterObject = new ClusterGeneric(code,logic);
            clusterObject.setPreferredPrecision(precision);
        }
        sourceBox.setModel(getComboBoxModel(clusterObject.getSourceEntries()));
        sourceBox.setSelectedIndex(clusterObject.getSourceDefault());
        targetBox.setModel(getComboBoxModel(clusterObject.getTargetEntries()));
        targetBox.setSelectedIndex(clusterObject.getTargetDefault());


        if (!code.equals(ClusterSpokenNumbers.CODE)) {
            targetMenuItemPlay.setEnabled(false);
            targetMenuItemStopPlaying.setEnabled(false);
            targetPopupMenuItemPlay.setEnabled(false);
            targetPopupMenuItemStopPlaying.setEnabled(false);
        }

        if (!code.equals(ClusterSpokenNumbers.CODE) &&
        !code.equals(ClusterCalendar.CODE) &&
        !code.equals(ClusterColor.CODE)) {
            cardLayout.show(cardPanel,"sciCard");
        }

        swapButton.setEnabled(!clusterObject.isOneway());
        programMenuItemSwap.setEnabled(!clusterObject.isOneway());

        // mit Wert vom letzten Cluster fllen (transferValue)
        String initValue = clusterObject.getInitialValue();
        sourceTextArea.setText(initValue == null ? transferValue : initValue);

        precisionerror=false;

        if (clusterObject.getPreferredPrecision() == -1) {
            precisionTextField.setText("0");
            precisionTextField.setEditable(false);
            precisionTextField.setEnabled(false);
            precisionLabel.setEnabled(false);
            targetMenuItemFigures.setEnabled(false);
        } else {
            precisionTextField.setText(Integer.toString(clusterObject.getPreferredPrecision()));
            clusterObject.setPrecision(clusterObject.getPreferredPrecision());
            precisionTextField.setEditable(true);
            precisionTextField.setEnabled(true);
            precisionLabel.setEnabled(true);
            targetMenuItemFigures.setEnabled(true);
        }
        sciCheckBox.setEnabled(clusterObject.isScientificSupported());
        clusterObject.setScientific(sciCheckBox.isSelected());
        int count = targetBox.getItemCount();
        if (sourceBox.getItemCount() > count) count=sourceBox.getItemCount();

        process(count);
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        sourceTextArea.requestFocus();
    }


    // perform a conversion with info about currently supported units
    private void process(int support) {
        if (precisionerror) messageError(rb.getString(ERROR_GUI_PRECISION)); else

            if (!clusterObject.acceptEmptyStrings() && sourceTextArea.getText().equals("")) {
                targetTextArea.setText("");
                messageOk(GeneralString.message(rb.getString("Message.GUI.Support"),support)+" "+rb.getString(MESSAGE_GUI_PROMPT),false);
            } else {
                clusterObject.setInput(sourceTextArea.getText());
                try {
                    targetTextArea.setText(clusterObject.getOutput(sourceBox.getSelectedIndex(),targetBox.getSelectedIndex()));
                    messageOk(GeneralString.message(rb.getString("Message.GUI.Support"),support),true);
                } catch (Exception e) {
                    messageError(e.getMessage()); // alert
                }

            }
    }

    /**
     * perform a conversion
     */
    private void process() {
        // no unneccessary process if precission is wrong
        if (precisionerror) messageError(rb.getString(ERROR_GUI_PRECISION)); else

            if (!clusterObject.acceptEmptyStrings() && sourceTextArea.getText().equals("")) {
                targetTextArea.setText("");
                messageOk(rb.getString(MESSAGE_GUI_PROMPT),false);
            } else {

                clusterObject.setInput(sourceTextArea.getText());
                try {
                    // eigentliche Konvertierung
                    targetTextArea.setText(clusterObject.getOutput(sourceBox.getSelectedIndex(),targetBox.getSelectedIndex()));
                    messageOk("ok.",true);
                } catch (Exception e) {
                    messageError(e.getMessage()); // alert
                }

            }
    }

    /**
     * method to add a value (usually -1 or +1)
     */
    private void addValue(BigDecimal plus) throws Exception {
        String temp = clusterObject.addValue(plus,sourceBox.getSelectedIndex());
        sourceTextArea.setText(temp);
        process();
    }

    /*
     * method to set a value for source
     * (is used for the feature "generate a list")
     */
    private void setValue(BigDecimal big) throws Exception {
        String temp = clusterObject.setValue(big,sourceBox.getSelectedIndex());
        sourceTextArea.setText(temp);
        process();
    }

    private javax.swing.ComboBoxModel getComboBoxModel(JLabel[] data) {
        javax.swing.JComboBox cb = new javax.swing.JComboBox(data);
        return cb.getModel();
    }

    private javax.swing.ComboBoxModel getComboBoxModel(String[] data) {
        javax.swing.JComboBox cb = new javax.swing.JComboBox(data);
        return cb.getModel();
    }

    private javax.swing.ComboBoxModel getComboBoxModel(Vector data) {
        javax.swing.JComboBox cb = new javax.swing.JComboBox(data);
        return cb.getModel();
    }

    private javax.swing.ListModel getListModel(Vector data) {
        javax.swing.JList jl = new javax.swing.JList(data);
        return jl.getModel();
    }


    private void messageLightError(String s) {
        messageTextField.setText(s);
        messageTextField.setBackground(Color.red);
        messageTextField.setForeground(Color.white);
    }

    private void messageError(String s) {
        messageLightError(s);
        targetTextArea.setText("");
        historyNewButton.setEnabled(false);
        recordMenuItemNew.setEnabled(false);
        historyPopupMenuItemNew.setEnabled(false);
        //        sourceMinusButton.setEnabled(false);
        //        sourcePlusButton.setEnabled(false);
    }

    private void messageOk(String s) {
        messageTextField.setText(s);
        messageTextField.setBackground(GREEN);
        messageTextField.setForeground(Color.black);
    }

    private void messageOk(String s, boolean b) {
        messageOk(s);
        historyNewButton.setEnabled(b);
        recordMenuItemNew.setEnabled(b);
        historyPopupMenuItemNew.setEnabled(b);
        //        sourceMinusButton.setEnabled(b);
        //        sourcePlusButton.setEnabled(b);
    }

    private void checkPopupMenu(MouseEvent event, JPopupMenu popupMenu) {
        if (event.isPopupTrigger()) {
            popupMenu.show(event.getComponent(),event.getX(),event.getY());
        }
    }

    private void textAreaClear(JTextArea textArea) {
        int start = textArea.getSelectionStart();
        int end = textArea.getSelectionEnd();
        if (start==end) {
            textArea.setText("");
            clusterObject.setInput("");
        } else {
            StringBuffer sb = new StringBuffer(textArea.getText());
            sb = sb.delete(start,end);
            textArea.setText(sb.toString());
            textArea.setCaretPosition(start);
            if (textArea.getText().equals("")) clusterObject.setInput("");
        }
        process();
        textArea.requestFocus();
    }


    private void sourceTextAreaInsert(char c) {
        StringBuffer  sb = new StringBuffer(sourceTextArea.getText());
        int i = sourceTextArea.getCaretPosition();
        sb.insert(i,c);
        sourceTextArea.setText(sb.toString());
        sourceTextArea.setCaretPosition(i+1);
        process();
        sourceTextArea.requestFocus();
    }


    private void textAreaCopy(JTextArea textArea) {
        String s = textArea.getSelectedText();
        if (s == null) {
            textArea.selectAll();
            s = textArea.getText();
        }
        GeneralIO.setClipboard(s);
        textArea.requestFocus();
    }

    private void textAreaPaste(JTextArea textArea) {
        try {
            int start = textArea.getSelectionStart();
            int end = textArea.getSelectionEnd();
            StringBuffer sb = new StringBuffer(textArea.getText());
            if (start==end) { // no selection
                start = textArea.getCaretPosition();
            } else {
                sb = sb.delete(start,end);
            }
            String s = GeneralIO.getClipboard();
            sb = sb.insert(start,GeneralIO.getClipboard());
            textArea.setText(sb.toString());
            textArea.setCaretPosition(start+s.length());
            process();
            textArea.requestFocus();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this,e.getMessage(),"Clipboard Info", JOptionPane.WARNING_MESSAGE);
        }
    }

    private void setLaf(String laf) {
        try {
            UIManager.setLookAndFeel(laf);
            updateLaF();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, rb.getString(ERROR_GUI_LOOKANDFEEL), "Info", JOptionPane.INFORMATION_MESSAGE);
            System.err.println(e);
        }
    }

    private void luxuryFrame(int i) {
        view=i;
        recordPanel.setVisible(i < 1);
        sourceButtonPanel.setVisible(i < 2);
        targetButtonPanel.setVisible(i < 2);
        headerPanel.setVisible(i < 3);
        userProps.setProperty("view",i+"");
        saveUserProperties();

        if (i > -1) {
            pack();
            GeneralGUI.centerComponent(this);
        } else {
            GeneralGUI.maximizeComponent(this);
        }
    }

    public void performNotice() {
        if (multipleNoticeCluster.getListFormat() != null) {
            notice_form = multipleNoticeCluster.getListFormat().getFormat();
            try {
                String result = multipleNoticeCluster.getListFormat().reformatDate(targetTextArea.getText());
                notice_form = GeneralString.replaceString(notice_form,"$TVALUE", result);
                result = multipleNoticeCluster.getListFormat().reformatUnit(((NcRecord)targetBox.getSelectedItem()).getString());
                notice_form = GeneralString.replaceString(notice_form,"$TUNIT", result);
                historyTextArea.insert(notice_form+"\n",historyTextArea.getCaretPosition());

            } catch (java.text.ParseException pe) {
                System.err.println(pe);
                // don't add it
            }
        }  else {
            notice_form = userProps.getProperty("notice_form",DEFAULT_NOTICE_FORM);
            notice_form = GeneralString.replaceString(notice_form,"$SVALUE", sourceTextArea.getText());
            notice_form = GeneralString.replaceString(notice_form,"$SUNIT", ((NcRecord)sourceBox.getSelectedItem()).getString());

            notice_form = GeneralString.replaceString(notice_form,"$TVALUE", targetTextArea.getText());
            notice_form = GeneralString.replaceString(notice_form,"$TUNIT", ((NcRecord)targetBox.getSelectedItem()).getString());

            notice_form = GeneralString.replaceString(notice_form,"$SFLAG", ((NcRecord)targetBox.getSelectedItem()).getIcon());
            notice_form = GeneralString.replaceString(notice_form,"$TFLAG", ((NcRecord)targetBox.getSelectedItem()).getIcon());
            historyTextArea.insert(notice_form+"\n",historyTextArea.getCaretPosition());

        }
    }



    // GUI-GENERATED
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        sourcePopupMenu = new javax.swing.JPopupMenu();
        sourcePopupMenuItemUndo = new javax.swing.JMenuItem();
        sourcePopupMenuItemRedo = new javax.swing.JMenuItem();
        jSeparator26 = new javax.swing.JSeparator();
        sourcePopupMenuItemClear = new javax.swing.JMenuItem();
        sourcePopupMenuItemCopy = new javax.swing.JMenuItem();
        sourcePopupMenuItemPaste = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        sourcePopupMenuItemPlus = new javax.swing.JMenuItem();
        sourcePopupMenuItemMinus = new javax.swing.JMenuItem();
        sourcePopupMenuDefaultValue = new javax.swing.JMenuItem();
        jSeparator16 = new javax.swing.JSeparator();
        sourcePopupMenuItemFindUnit = new javax.swing.JMenuItem();
        sourcePopupMenuItemDefault = new javax.swing.JMenuItem();
        targetPopupMenu = new javax.swing.JPopupMenu();
        targetPopupMenuItemCopy = new javax.swing.JMenuItem();
        jSeparator11 = new javax.swing.JSeparator();
        targetPopupMenuItemFindUnit = new javax.swing.JMenuItem();
        targetPopupMenuItemDefault = new javax.swing.JMenuItem();
        jSeparator14 = new javax.swing.JSeparator();
        targetPopupMenuItemPlay = new javax.swing.JMenuItem();
        targetPopupMenuItemStopPlaying = new javax.swing.JMenuItem();
        historyPopupMenu = new javax.swing.JPopupMenu();
        historyMenuItemUndo = new javax.swing.JMenuItem();
        historyMenuItemRedo = new javax.swing.JMenuItem();
        jSeparator24 = new javax.swing.JSeparator();
        historyMenuItemClear = new javax.swing.JMenuItem();
        historyMenuItemCopy = new javax.swing.JMenuItem();
        historyMenuItemPaste = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        historyPopupMenuItemNew = new javax.swing.JMenuItem();
        historyPopupMenuItemMakeList = new javax.swing.JMenuItem();
        historyPopupMenuenuItemFormat = new javax.swing.JMenuItem();
        historyMenuFontSize = new javax.swing.JMenuItem();
        historyPopupMenuItemHTML = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        historyMenuItemLoad = new javax.swing.JMenuItem();
        historyMenuItemSave = new javax.swing.JMenuItem();
        frameButtonGroup = new javax.swing.ButtonGroup();
        lafButtonGroup = new javax.swing.ButtonGroup();
        langButtonGroup = new javax.swing.ButtonGroup();
        converterPanel = new javax.swing.JPanel();
        mainPanel = new javax.swing.JPanel();
        headerPanel = new javax.swing.JPanel();
        categoryBox = new javax.swing.JComboBox();
        conversionLabel = new javax.swing.JLabel();
        STLabel = new javax.swing.JLabel();
        swapButton = new javax.swing.JButton();
        filterLabel = new javax.swing.JLabel();
        filterLabel.setVisible(false);
        sourcePanel = new javax.swing.JPanel();
        sourceHeaderPanel = new javax.swing.JPanel();
        sourceBox = new javax.swing.JComboBox();
        sourceScrollPane = new javax.swing.JScrollPane();
        sourceTextArea = new javax.swing.JTextArea();
        findSourceButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        sourceBackButton = new javax.swing.JButton();
        sourceNextButton = new javax.swing.JButton();
        sourceButtonPanel = new javax.swing.JPanel();
        sourceClearButton = new javax.swing.JButton();
        sourceMinusButton = new javax.swing.JButton();
        sourcePlusButton = new javax.swing.JButton();
        sourceCopyButton = new javax.swing.JButton();
        sourcePasteButton = new javax.swing.JButton();
        targetPanel = new javax.swing.JPanel();
        targetHeaderPanel = new javax.swing.JPanel();
        targetBox = new javax.swing.JComboBox();
        targetScrollPane = new javax.swing.JScrollPane();
        targetTextArea = new javax.swing.JTextArea();
        jPanel2 = new javax.swing.JPanel();
        backButton = new javax.swing.JButton();
        nextButton = new javax.swing.JButton();
        findTargetButton = new javax.swing.JButton();
        targetButtonPanel = new javax.swing.JPanel();
        targetCopyButton = new javax.swing.JButton();
        cardPanel = new javax.swing.JPanel();
        sciPanel = new javax.swing.JPanel();
        precisionLabel = new javax.swing.JLabel();
        precisionTextField = new javax.swing.JTextField();
        sciCheckBox = new javax.swing.JCheckBox();
        soundPanel = new javax.swing.JPanel();
        playButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        calPanel = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        colPanel = new javax.swing.JPanel();
        colorButton = new javax.swing.JButton();
        recordPanel = new javax.swing.JPanel();
        historyPane = new javax.swing.JScrollPane();
        historyTextArea = new javax.swing.JTextArea();
        historyButtonPanel = new javax.swing.JPanel();
        historyNewButton = new javax.swing.JButton();
        noticesButton = new javax.swing.JButton();
        historyClearButton = new javax.swing.JButton();
        historyCopyButton = new javax.swing.JButton();
        historyPasteButton = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        bottomPanel = new javax.swing.JPanel();
        messageTextField = new javax.swing.JTextField();
        filterCatIndicatorLabel = new javax.swing.JLabel();
        filterCatIndicatorLabel.setVisible(false);
        ncMenuBar = new javax.swing.JMenuBar();
        programMenu = new javax.swing.JMenu();
        countUnitsMenuItem = new javax.swing.JMenuItem();
        updateExchangeRatesMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        tablesMenuItem = new javax.swing.JMenuItem();
        spellMenuItem = new javax.swing.JMenuItem();
        jSeparator19 = new javax.swing.JSeparator();
        restartItem = new javax.swing.JMenuItem();
        exitItem = new javax.swing.JMenuItem();
        sourceMenu = new javax.swing.JMenu();
        sourceMenuItemUndo = new javax.swing.JMenuItem();
        sourceMenuItemRedo = new javax.swing.JMenuItem();
        jSeparator9 = new javax.swing.JSeparator();
        sourceMenuItemClear = new javax.swing.JMenuItem();
        sourceMenuItemCopy = new javax.swing.JMenuItem();
        sourceMenuItemPaste = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        sourceMenuItemPlus = new javax.swing.JMenuItem();
        sourceMenuItemMinus = new javax.swing.JMenuItem();
        sourceMenuItemDefaultValue = new javax.swing.JMenuItem();
        jSeparator25 = new javax.swing.JSeparator();
        findSourceUnitMenuItem = new javax.swing.JMenuItem();
        sourceMenuItemDefault = new javax.swing.JMenuItem();
        targetMenu = new javax.swing.JMenu();
        targetMenuItemCopy = new javax.swing.JMenuItem();
        targetMenuItemFigures = new javax.swing.JMenuItem();
        jSeparator10 = new javax.swing.JSeparator();
        targetMenuItemFindUnit = new javax.swing.JMenuItem();
        targetMenuItemDefault = new javax.swing.JMenuItem();
        jSeparator13 = new javax.swing.JSeparator();
        targetMenuItemPlay = new javax.swing.JMenuItem();
        targetMenuItemStopPlaying = new javax.swing.JMenuItem();
        recordMenu = new javax.swing.JMenu();
        undoMenuItem = new javax.swing.JMenuItem();
        redoMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        recordMenuItemClear = new javax.swing.JMenuItem();
        recordMenuItemCopy = new javax.swing.JMenuItem();
        recordMenuItemPaste = new javax.swing.JMenuItem();
        jSeparator22 = new javax.swing.JSeparator();
        recordMenuItemNew = new javax.swing.JMenuItem();
        recordMenuItemMakeList = new javax.swing.JMenuItem();
        recordMenuItemFormat = new javax.swing.JMenuItem();
        recordMenuItemFontsize = new javax.swing.JMenuItem();
        htmlMenuItem = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        loadMenuItem = new javax.swing.JMenuItem();
        saveMenuItem = new javax.swing.JMenuItem();
        navigationMenu = new javax.swing.JMenu();
        favoritesMenu = new javax.swing.JMenu();
        addFavoriteMenuItem = new javax.swing.JMenuItem();
        manageFavoritesMenuItem = new javax.swing.JMenuItem();
        jSeparator17 = new javax.swing.JSeparator();
        cancelFavoritesMenuItem = new javax.swing.JMenuItem();
        groupsMenu = new javax.swing.JMenu();
        deactivateFilterCategoriesMenuItem = new javax.swing.JMenuItem();
        jSeparator23 = new javax.swing.JSeparator();
        cancelGroupsMenuItem = new javax.swing.JMenuItem();
        searchMenu = new javax.swing.JMenu();
        searchSourceMenuItem = new javax.swing.JMenuItem();
        searchTargetMenuItem = new javax.swing.JMenuItem();
        jSeparator21 = new javax.swing.JSeparator();
        searchCategoriesMenuItem = new javax.swing.JMenuItem();
        programMenuItemSwap = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        sourceMenuItemColor = new javax.swing.JMenuItem();
        sourceMenuItemCalendar = new javax.swing.JMenuItem();
        configMenu = new javax.swing.JMenu();
        frameMenu = new javax.swing.JMenu();
        miserlyRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        economicalRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        standardRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        luxuryRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        maximumRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        LaFMenu = new javax.swing.JMenu();
        standardMenu = new javax.swing.JMenu();
        aquaRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        gtkRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        oceanRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        swingRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        motifRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        windowsRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        macRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        jSeparator12 = new javax.swing.JSeparator();
        systemRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        themesMenu = new javax.swing.JMenu();
        kunststoffMenu = new javax.swing.JMenu();
        metouiaMenu = new javax.swing.JMenu();
        plasticMenuItem = new javax.swing.JMenuItem();
        squarenessMenuItem = new javax.swing.JMenuItem();
        languageMenu = new javax.swing.JMenu();
        enRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        usRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        deRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        frRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        ptRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        esRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        heRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        elRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        jSeparator18 = new javax.swing.JSeparator();
        categoriesMenu = new javax.swing.JMenu();
        editCategoriesMenuItem = new javax.swing.JMenuItem();
        reloadCategoriesMenuItem = new javax.swing.JMenuItem();
        unitsMenu = new javax.swing.JMenu();
        editUnitsMenuItem = new javax.swing.JMenuItem();
        reloadUnitsItem = new javax.swing.JMenuItem();
        jSeparator15 = new javax.swing.JSeparator();
        propsMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        homepageMenuItem = new javax.swing.JMenuItem();
        supportMenuItem = new javax.swing.JMenuItem();
        jSeparator20 = new javax.swing.JSeparator();
        helpIndexMenuItem = new javax.swing.JMenuItem();
        helpMenuItemAbout = new javax.swing.JMenuItem();

        sourcePopupMenuItemUndo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Undo16.gif")));
        sourcePopupMenuItemUndo.setText(rb.getString("GUI.General.Undo"));
        sourcePopupMenuItemUndo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemUndoActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemUndo);

        sourcePopupMenuItemRedo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Redo16.gif")));
        sourcePopupMenuItemRedo.setText(rb.getString("GUI.General.Redo"));
        sourcePopupMenuItemRedo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemRedoActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemRedo);

        sourcePopupMenu.add(jSeparator26);

        sourcePopupMenuItemClear.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        sourcePopupMenuItemClear.setText(rb.getString("GUI.General.Clear"));
        sourcePopupMenuItemClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemClearActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemClear);

        sourcePopupMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        sourcePopupMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        sourcePopupMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemCopyActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemCopy);

        sourcePopupMenuItemPaste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        sourcePopupMenuItemPaste.setText(rb.getString("GUI.General.Paste"));
        sourcePopupMenuItemPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemPasteActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemPaste);

        sourcePopupMenu.add(jSeparator2);

        sourcePopupMenuItemPlus.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/up.png")));
        sourcePopupMenuItemPlus.setText("+1");
        sourcePopupMenuItemPlus.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemPlusActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemPlus);

        sourcePopupMenuItemMinus.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/down.png")));
        sourcePopupMenuItemMinus.setText("-1");
        sourcePopupMenuItemMinus.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemMinusActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemMinus);

        sourcePopupMenuDefaultValue.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        sourcePopupMenuDefaultValue.setText(rb.getString("GUI.Menu.Source.DefaultValue"));
        sourcePopupMenuDefaultValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuDefaultValueActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuDefaultValue);

        sourcePopupMenu.add(jSeparator16);

        sourcePopupMenuItemFindUnit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        sourcePopupMenuItemFindUnit.setText(rb.getString("GUI.Menu.Source.FindUnit"));
        sourcePopupMenuItemFindUnit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemFindUnitActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemFindUnit);

        sourcePopupMenuItemDefault.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        sourcePopupMenuItemDefault.setText(rb.getString("GUI.Menu.Source.Default"));
        sourcePopupMenuItemDefault.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePopupMenuItemDefaultActionPerformed(evt);
            }
        });

        sourcePopupMenu.add(sourcePopupMenuItemDefault);

        targetPopupMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        targetPopupMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        targetPopupMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetPopupMenuItemCopyActionPerformed(evt);
            }
        });

        targetPopupMenu.add(targetPopupMenuItemCopy);

        targetPopupMenu.add(jSeparator11);

        targetPopupMenuItemFindUnit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        targetPopupMenuItemFindUnit.setText(rb.getString("GUI.Menu.Target.FindUnit"));
        targetPopupMenuItemFindUnit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetPopupMenuItemFindUnitActionPerformed(evt);
            }
        });

        targetPopupMenu.add(targetPopupMenuItemFindUnit);

        targetPopupMenuItemDefault.setText(rb.getString("GUI.Menu.Target.Default"));
        targetPopupMenuItemDefault.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        targetPopupMenuItemDefault.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetPopupMenuItemDefaultActionPerformed(evt);
            }
        });

        targetPopupMenu.add(targetPopupMenuItemDefault);

        targetPopupMenu.add(jSeparator14);

        targetPopupMenuItemPlay.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Volume16.gif")));
        targetPopupMenuItemPlay.setText(rb.getString("GUI.Menu.Target.Play"));
        targetPopupMenuItemPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetPopupMenuItemPlayActionPerformed(evt);
            }
        });

        targetPopupMenu.add(targetPopupMenuItemPlay);

        targetPopupMenuItemStopPlaying.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/StopVolume16.gif")));
        targetPopupMenuItemStopPlaying.setText(rb.getString("GUI.Menu.Target.StopPlaying"));
        targetPopupMenuItemStopPlaying.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetPopupMenuItemStopPlayingActionPerformed(evt);
            }
        });

        targetPopupMenu.add(targetPopupMenuItemStopPlaying);

        historyMenuItemUndo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Undo16.gif")));
        historyMenuItemUndo.setText(rb.getString("GUI.General.Undo"));
        historyMenuItemUndo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemUndoActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemUndo);

        historyMenuItemRedo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Redo16.gif")));
        historyMenuItemRedo.setText(rb.getString("GUI.General.Redo"));
        historyMenuItemRedo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemRedoActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemRedo);

        historyPopupMenu.add(jSeparator24);

        historyMenuItemClear.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        historyMenuItemClear.setText(rb.getString("GUI.General.Clear"));
        historyMenuItemClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemClearActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemClear);

        historyMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        historyMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        historyMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemCopyActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemCopy);

        historyMenuItemPaste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        historyMenuItemPaste.setText(rb.getString("GUI.General.Paste"));
        historyMenuItemPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemPasteActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemPaste);

        historyPopupMenu.add(jSeparator3);

        historyPopupMenuItemNew.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Notice16.gif")));
        historyPopupMenuItemNew.setText(rb.getString("GUI.General.Notice"));
        historyPopupMenuItemNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPopupMenuItemNewActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyPopupMenuItemNew);

        historyPopupMenuItemMakeList.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/History16.gif")));
        historyPopupMenuItemMakeList.setText(rb.getString("GUI.General.MakeList"));
        historyPopupMenuItemMakeList.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPopupMenuItemMakeListActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyPopupMenuItemMakeList);

        historyPopupMenuenuItemFormat.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Properties16.gif")));
        historyPopupMenuenuItemFormat.setText(rb.getString("GUI.Menu.Records.Format"));
        historyPopupMenuenuItemFormat.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPopupMenuenuItemFormatActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyPopupMenuenuItemFormat);

        historyMenuFontSize.setText(rb.getString("GUI.Menu.Records.Fontsize"));
        historyMenuFontSize.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/fontsize16.png")));
        historyMenuFontSize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuFontSizeActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuFontSize);

        historyPopupMenuItemHTML.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/editor.gif")));
        historyPopupMenuItemHTML.setText(rb.getString("GUI.Menu.Records.AddHTML"));
        historyPopupMenuItemHTML.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPopupMenuItemHTMLActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyPopupMenuItemHTML);

        historyPopupMenu.add(jSeparator8);

        historyMenuItemLoad.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Open16.gif")));
        historyMenuItemLoad.setText(rb.getString("GUI.General.Load"));
        historyMenuItemLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemLoadActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemLoad);

        historyMenuItemSave.setText(rb.getString("GUI.General.Save"));
        historyMenuItemSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Save16.gif")));
        historyMenuItemSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemSaveActionPerformed(evt);
            }
        });

        historyPopupMenu.add(historyMenuItemSave);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(PROGRAM+" [http://www.jonelo.de/java/nc/index.html]");
        setIconImage(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/chameleon.png")).getImage());
        setName("NumericalChameleon");
        getAccessibleContext().setAccessibleName("NC Frame");
        getAccessibleContext().setAccessibleDescription("This is the NC");
        converterPanel.setLayout(new java.awt.GridBagLayout());

        mainPanel.setLayout(new java.awt.GridBagLayout());

        headerPanel.setLayout(new java.awt.GridBagLayout());

        headerPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), rb.getString("GUI.Menu.Main")));
        categoryBox.setMaximumRowCount(12);
        categoryBox.setToolTipText(rb.getString("GUI.Area.Main.Conversion.ToolTip"));
        categoryBox.setOpaque(false);
        categoryBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                categoryBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        headerPanel.add(categoryBox, gridBagConstraints);

        conversionLabel.setText(rb.getString("GUI.Area.Main.Conversion.Label"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 5);
        headerPanel.add(conversionLabel, gridBagConstraints);

        STLabel.setText(rb.getString("GUI.Area.Main.Switch.Label"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 5);
        headerPanel.add(STLabel, gridBagConstraints);

        swapButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/switch.png")));
        swapButton.setMnemonic('w');
        swapButton.setText(rb.getString("GUI.Area.Main.Switch.Button"));
        swapButton.setToolTipText(rb.getString("GUI.Menu.Main.Switch"));
        swapButton.setMaximumSize(new java.awt.Dimension(32767, 32767));
        swapButton.setMinimumSize(null);
        swapButton.setPreferredSize(null);
        swapButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                swapButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        headerPanel.add(swapButton, gridBagConstraints);

        filterLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/filter24.png")));
        filterLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                filterLabelMouseClicked(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 5);
        headerPanel.add(filterLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        mainPanel.add(headerPanel, gridBagConstraints);

        sourcePanel.setLayout(new java.awt.GridBagLayout());

        sourcePanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), rb.getString("GUI.Menu.Source")));
        sourceHeaderPanel.setLayout(new java.awt.GridBagLayout());

        sourceBox.setMaximumRowCount(20);
        sourceBox.setToolTipText(rb.getString("GUI.Area.Source.ComboBox.ToolTip"));
        sourceBox.setMinimumSize(new java.awt.Dimension(260, 25));
        sourceBox.setPreferredSize(new java.awt.Dimension(260, 25));
        sourceBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        sourceHeaderPanel.add(sourceBox, gridBagConstraints);

        sourceScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        sourceScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        sourceScrollPane.setMinimumSize(new java.awt.Dimension(300, 40));
        sourceScrollPane.setPreferredSize(new java.awt.Dimension(300, 40));
        sourceTextArea.setFont(new java.awt.Font("Dialog", 0, 14));
        sourceTextArea.setToolTipText(rb.getString("GUI.Area.Source.TextField.ToolTip"));
        sourceTextArea.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                sourceTextAreaKeyReleased(evt);
            }
        });
        sourceTextArea.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                sourceTextAreaMouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                sourceTextAreaMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                sourceTextAreaMouseReleased(evt);
            }
        });

        sourceScrollPane.setViewportView(sourceTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        sourceHeaderPanel.add(sourceScrollPane, gridBagConstraints);

        findSourceButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        findSourceButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        findSourceButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                findSourceButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        sourceHeaderPanel.add(findSourceButton, gridBagConstraints);

        jPanel1.setLayout(new java.awt.GridLayout(2, 0));

        sourceBackButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/up3.gif")));
        sourceBackButton.setToolTipText(rb.getString("GUI.Area.Back.Button"));
        sourceBackButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
        sourceBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceBackButtonActionPerformed(evt);
            }
        });

        jPanel1.add(sourceBackButton);

        sourceNextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/down3.gif")));
        sourceNextButton.setToolTipText(rb.getString("GUI.Area.Next.Button"));
        sourceNextButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
        sourceNextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceNextButtonActionPerformed(evt);
            }
        });

        jPanel1.add(sourceNextButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        sourceHeaderPanel.add(jPanel1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 5);
        sourcePanel.add(sourceHeaderPanel, gridBagConstraints);

        sourceButtonPanel.setLayout(new java.awt.GridBagLayout());

        sourceClearButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        sourceClearButton.setToolTipText(rb.getString("GUI.Area.Source.Cut.ToolTip"));
        sourceClearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceClearButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        sourceButtonPanel.add(sourceClearButton, gridBagConstraints);

        sourceMinusButton.setMnemonic('-');
        sourceMinusButton.setText("-1");
        sourceMinusButton.setToolTipText(rb.getString("GUI.Area.Source.Sub1.ToolTip"));
        sourceMinusButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMinusButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        sourceButtonPanel.add(sourceMinusButton, gridBagConstraints);

        sourcePlusButton.setMnemonic('1');
        sourcePlusButton.setText("+1");
        sourcePlusButton.setToolTipText(rb.getString("GUI.Area.Source.Add1.ToolTip"));
        sourcePlusButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePlusButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        sourceButtonPanel.add(sourcePlusButton, gridBagConstraints);

        sourceCopyButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        sourceCopyButton.setToolTipText(rb.getString("GUI.Area.Source.Copy.ToolTip"));
        sourceCopyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceCopyButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        sourceButtonPanel.add(sourceCopyButton, gridBagConstraints);

        sourcePasteButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        sourcePasteButton.setToolTipText(rb.getString("GUI.Area.Source.Paste.ToolTip"));
        sourcePasteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourcePasteButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        sourceButtonPanel.add(sourcePasteButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        sourcePanel.add(sourceButtonPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        mainPanel.add(sourcePanel, gridBagConstraints);

        targetPanel.setLayout(new java.awt.GridBagLayout());

        targetPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), rb.getString("GUI.Menu.Target")));
        targetHeaderPanel.setLayout(new java.awt.GridBagLayout());

        targetBox.setMaximumRowCount(20);
        targetBox.setToolTipText(rb.getString("GUI.Area.Target.ComboBox.ToolTip"));
        targetBox.setMinimumSize(new java.awt.Dimension(260, 25));
        targetBox.setPreferredSize(new java.awt.Dimension(260, 25));
        targetBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        targetHeaderPanel.add(targetBox, gridBagConstraints);

        targetScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        targetScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        targetScrollPane.setMinimumSize(new java.awt.Dimension(300, 40));
        targetScrollPane.setPreferredSize(new java.awt.Dimension(300, 40));
        targetTextArea.setBackground(new Color(230,230,230));
        targetTextArea.setEditable(false);
        targetTextArea.setFont(new java.awt.Font("Dialog", 0, 14));
        targetTextArea.setToolTipText(rb.getString("GUI.Area.Target.TextField.ToolTip"));
        targetTextArea.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                targetTextAreaMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                targetTextAreaMouseReleased(evt);
            }
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                targetTextAreaMouseClicked(evt);
            }
        });

        targetScrollPane.setViewportView(targetTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        targetHeaderPanel.add(targetScrollPane, gridBagConstraints);

        jPanel2.setLayout(new java.awt.GridLayout(2, 0));

        backButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/up3.gif")));
        backButton.setToolTipText(rb.getString("GUI.Area.Back.Button"));
        backButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
        backButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backButtonActionPerformed(evt);
            }
        });

        jPanel2.add(backButton);

        nextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/down3.gif")));
        nextButton.setToolTipText(rb.getString("GUI.Area.Next.Button"));
        nextButton.setMargin(new java.awt.Insets(1, 1, 1, 1));
        nextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextButtonActionPerformed(evt);
            }
        });

        jPanel2.add(nextButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        targetHeaderPanel.add(jPanel2, gridBagConstraints);

        findTargetButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        findTargetButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        findTargetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                findTargetButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        targetHeaderPanel.add(findTargetButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 5);
        targetPanel.add(targetHeaderPanel, gridBagConstraints);

        targetButtonPanel.setLayout(new java.awt.GridBagLayout());

        targetCopyButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        targetCopyButton.setToolTipText(rb.getString("GUI.Area.Source.Copy.ToolTip"));
        targetCopyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetCopyButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        targetButtonPanel.add(targetCopyButton, gridBagConstraints);

        cardPanel.setLayout(new java.awt.CardLayout());

        cardLayout = new java.awt.CardLayout();
        cardPanel.setLayout(cardLayout);
        sciPanel.setLayout(new java.awt.GridBagLayout());

        precisionLabel.setText(rb.getString("GUI.Menu.Target.SignificantFigures"));
        precisionLabel.setToolTipText(rb.getString("GUI.Area.Target.SignificantFigures.ToolTip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 5);
        sciPanel.add(precisionLabel, gridBagConstraints);

        precisionTextField.setText("2");
        precisionTextField.setToolTipText(rb.getString("GUI.Area.Target.SignificantFigures.ToolTip"));
        precisionTextField.setMinimumSize(new java.awt.Dimension(30, 20));
        precisionTextField.setPreferredSize(new java.awt.Dimension(30, 20));
        precisionTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                precisionTextFieldKeyReleased(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        sciPanel.add(precisionTextField, gridBagConstraints);

        sciCheckBox.setText("SCI");
        sciCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sciCheckBoxActionPerformed(evt);
            }
        });
        sciCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                sciCheckBoxItemStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        sciPanel.add(sciCheckBox, gridBagConstraints);

        cardPanel.add(sciPanel, "sciCard");

        soundPanel.setLayout(new java.awt.GridBagLayout());

        playButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Volume16.gif")));
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        soundPanel.add(playButton, gridBagConstraints);

        stopButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/StopVolume16.gif")));
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        soundPanel.add(stopButton, new java.awt.GridBagConstraints());

        cardPanel.add(soundPanel, "soundCard");

        calPanel.setLayout(new java.awt.GridBagLayout());

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/calendar16.png")));
        jButton1.setText(rb.getString("GUI.General.DatePicker"));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        calPanel.add(jButton1, new java.awt.GridBagConstraints());

        cardPanel.add(calPanel, "calCard");

        colPanel.setLayout(new java.awt.GridBagLayout());

        colorButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/color.png")));
        colorButton.setText(rb.getString("GUI.General.ColorPicker"));
        colorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                colorButtonActionPerformed(evt);
            }
        });

        colPanel.add(colorButton, new java.awt.GridBagConstraints());

        cardPanel.add(colPanel, "colCard");

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        targetButtonPanel.add(cardPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        targetPanel.add(targetButtonPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        mainPanel.add(targetPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        converterPanel.add(mainPanel, gridBagConstraints);

        recordPanel.setLayout(new java.awt.GridBagLayout());

        recordPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), rb.getString("GUI.Menu.Records")));
        recordPanel.setMinimumSize(new java.awt.Dimension(642, 300));
        recordPanel.setPreferredSize(new java.awt.Dimension(642, 300));
        // pre init for Mac OS X support (Apple HI guidelines)
        if (System.getProperty("os.name").equals(MAC_OS_X))
        {
            historyPane.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
            historyPane.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        }

        historyTextArea.setRows(10);
        historyTextArea.setToolTipText(rb.getString("GUI.Area.Records.TextArea.ToolTip"));
        historyTextArea.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                historyAreaMouseClicked(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                historyAreaMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                historyAreaMouseReleased(evt);
            }
        });

        historyPane.setViewportView(historyTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        recordPanel.add(historyPane, gridBagConstraints);

        historyButtonPanel.setLayout(new java.awt.GridBagLayout());

        historyNewButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Notice16.gif")));
        historyNewButton.setText(rb.getString("GUI.General.Notice"));
        historyNewButton.setToolTipText(rb.getString("GUI.Area.Records.Notice.ToolTip"));
        historyNewButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyNewButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(historyNewButton, new java.awt.GridBagConstraints());

        noticesButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/History16.gif")));
        noticesButton.setText(rb.getString("GUI.General.MakeList.Button"));
        noticesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noticesButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(noticesButton, new java.awt.GridBagConstraints());

        historyClearButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        historyClearButton.setText(rb.getString("GUI.General.Clear"));
        historyClearButton.setToolTipText(rb.getString("GUI.Area.Records.Cut.ToolTip"));
        historyClearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyClearButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(historyClearButton, new java.awt.GridBagConstraints());

        historyCopyButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        historyCopyButton.setText(rb.getString("GUI.General.Copy"));
        historyCopyButton.setToolTipText(rb.getString("GUI.Area.Records.Copy.ToolTip"));
        historyCopyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyCopyButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(historyCopyButton, new java.awt.GridBagConstraints());

        historyPasteButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        historyPasteButton.setText(rb.getString("GUI.General.Paste"));
        historyPasteButton.setToolTipText(rb.getString("GUI.Area.Records.Paste.ToolTip"));
        historyPasteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyPasteButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(historyPasteButton, new java.awt.GridBagConstraints());

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Save16.gif")));
        saveButton.setText(rb.getString("GUI.General.Save"));
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        historyButtonPanel.add(saveButton, new java.awt.GridBagConstraints());

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        recordPanel.add(historyButtonPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        converterPanel.add(recordPanel, gridBagConstraints);

        bottomPanel.setLayout(new java.awt.GridBagLayout());

        messageTextField.setEditable(false);
        messageTextField.setToolTipText(rb.getString("GUI.Area.Message.ToolTip"));
        messageTextField.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0)));
        messageTextField.setMargin(new java.awt.Insets(2, 2, 2, 2));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 5);
        bottomPanel.add(messageTextField, gridBagConstraints);

        filterCatIndicatorLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/filter16.png")));
        filterCatIndicatorLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                filterCatIndicatorLabelMouseClicked(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 5);
        bottomPanel.add(filterCatIndicatorLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 5, 5, 5);
        converterPanel.add(bottomPanel, gridBagConstraints);

        getContentPane().add(converterPanel, java.awt.BorderLayout.CENTER);

        programMenu.setText(rb.getString("GUI.Menu.Main"));
        countUnitsMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/help.gif")));
        countUnitsMenuItem.setText(rb.getString("GUI.Menu.Count"));
        countUnitsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                countUnitsMenuItemActionPerformed(evt);
            }
        });

        programMenu.add(countUnitsMenuItem);

        updateExchangeRatesMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Import16_anim.gif")));
        updateExchangeRatesMenuItem.setText(rb.getString("GUI.Menu.Main.UpdateExchangeRates"));
        updateExchangeRatesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateExchangeRatesMenuItemActionPerformed(evt);
            }
        });

        programMenu.add(updateExchangeRatesMenuItem);

        programMenu.add(jSeparator4);

        tablesMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/table16.png")));
        tablesMenuItem.setText(rb.getString("GUI.Menu.Service.Tables"));
        tablesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tablesMenuItemActionPerformed(evt);
            }
        });

        programMenu.add(tablesMenuItem);

        spellMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/abc16.png")));
        spellMenuItem.setText(rb.getString("GUI.Menu.Service.Spell"));
        spellMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                spellMenuItemActionPerformed(evt);
            }
        });

        programMenu.add(spellMenuItem);

        programMenu.add(jSeparator19);

        restartItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/restart.png")));
        restartItem.setText(rb.getString("GUI.Menu.Main.Restart"));
        restartItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                restartItemActionPerformed(evt);
            }
        });

        programMenu.add(restartItem);

        exitItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/exit.png")));
        exitItem.setText(rb.getString("GUI.Menu.Main.Exit"));
        exitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitItemActionPerformed(evt);
            }
        });

        programMenu.add(exitItem);

        ncMenuBar.add(programMenu);

        sourceMenu.setText(rb.getString("GUI.Menu.Source"));
        sourceMenuItemUndo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Undo16.gif")));
        sourceMenuItemUndo.setText(rb.getString("GUI.General.Undo"));
        sourceMenuItemUndo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemUndoActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemUndo);

        sourceMenuItemRedo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Redo16.gif")));
        sourceMenuItemRedo.setText(rb.getString("GUI.General.Redo"));
        sourceMenuItemRedo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemRedoActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemRedo);

        sourceMenu.add(jSeparator9);

        sourceMenuItemClear.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        sourceMenuItemClear.setText(rb.getString("GUI.General.Clear"));
        sourceMenuItemClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemClearActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemClear);

        sourceMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        sourceMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        sourceMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemCopyActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemCopy);

        sourceMenuItemPaste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        sourceMenuItemPaste.setText(rb.getString("GUI.General.Paste"));
        sourceMenuItemPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemPasteActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemPaste);

        sourceMenu.add(jSeparator7);

        sourceMenuItemPlus.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/up.png")));
        sourceMenuItemPlus.setText("+1");
        sourceMenuItemPlus.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemPlusActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemPlus);

        sourceMenuItemMinus.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/down.png")));
        sourceMenuItemMinus.setText("-1");
        sourceMenuItemMinus.setPreferredSize(new java.awt.Dimension(69, 21));
        sourceMenuItemMinus.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemMinusActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemMinus);

        sourceMenuItemDefaultValue.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        sourceMenuItemDefaultValue.setText(rb.getString("GUI.Menu.Source.DefaultValue"));
        sourceMenuItemDefaultValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemDefaultValueActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemDefaultValue);

        sourceMenu.add(jSeparator25);

        findSourceUnitMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        findSourceUnitMenuItem.setText(rb.getString("GUI.Menu.Source.FindUnit"));
        findSourceUnitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                findSourceUnitMenuItemActionPerformed(evt);
            }
        });

        sourceMenu.add(findSourceUnitMenuItem);

        sourceMenuItemDefault.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        sourceMenuItemDefault.setText(rb.getString("GUI.Menu.Source.Default"));
        sourceMenuItemDefault.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemDefaultActionPerformed(evt);
            }
        });

        sourceMenu.add(sourceMenuItemDefault);

        ncMenuBar.add(sourceMenu);

        targetMenu.setText(rb.getString("GUI.Menu.Target"));
        targetMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        targetMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        targetMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemCopyActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemCopy);

        targetMenuItemFigures.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Properties16.gif")));
        targetMenuItemFigures.setText(rb.getString("GUI.Menu.Target.SignificantFigures"));
        targetMenuItemFigures.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemFiguresActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemFigures);

        targetMenu.add(jSeparator10);

        targetMenuItemFindUnit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        targetMenuItemFindUnit.setText(rb.getString("GUI.Menu.Target.FindUnit"));
        targetMenuItemFindUnit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemFindUnitActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemFindUnit);

        targetMenuItemDefault.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Stop16.gif")));
        targetMenuItemDefault.setText(rb.getString("GUI.Menu.Target.Default"));
        targetMenuItemDefault.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemDefaultActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemDefault);

        targetMenu.add(jSeparator13);

        targetMenuItemPlay.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Volume16.gif")));
        targetMenuItemPlay.setText(rb.getString("GUI.Menu.Target.Play"));
        targetMenuItemPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemPlayActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemPlay);

        targetMenuItemStopPlaying.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/StopVolume16.gif")));
        targetMenuItemStopPlaying.setText(rb.getString("GUI.Menu.Target.StopPlaying"));
        targetMenuItemStopPlaying.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetMenuItemStopPlayingActionPerformed(evt);
            }
        });

        targetMenu.add(targetMenuItemStopPlaying);

        ncMenuBar.add(targetMenu);

        recordMenu.setText(rb.getString("GUI.Menu.Records"));
        undoMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Undo16.gif")));
        undoMenuItem.setText(rb.getString("GUI.General.Undo"));
        undoMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undoMenuItemActionPerformed(evt);
            }
        });

        recordMenu.add(undoMenuItem);

        redoMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Redo16.gif")));
        redoMenuItem.setText(rb.getString("GUI.General.Redo"));
        redoMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                redoMenuItemActionPerformed(evt);
            }
        });

        recordMenu.add(redoMenuItem);

        recordMenu.add(jSeparator1);

        recordMenuItemClear.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/cut.gif")));
        recordMenuItemClear.setText(rb.getString("GUI.General.Clear"));
        recordMenuItemClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemClearActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemClear);

        recordMenuItemCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/copy.gif")));
        recordMenuItemCopy.setText(rb.getString("GUI.General.Copy"));
        recordMenuItemCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemCopyActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemCopy);

        recordMenuItemPaste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/paste.gif")));
        recordMenuItemPaste.setText(rb.getString("GUI.General.Paste"));
        recordMenuItemPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemPasteActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemPaste);

        recordMenu.add(jSeparator22);

        recordMenuItemNew.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Notice16.gif")));
        recordMenuItemNew.setText(rb.getString("GUI.General.Notice"));
        recordMenuItemNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemNewActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemNew);

        recordMenuItemMakeList.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/History16.gif")));
        recordMenuItemMakeList.setText(rb.getString("GUI.General.MakeList"));
        recordMenuItemMakeList.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemMakeListActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemMakeList);

        recordMenuItemFormat.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Properties16.gif")));
        recordMenuItemFormat.setText(rb.getString("GUI.Menu.Records.Format"));
        recordMenuItemFormat.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemFormatActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemFormat);

        recordMenuItemFontsize.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/fontsize16.png")));
        recordMenuItemFontsize.setText(rb.getString("GUI.Menu.Records.Fontsize"));
        recordMenuItemFontsize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recordMenuItemFontsizeActionPerformed(evt);
            }
        });

        recordMenu.add(recordMenuItemFontsize);

        htmlMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/editor.gif")));
        htmlMenuItem.setText(rb.getString("GUI.Menu.Records.AddHTML"));
        htmlMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                htmlMenuItemActionPerformed(evt);
            }
        });

        recordMenu.add(htmlMenuItem);

        recordMenu.add(jSeparator6);

        loadMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Open16.gif")));
        loadMenuItem.setText(rb.getString("GUI.General.Load"));
        loadMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadMenuItemActionPerformed(evt);
            }
        });

        recordMenu.add(loadMenuItem);

        saveMenuItem.setText(rb.getString("GUI.General.Save"));
        saveMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Save16.gif")));
        saveMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveMenuItemActionPerformed(evt);
            }
        });

        recordMenu.add(saveMenuItem);

        ncMenuBar.add(recordMenu);

        navigationMenu.setText(rb.getString("GUI.Menu.Navigation"));
        favoritesMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/favorites.png")));
        favoritesMenu.setText(rb.getString("GUI.Menu.Favorites"));
        addFavoriteMenuItem.setText(rb.getString("GUI.Menu.Favorites.Add"));
        addFavoriteMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addFavoriteMenuItemActionPerformed(evt);
            }
        });

        favoritesMenu.add(addFavoriteMenuItem);

        manageFavoritesMenuItem.setText(rb.getString("GUI.Menu.Favorites.Manage"));
        manageFavoritesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                manageFavoritesMenuItemActionPerformed(evt);
            }
        });

        favoritesMenu.add(manageFavoritesMenuItem);

        favoritesMenu.add(jSeparator17);

        cancelFavoritesMenuItem.setText(rb.getString("GUI.General.Cancel"));
        cancelFavoritesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelFavoritesMenuItemActionPerformed(evt);
            }
        });

        favoritesMenu.add(cancelFavoritesMenuItem);

        navigationMenu.add(favoritesMenu);

        groupsMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/filter16.png")));
        groupsMenu.setText(rb.getString("GUI.Menu.Filter"));
        deactivateFilterCategoriesMenuItem.setText(rb.getString("GUI.Filter.NoFilter"));
        deactivateFilterCategoriesMenuItem.setEnabled(false);
        deactivateFilterCategoriesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deactivateFilterCategoriesMenuItemActionPerformed(evt);
            }
        });

        groupsMenu.add(deactivateFilterCategoriesMenuItem);

        groupsMenu.add(jSeparator23);

        cancelGroupsMenuItem.setText(rb.getString("GUI.General.Cancel"));
        groupsMenu.add(cancelGroupsMenuItem);

        navigationMenu.add(groupsMenu);

        searchMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Zoom16.gif")));
        searchMenu.setText(rb.getString("GUI.Menu.Find"));
        searchSourceMenuItem.setText(rb.getString("GUI.Menu.Find.FindSourceUnit"));
        searchSourceMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchSourceMenuItemActionPerformed(evt);
            }
        });

        searchMenu.add(searchSourceMenuItem);

        searchTargetMenuItem.setText(rb.getString("GUI.Menu.Find.FindTargetUnit"));
        searchTargetMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchTargetMenuItemActionPerformed(evt);
            }
        });

        searchMenu.add(searchTargetMenuItem);

        searchMenu.add(jSeparator21);

        searchCategoriesMenuItem.setText(rb.getString("GUI.Menu.Find.FindCategory"));
        searchCategoriesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchCategoriesMenuItemActionPerformed(evt);
            }
        });

        searchMenu.add(searchCategoriesMenuItem);

        navigationMenu.add(searchMenu);

        programMenuItemSwap.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/switch16.png")));
        programMenuItemSwap.setText(rb.getString("GUI.Menu.Main.Switch"));
        programMenuItemSwap.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                programMenuItemSwapActionPerformed(evt);
            }
        });

        navigationMenu.add(programMenuItemSwap);

        navigationMenu.add(jSeparator5);

        sourceMenuItemColor.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/color.png")));
        sourceMenuItemColor.setText(rb.getString("GUI.General.ColorPicker"));
        sourceMenuItemColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemColorActionPerformed(evt);
            }
        });

        navigationMenu.add(sourceMenuItemColor);

        sourceMenuItemCalendar.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/calendar16.png")));
        sourceMenuItemCalendar.setText(rb.getString("GUI.General.DatePicker"));
        sourceMenuItemCalendar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sourceMenuItemCalendarActionPerformed(evt);
            }
        });

        navigationMenu.add(sourceMenuItemCalendar);

        ncMenuBar.add(navigationMenu);

        configMenu.setText(rb.getString("GUI.Menu.Config"));
        frameMenu.setText(rb.getString("GUI.Menu.Frame"));
        miserlyRadioButtonMenuItem.setText(rb.getString("GUI.Menu.Frame.Miserly"));
        miserlyRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                miserlyRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        frameMenu.add(miserlyRadioButtonMenuItem);

        economicalRadioButtonMenuItem.setText(rb.getString("GUI.Menu.Frame.Economical"));
        economicalRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                economicalRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        frameMenu.add(economicalRadioButtonMenuItem);

        standardRadioButtonMenuItem.setText(rb.getString("GUI.Menu.Frame.Standard"));
        standardRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                standardRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        frameMenu.add(standardRadioButtonMenuItem);

        luxuryRadioButtonMenuItem.setSelected(true);
        luxuryRadioButtonMenuItem.setText(rb.getString("GUI.Menu.Frame.Luxury"));
        luxuryRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                luxuryRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        frameMenu.add(luxuryRadioButtonMenuItem);

        maximumRadioButtonMenuItem.setText(rb.getString("GUI.Menu.Frame.Maximum"));
        maximumRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                maximumRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        frameMenu.add(maximumRadioButtonMenuItem);

        configMenu.add(frameMenu);

        LaFMenu.setText(rb.getString("GUI.Menu.Config.LookAndFeel"));
        standardMenu.setText("Standards");
        aquaRadioButtonMenuItem.setText("Aqua (Mac OS X)");
        aquaRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                aquaRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(aquaRadioButtonMenuItem);

        gtkRadioButtonMenuItem.setText("GTK+ 2.0");
        gtkRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                gtkRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(gtkRadioButtonMenuItem);

        oceanRadioButtonMenuItem.setText("Ocean (J2SE 1.5+)");
        oceanRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                oceanRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(oceanRadioButtonMenuItem);

        swingRadioButtonMenuItem.setText("Metal (Java)");
        swingRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                swingRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(swingRadioButtonMenuItem);

        motifRadioButtonMenuItem.setText("Motif");
        motifRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                motifRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(motifRadioButtonMenuItem);

        windowsRadioButtonMenuItem.setText("Windows");
        windowsRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                windowsRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(windowsRadioButtonMenuItem);

        macRadioButtonMenuItem.setText("Mac");
        macRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                macRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(macRadioButtonMenuItem);

        standardMenu.add(jSeparator12);

        systemRadioButtonMenuItem.setText("System");
        systemRadioButtonMenuItem.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                systemRadioButtonMenuItemItemStateChanged(evt);
            }
        });

        standardMenu.add(systemRadioButtonMenuItem);

        LaFMenu.add(standardMenu);

        themesMenu.setText("Metal-Themes");
        LaFMenu.add(themesMenu);

        kunststoffMenu.setText("Kunststoff-Themes");
        LaFMenu.add(kunststoffMenu);

        metouiaMenu.setText("Metouia-Themes");
        LaFMenu.add(metouiaMenu);

        plasticMenuItem.setText("Plastic");
        plasticMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                plasticMenuItemActionPerformed(evt);
            }
        });

        LaFMenu.add(plasticMenuItem);

        squarenessMenuItem.setText("Squareness");
        squarenessMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                squarenessMenuItemActionPerformed(evt);
            }
        });

        LaFMenu.add(squarenessMenuItem);

        configMenu.add(LaFMenu);

        languageMenu.setText(rb.getString("GUI.Menu.Language"));
        enRadioButtonMenuItem.setText(rb.getString("Language.en"));
        enRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/gb.png")));
        enRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                enRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(enRadioButtonMenuItem);

        usRadioButtonMenuItem.setText(rb.getString("Language.us"));
        usRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/us.png")));
        usRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                usRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(usRadioButtonMenuItem);

        deRadioButtonMenuItem.setText(rb.getString("Language.de"));
        deRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/de.png")));
        deRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(deRadioButtonMenuItem);

        frRadioButtonMenuItem.setText(rb.getString("Language.fr"));
        frRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/fr.png")));
        frRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                frRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(frRadioButtonMenuItem);

        ptRadioButtonMenuItem.setText(rb.getString("Language.br"));
        ptRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/br.png")));
        ptRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ptRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(ptRadioButtonMenuItem);

        esRadioButtonMenuItem.setText(rb.getString("Language.mx"));
        esRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/mx.png")));
        esRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                esRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(esRadioButtonMenuItem);

        heRadioButtonMenuItem.setText(rb.getString("Language.il"));
        heRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/il.png")));
        heRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                heRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(heRadioButtonMenuItem);

        elRadioButtonMenuItem.setText(rb.getString("Language.gr"));
        elRadioButtonMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/data/images/flags/gr.png")));
        elRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                elRadioButtonMenuItemActionPerformed(evt);
            }
        });

        languageMenu.add(elRadioButtonMenuItem);

        configMenu.add(languageMenu);

        configMenu.add(jSeparator18);

        categoriesMenu.setText(rb.getString("GUI.Config.Categories"));
        editCategoriesMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/editor.gif")));
        editCategoriesMenuItem.setText(rb.getString("GUI.Config.Category.Edit"));
        editCategoriesMenuItem.setEnabled(false);
        categoriesMenu.add(editCategoriesMenuItem);

        reloadCategoriesMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Refresh16.gif")));
        reloadCategoriesMenuItem.setText(rb.getString("GUI.Config.Category.Reload"));
        reloadCategoriesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                reloadCategoriesMenuItemActionPerformed(evt);
            }
        });

        categoriesMenu.add(reloadCategoriesMenuItem);

        configMenu.add(categoriesMenu);

        unitsMenu.setText(rb.getString("GUI.Config.Units"));
        editUnitsMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/editor.gif")));
        editUnitsMenuItem.setText(rb.getString("GUI.Config.Units.Edit"));
        editUnitsMenuItem.setEnabled(false);
        unitsMenu.add(editUnitsMenuItem);

        reloadUnitsItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/Refresh16.gif")));
        reloadUnitsItem.setText(rb.getString("GUI.Config.Units.Reload"));
        reloadUnitsItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                reloadUnitsItemActionPerformed(evt);
            }
        });

        unitsMenu.add(reloadUnitsItem);

        configMenu.add(unitsMenu);

        configMenu.add(jSeparator15);

        propsMenuItem.setText(rb.getString("GUI.Menu.Config.Options"));
        propsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                propsMenuItemActionPerformed(evt);
            }
        });

        configMenu.add(propsMenuItem);

        ncMenuBar.add(configMenu);

        helpMenu.setText(rb.getString("GUI.Menu.Help"));
        homepageMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/home.png")));
        homepageMenuItem.setText(rb.getString("GUI.General.Homepage"));
        homepageMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                homepageMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(homepageMenuItem);

        supportMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/support.png")));
        supportMenuItem.setText(rb.getString("GUI.Menu.Support"));
        supportMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                supportMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(supportMenuItem);

        helpMenu.add(jSeparator20);

        helpIndexMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/help3.png")));
        helpIndexMenuItem.setText(rb.getString("GUI.Menu.Help.Index"));
        helpIndexMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpIndexMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(helpIndexMenuItem);

        helpMenuItemAbout.setIcon(new javax.swing.ImageIcon(getClass().getResource("/core/images/main/about.png")));
        helpMenuItemAbout.setText(rb.getString("GUI.Menu.Help.About"));
        helpMenuItemAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuItemAboutActionPerformed(evt);
            }
        });

        helpMenu.add(helpMenuItemAbout);

        ncMenuBar.add(helpMenu);

        setJMenuBar(ncMenuBar);
        ncMenuBar.getAccessibleContext().setAccessibleName("Menu");
        ncMenuBar.getAccessibleContext().setAccessibleDescription("Menu");

    }//GEN-END:initComponents

    private void squarenessMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_squarenessMenuItemActionPerformed
        // Add your handling code here:
        squarenessLnfPressed();
    }//GEN-LAST:event_squarenessMenuItemActionPerformed

    private void elRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_elRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("el","GR","en");
        elRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_elRadioButtonMenuItemActionPerformed

    private void heRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_heRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("iw","IL","en");
        heRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_heRadioButtonMenuItemActionPerformed

    private void sourcePopupMenuItemRedoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemRedoActionPerformed
        // Add your handling code here:
        redo(sourceUndo);
    }//GEN-LAST:event_sourcePopupMenuItemRedoActionPerformed

    private void sourcePopupMenuItemUndoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemUndoActionPerformed
        // Add your handling code here:
        undo(sourceUndo);
    }//GEN-LAST:event_sourcePopupMenuItemUndoActionPerformed

    private void sourceMenuItemRedoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemRedoActionPerformed
        // Add your handling code here:
        redo(sourceUndo);
    }//GEN-LAST:event_sourceMenuItemRedoActionPerformed

    private void sourceMenuItemUndoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemUndoActionPerformed
        // Add your handling code here:
        undo(sourceUndo);
    }//GEN-LAST:event_sourceMenuItemUndoActionPerformed

    private void oceanRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_oceanRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setMetalTheme(oceanTheme);
          userProps.setProperty("lnf","metal_ocean");
        }
    }//GEN-LAST:event_oceanRadioButtonMenuItemItemStateChanged

    private void supportMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_supportMenuItemActionPerformed
        // Add your handling code here:
        historyTextArea.append("application.name="+PROGRAMNAME+"\n");
        historyTextArea.append("application.version="+VERSION+"\n");
        historyTextArea.append("application.patch.level="+PATCHLEVEL+"\n");
        Properties debugProperties = System.getProperties();

        // debugProperties.list(System.out);
        for (Enumeration e = debugProperties.keys(); e.hasMoreElements() ;) {
           String o = (String)e.nextElement();
           historyTextArea.append(o+"="+debugProperties.getProperty(o)+"\n");
        }

        historyTextArea.append("system.lnf="+UIManager.getSystemLookAndFeelClassName()+"\n");

    }//GEN-LAST:event_supportMenuItemActionPerformed

    private void findTargetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_findTargetButtonActionPerformed
        // Add your handling code here:
         findTargetUnit();
    }//GEN-LAST:event_findTargetButtonActionPerformed

    private void findSourceButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_findSourceButtonActionPerformed
        // Add your handling code here:
         findSourceUnit();
    }//GEN-LAST:event_findSourceButtonActionPerformed

    private void reloadCategoriesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_reloadCategoriesMenuItemActionPerformed
        // Add your handling code here:
        loadResourceBundle();
        register();
    }//GEN-LAST:event_reloadCategoriesMenuItemActionPerformed

    private void historyMenuItemRedoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemRedoActionPerformed
        // Add your handling code here:
        redoMenuItemActionPerformed(evt);
    }//GEN-LAST:event_historyMenuItemRedoActionPerformed

    private void historyMenuItemUndoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemUndoActionPerformed
        // Add your handling code here:
        undoMenuItemActionPerformed(evt);
    }//GEN-LAST:event_historyMenuItemUndoActionPerformed

    private void historyPopupMenuItemHTMLActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPopupMenuItemHTMLActionPerformed
        // Add your handling code here:
        htmlMenuItemActionPerformed(evt);
    }//GEN-LAST:event_historyPopupMenuItemHTMLActionPerformed

    private void deactivateFilterCategoriesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deactivateFilterCategoriesMenuItemActionPerformed
        // Add your handling code here:
        clickOnFilterCategoryIndicator();
    }//GEN-LAST:event_deactivateFilterCategoriesMenuItemActionPerformed

    private void redoMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_redoMenuItemActionPerformed
        // Add your handling code here:
        redo(historyUndo);
    }//GEN-LAST:event_redoMenuItemActionPerformed

    private void undoMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_undoMenuItemActionPerformed
        // Add your handling code here:
        undo(historyUndo);
    }//GEN-LAST:event_undoMenuItemActionPerformed

    private void manageFavoritesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_manageFavoritesMenuItemActionPerformed
        // Add your handling code here:
        new BookmarksDialog(this,true,rb,bookmarks,categoriesHashtable);
        // update the JList
        if (bookmarks.size() < 10)
            bookmarkList.setVisibleRowCount(bookmarks.size()); else
               bookmarkList.setVisibleRowCount(10);
    }//GEN-LAST:event_manageFavoritesMenuItemActionPerformed

    private void plasticMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_plasticMenuItemActionPerformed
        // Add your handling code here:
        plasticLnfPressed();
    }//GEN-LAST:event_plasticMenuItemActionPerformed

    private void htmlMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_htmlMenuItemActionPerformed
        // Add your handling code here:
        boolean add = true;
        if (historyTextArea.getText().startsWith("<html>")) {
          // html header already there really add?
          add = showQuestion(this, "",rb.getString("Edit.HTMLTemplateAddAgain"));
        }
        if (add) {
          historyTextArea.insert(HTML_HEADER, 0);
          historyTextArea.append(HTML_FOOTER);
        }
    }//GEN-LAST:event_htmlMenuItemActionPerformed

    private void cancelFavoritesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelFavoritesMenuItemActionPerformed
        // Add your handling code here:

    }//GEN-LAST:event_cancelFavoritesMenuItemActionPerformed

    private void addFavoriteMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addFavoriteMenuItemActionPerformed
        // Add your handling code here:

        // add bookmark to bookmarks
        Bookmark bookmark = new Bookmark(
           (SectionRecord)categoryBox.getSelectedItem(),
           (NcRecord)sourceBox.getSelectedItem(),
           (NcRecord)targetBox.getSelectedItem()
        );
        // for a specific locale!
        bookmark.setLanguage(userProps.getProperty("language"));
        bookmark.setCountry(userProps.getProperty("country"));
        bookmark.setValue(sourceTextArea.getText());


        // Append an item
        int pos = bookmarkList.getModel().getSize();
        bookmarks.add(pos, bookmark);

        // update the JList
            if (bookmarks.size() < 10)
                bookmarkList.setVisibleRowCount(bookmarks.size()); else
                    bookmarkList.setVisibleRowCount(10);

        // save all bookmarks
        try {
          bookmarks.save();
        } catch (IOException ioe) {
            System.err.println(ioe);
        }

    }//GEN-LAST:event_addFavoriteMenuItemActionPerformed

    private void searchCategoriesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchCategoriesMenuItemActionPerformed
        // Add your handling code here:
        String find = JOptionPane.showInputDialog(this,rb.getString("GUI.Menu.Find.FindCategory"),rb.getString("GUI.Menu.Find.FindCategory"),JOptionPane.PLAIN_MESSAGE);
        if ((find!=null) && (!find.equals(""))) { // user doesn't cancel
           // search in the combobox text
           boolean found=false;
           for (int i=0; i < categoryBox.getItemCount(); i++) {
              String name = ((SectionRecord)categoryBox.getItemAt(i)).getString();
              if ( name.toLowerCase().indexOf(find.toLowerCase()) > -1) {
                  categoryBox.setSelectedIndex(i);
                  i=categoryBox.getItemCount();
                  found=true;
                  break;
              }
           }
           if (!found) {
               JOptionPane.showMessageDialog(this,rb.getString("Message.CategoryNotFound"), rb.getString("Message.CategoryNotFound"), JOptionPane.INFORMATION_MESSAGE);
           }
        }
    }//GEN-LAST:event_searchCategoriesMenuItemActionPerformed

    private void searchTargetMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchTargetMenuItemActionPerformed
        // Add your handling code here:
        findTargetUnit();
    }//GEN-LAST:event_searchTargetMenuItemActionPerformed

    private void searchSourceMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchSourceMenuItemActionPerformed
        // Add your handling code here:
        findSourceUnit();
    }//GEN-LAST:event_searchSourceMenuItemActionPerformed

    private void colorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_colorButtonActionPerformed
        // Add your handling code here:
       menuItemColorPressed();
    }//GEN-LAST:event_colorButtonActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // Add your handling code here:
       menuItemCalendarPressed();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        // Add your handling code here:
      clusterObject.stopPlaying();
    }//GEN-LAST:event_stopButtonActionPerformed

    private void playButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playButtonActionPerformed
        // Add your handling code here:
        playNumber();
    }//GEN-LAST:event_playButtonActionPerformed

    private void homepageMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_homepageMenuItemActionPerformed
        // Add your handling code here:
        try {
          Browser.displayURL("http://www.jonelo.de/java/nc/index.html");
        } catch (Exception e) {
          System.out.println(e);
        }
    }//GEN-LAST:event_homepageMenuItemActionPerformed

    private void tablesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tablesMenuItemActionPerformed
        // Add your handling code here:
        if (!tablesGenerated) {
            try {
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                generatingTables();
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                tablesGenerated=true;
                tablesMenuItem.addActionListener(new CSH.DisplayHelpFromSource(tableHB));
                tablesMenuItem.doClick();
            } catch (Exception ee) {
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                error(ee.toString());
            }
        }
    }//GEN-LAST:event_tablesMenuItemActionPerformed

    private void spellMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_spellMenuItemActionPerformed
        // Add your handling code here:
        new PhoneticAlphabetDialog(this,false,rb);
    }//GEN-LAST:event_spellMenuItemActionPerformed

    private void sourceMenuItemDefaultValueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemDefaultValueActionPerformed
        // Add your handling code here:
        defaultValuePressed();
    }//GEN-LAST:event_sourceMenuItemDefaultValueActionPerformed

    private void sourcePopupMenuDefaultValueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuDefaultValueActionPerformed
        // Add your handling code here:
        defaultValuePressed();
    }//GEN-LAST:event_sourcePopupMenuDefaultValueActionPerformed

    private void defaultValuePressed() {
        sourceBox.setSelectedIndex(clusterObject.getSourceDefault());
        String initValue = clusterObject.getInitialValue();
        sourceTextArea.setText(initValue == null ? "1" : initValue);
        process();
    }

    private void sourceMenuItemCalendarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemCalendarActionPerformed
        // Add your handling code here:
        menuItemCalendarPressed();
    }//GEN-LAST:event_sourceMenuItemCalendarActionPerformed

    private void menuItemCalendarPressed() {
        GregorianCalendar calendar = new GregorianCalendar();
        boolean[] status={ false };
        new CalendarChooserDialog(this, rb, calendar, status);

        if (status[0]==true) { // not cancelled
           if (categoryBox.getSelectedItem() != categoriesHashtable.get("CALENDAR")) {
             categoryBox.setSelectedItem(categoriesHashtable.get("CALENDAR"));
             categoryBoxItemStateChanged();
           }
           try {
             DecimalFormat digs2 = new DecimalFormat("00");
             DecimalFormat digs4 = new DecimalFormat("0000");
             StringBuffer sb = new StringBuffer(8);
             sb.append(digs4.format(calendar.get(Calendar.YEAR)));
             sb.append(digs2.format(calendar.get(Calendar.MONTH)+1));
             sb.append(digs2.format(calendar.get(Calendar.DATE)));

             setValue(new BigDecimal(sb.toString()));
           } catch (Exception e) {
             System.out.println(e.toString());
           }
        }

    }

    private void filterLabelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_filterLabelMouseClicked
        // Add your handling code here:
        clickOnFilterCategoryIndicator();
    }//GEN-LAST:event_filterLabelMouseClicked

    private void usRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_usRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("en","US","en");
        usRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_usRadioButtonMenuItemActionPerformed

    private void filterCatIndicatorLabelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_filterCatIndicatorLabelMouseClicked
        // Add your handling code here:
        clickOnFilterCategoryIndicator();
    }//GEN-LAST:event_filterCatIndicatorLabelMouseClicked

    private void clickOnFilterCategoryIndicator() {
        groupsHashtable.clear();
        fillCategoryModel(groupAllCategories);
        lastTypeBoxSelectedIndex=-1;
        filterCatIndicatorLabel.setVisible(false);
        filterLabel.setToolTipText(null);
        filterLabel.setVisible(false);
        deactivateFilterCategoriesMenuItem.setEnabled(false);

        // SwingUtilities.updateComponentTreeUI(getRootPane());
        categoryBoxItemStateChanged();
        //register();
    }

    private void propsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_propsMenuItemActionPerformed
        // Add your handling code here:
        new PropsDialog(this,true,rb,userProps).show();
        if (userProps.containsKey("save")) {
            userProps.remove("save");
            saveUserProperties();
        }
        // set running options
        setToolTips(userProps.getProperty("tooltips","true").equals("true"));
        try {
          ToolTipManager.sharedInstance().setInitialDelay(Integer.parseInt(userProps.getProperty("tooltips.delay","750")));
        } catch (NumberFormatException nfe) {
        }
    }//GEN-LAST:event_propsMenuItemActionPerformed

    private void targetMenuItemFindUnitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemFindUnitActionPerformed
        // Add your handling code here:
        findTargetUnit();
    }//GEN-LAST:event_targetMenuItemFindUnitActionPerformed

    private void targetPopupMenuItemFindUnitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetPopupMenuItemFindUnitActionPerformed
        // Add your handling code here:
        findTargetUnit();
    }//GEN-LAST:event_targetPopupMenuItemFindUnitActionPerformed

    private void sourcePopupMenuItemFindUnitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemFindUnitActionPerformed
        // Add your handling code here:
        findSourceUnit();
    }//GEN-LAST:event_sourcePopupMenuItemFindUnitActionPerformed

    private void findSourceUnitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_findSourceUnitMenuItemActionPerformed
        // Add your handling code here:
        findSourceUnit();
    }//GEN-LAST:event_findSourceUnitMenuItemActionPerformed


    private int findCategoryWithCode(String code)
    {
           for (int i=0; i < categoryBox.getItemCount(); i++) {
              String name = ((SectionRecord)categoryBox.getItemAt(i)).getName();
              if (code.equals(name)) {
                  return i;
              }
           }
           return -1;
    }

    // source
    private void findSourceUnit() {
        //String find = JOptionPane.showInputDialog(this,"Find what:","Find",JOptionPane.PLAIN_MESSAGE);
        new FindDialog(this,true,this,true);
    }

    public JComboBox getSourceBox() {
        return sourceBox;
    }


    public int findEntry(Class myCast, JComboBox box, String find, int begin, boolean matchCase, boolean exact) {
        if ((find!=null) && (!find.equals(""))) { // user didn't cancel
           // search in the combobox text
           for (int i=begin; i < box.getItemCount(); i++) {
              String name = null;
              if (myCast == NcRecord.class) {
                 name = ((NcRecord)box.getItemAt(i)).getString();
              } else {
                 name = ((SectionRecord)box.getItemAt(i)).getString();
              }
              if (matchCase) {
                  if (exact) {
                     if (name.equals(find)) {
                         return i;
                     }
                  } else {
                     if (name.indexOf(find) > -1) {
                         return i;
                     }
                  }
              } else {
                  if (exact) {
                     if (name.toLowerCase().equals(find.toLowerCase())) {
                         return i;
                     }
                  } else {
                     if (name.toLowerCase().indexOf(find.toLowerCase()) > -1) {
                         return i;
                     }
                  }
              }
           }
        }
        return -1;

    }

    public int findEntry(Class myCast, JComboBox box, String find, int begin, boolean matchCase) {
       return findEntry(myCast,  box,  find, begin, matchCase, false);
    }

    public int findUnit(JComboBox box, String find, int begin, boolean matchCase) {
        return findEntry(NcRecord.class, box, find, begin, matchCase, false);
    }

    public int findUnitExact(JComboBox box, String find, int begin, boolean matchCase) {
        return findEntry(NcRecord.class, box, find, begin, matchCase, true);
    }

    public JComboBox getTargetBox() {
        return targetBox;
    }

    private void findTargetUnit() {
        new FindDialog(this,true,this,false);
    }


    private void gtkRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_gtkRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(GeneralGUI.LAF_GTK);
          userProps.setProperty("lnf","gtk");
          checkForFrameDecoration();
        }
    }//GEN-LAST:event_gtkRadioButtonMenuItemItemStateChanged

    private void aquaRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_aquaRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(GeneralGUI.LAF_AQUA);
          userProps.setProperty("lnf","aqua");
          checkForFrameDecoration();
       }
    }//GEN-LAST:event_aquaRadioButtonMenuItemItemStateChanged

    private void systemRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_systemRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(UIManager.getSystemLookAndFeelClassName());
          userProps.setProperty("lnf","system");
          checkForFrameDecoration();
        }
    }//GEN-LAST:event_systemRadioButtonMenuItemItemStateChanged

    private void targetPopupMenuItemStopPlayingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetPopupMenuItemStopPlayingActionPerformed
        // Add your handling code here:
        clusterObject.stopPlaying();
    }//GEN-LAST:event_targetPopupMenuItemStopPlayingActionPerformed

    private void targetPopupMenuItemPlayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetPopupMenuItemPlayActionPerformed
        // Add your handling code here:
        playNumber();
    }//GEN-LAST:event_targetPopupMenuItemPlayActionPerformed

    public void playNumber() {
        try {
            clusterObject.play(targetBox.getSelectedIndex());
        } catch (Exception e) {
            messageLightError(rb.getString("Message.GUI.SoundsNotFound"));
        }
    }

    private void targetMenuItemStopPlayingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemStopPlayingActionPerformed
        // Add your handling code here:
        clusterObject.stopPlaying();
    }//GEN-LAST:event_targetMenuItemStopPlayingActionPerformed

    private void targetBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetBoxActionPerformed
        // Add your handling code here:
        process();
        sourceTextArea.requestFocus();
    }//GEN-LAST:event_targetBoxActionPerformed

    private void categoryBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_categoryBoxActionPerformed
        // Add your handling code here:
        // do it, if it is not the same as before
        if ((clusterObject != null) && (lastTypeBoxSelectedIndex != categoryBox.getSelectedIndex())) {
            // stop any sound (if there is any sound ;-)
            clusterObject.stopPlaying();
            // save the last index
            lastTypeBoxSelectedIndex=categoryBox.getSelectedIndex();

            // backup the value from the current category
            transferValue = "";
            try {
                transferValue=clusterObject.getTransferValue(sourceBox.getSelectedIndex());
            } catch (Exception e) {}

            categoryBoxItemStateChanged();

        }
    }//GEN-LAST:event_categoryBoxActionPerformed

    private void sourceBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceBoxActionPerformed
        // Add your handling code here:
        process();
        sourceTextArea.requestFocus();
    }//GEN-LAST:event_sourceBoxActionPerformed

    private void sciCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sciCheckBoxActionPerformed
        // Add your handling code here:
        process();
    }//GEN-LAST:event_sciCheckBoxActionPerformed

    private void sciCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_sciCheckBoxItemStateChanged
        // Add your handling code here:
        int change = evt.getStateChange();
        if (change == ItemEvent.SELECTED) {
            clusterObject.setScientific(true);
/*           precisionTextField.setText("1000");
            clusterObject.setPrecision(1000);
            precisionTextField.setEditable(false);
            precisionLabel.setEnabled(false);
            targetMenuItemFigures.setEnabled(false);
 */
        } else
        if (change == ItemEvent.DESELECTED) {
            clusterObject.setScientific(false);
/*          clusterObject.setPrecision(clusterObject.getPreferredPrecision());
            precisionTextField.setText(Integer.toString(clusterObject.getPrecision()));
            precisionTextField.setEditable(true);
            precisionLabel.setEnabled(true);
            targetMenuItemFigures.setEnabled(true);
 */
        }
    }//GEN-LAST:event_sciCheckBoxItemStateChanged

    private void targetPopupMenuItemDefaultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetPopupMenuItemDefaultActionPerformed
        // Add your handling code here:
        targetBox.setSelectedIndex(clusterObject.getTargetDefault());
    }//GEN-LAST:event_targetPopupMenuItemDefaultActionPerformed

    private void sourcePopupMenuItemDefaultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemDefaultActionPerformed
        // Add your handling code here:
        sourceBox.setSelectedIndex(clusterObject.getSourceDefault());
    }//GEN-LAST:event_sourcePopupMenuItemDefaultActionPerformed

    private void targetMenuItemDefaultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemDefaultActionPerformed
        // Add your handling code here:
        targetBox.setSelectedIndex(clusterObject.getTargetDefault());
    }//GEN-LAST:event_targetMenuItemDefaultActionPerformed

    private void sourceMenuItemDefaultActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemDefaultActionPerformed
        // Add your handling code here:
        sourceBox.setSelectedIndex(clusterObject.getSourceDefault());
    }//GEN-LAST:event_sourceMenuItemDefaultActionPerformed

    private void recordMenuItemFontsizeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemFontsizeActionPerformed
        // Add your handling code here:

        String input = null;
        Font font = historyTextArea.getFont();

        boolean end = false;
        while (!end) {
            if (GeneralProgram.isSupportFor("1.4"))
            input = JOptionPane.showInputDialog(this,rb.getString("GUI.Menu.Records.Fontsize"),Float.toString(font.getSize2D()));
             else
            input = JOptionPane.showInputDialog(this,rb.getString("GUI.Menu.Records.Fontsize"));
            if (input!=null) {
                try {
                    historyTextArea.setFont(font.deriveFont( Float.valueOf(input).floatValue()) );
                    end=true;
                    userProps.setProperty("fontsize",input);
                } catch (NumberFormatException e) {
                    System.err.println(e.toString());
                }
            } else end=true;
        }
    }//GEN-LAST:event_recordMenuItemFontsizeActionPerformed

    private void historyMenuFontSizeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuFontSizeActionPerformed
        // Add your handling code here:
        recordMenuItemFontsizeActionPerformed(evt);
    }//GEN-LAST:event_historyMenuFontSizeActionPerformed

    private void historyMenuItemLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemLoadActionPerformed
        // Add your handling code here:
        loadMenuItemActionPerformed(evt);
    }//GEN-LAST:event_historyMenuItemLoadActionPerformed

    private void loadMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadMenuItemActionPerformed
        // Add your handling code here:

        String NEWLINE = "\n";
        if (fc_load==null) {
            fc_load = new JFileChooser();
            fc_load.removeChoosableFileFilter(fc_load.getAcceptAllFileFilter());
            if (utf8==null) utf8 = new ExampleFileFilter("txt", "Unicode, UTF-8");
            if (utf16==null) utf16 = new ExampleFileFilter("txt", "Unicode, UTF-16");
            if (iso8859==null) iso8859 = new ExampleFileFilter("txt", "Text, ISO8859_1");
            fc_load.addChoosableFileFilter(iso8859);
            fc_load.addChoosableFileFilter(utf8);
            fc_load.addChoosableFileFilter(utf16);
            fc_load.setFileFilter(utf8);
        }
        boolean error=false;
        int returnVal=0;

        do {
            error=false;
            returnVal = fc_load.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                String encoding=null;
                ExampleFileFilter ff = (ExampleFileFilter)fc_load.getFileFilter();
                if (ff.equals(utf8)) encoding="UTF-8";  else
                if (ff.equals(utf16)) encoding="UTF-16"; else
                   encoding="ISO8859_1";

                File selFile = fc_load.getSelectedFile();
                FileInputStream fis = null;
                try {
                    fis = new FileInputStream(selFile);
                    BufferedReader br = new BufferedReader(new InputStreamReader(fis,encoding));
                    String thisLine=null;
                    StringBuffer sb = new StringBuffer();

                    historyTextArea.setText("");
                    while ((thisLine = br.readLine()) != null) {
                        historyTextArea.append(thisLine);
                        historyTextArea.append(NEWLINE);
                    }

                    br.close();
                    fis.close();

                } catch (Exception e) {
                    error=true;
                    JOptionPane.showMessageDialog(this,e.getMessage(),"Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        } while (returnVal==JFileChooser.APPROVE_OPTION && error);

    }//GEN-LAST:event_loadMenuItemActionPerformed

    private void esRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_esRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("es","MX","en");
        esRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_esRadioButtonMenuItemActionPerformed

    private void targetMenuItemPlayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemPlayActionPerformed
        // Add your handling code here:
        try {
            clusterObject.play(targetBox.getSelectedIndex());
        } catch (Exception e) {
            messageLightError(e.getMessage());
        }
    }//GEN-LAST:event_targetMenuItemPlayActionPerformed

    private void sourceMenuItemColorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemColorActionPerformed
        menuItemColorPressed();
    }//GEN-LAST:event_sourceMenuItemColorActionPerformed

    private void menuItemColorPressed()
    {
        // Add your handling code here:

        BigDecimal big = null;
        try {
            big = new BigDecimal(clusterObject.getTransferValue(sourceBox.getSelectedIndex()));
        } catch (Exception e) {}
        Color color=Color.black;

        if (big != null) {
            BigInteger bigi = big.toBigInteger();

            if ((bigi.compareTo(new BigInteger("FFFFFF",16)) <= 0) &&
            (bigi.compareTo(new BigInteger("0")) > 0))
                color=new Color(bigi.intValue());
        }

        Color newColor = JColorChooser.showDialog(this, rb.getString("GUI.General.ColorPicker"), color);

        if (newColor !=null) {
            categoryBox.setSelectedItem(categoriesHashtable.get("COLOR"));
            try {
                setValue( new BigDecimal(Integer.toString(newColor.getRGB() & 0x00ffffff)));
            } catch (Exception e) { System.out.println(e.toString()); }
        }
    }

    private void historyMenuItemSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemSaveActionPerformed
        // Add your handling code here:
        saveButtonActionPerformed(evt);
    }//GEN-LAST:event_historyMenuItemSaveActionPerformed

    private void saveMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveMenuItemActionPerformed
        // Add your handling code here:
        saveButtonActionPerformed(evt);
    }//GEN-LAST:event_saveMenuItemActionPerformed

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // Add your handling code here:
        if (fc_save==null) {
            fc_save = new JFileChooser();
            fc_save.removeChoosableFileFilter(fc_save.getAcceptAllFileFilter());
            if (html==null) html = new ExampleFileFilter("html", "HTML, UTF-8");
            if (vcs==null) vcs = new ExampleFileFilter("vcs", "vCalendar");
            if (utf8==null) utf8 = new ExampleFileFilter("txt", "Unicode text, UTF-8");
            if (iso8859==null) iso8859 = new ExampleFileFilter("txt", "Text, ISO8859_1");
            if (utf16le==null) utf16le = new ExampleFileFilter("txt", "Unicode text, UTF-16 little-endian");
            if (utf16be==null) utf16be = new ExampleFileFilter("txt", "Unicode text, UTF-16 big-endian");

            fc_save.addChoosableFileFilter(html);
            fc_save.addChoosableFileFilter(vcs);
            fc_save.addChoosableFileFilter(utf8);
            fc_save.addChoosableFileFilter(iso8859);
            fc_save.addChoosableFileFilter(utf16le);
            fc_save.addChoosableFileFilter(utf16be);

            fc_save.setFileFilter(utf8);
        }

        int returnVal = fc_save.showSaveDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selFile = fc_save.getSelectedFile();
            boolean overwrite=true;
            if (selFile.exists() && selFile.isFile()) {
                overwrite = showQuestion(this, "",GeneralString.message(rb.getString("Error.FileAlreadyExists"),selFile.getPath()));
            }
            if (overwrite) {
                FileOutputStream fos = null;
                try {
                    String encoding=null;
                    boolean htmltag=false;
                    fos = new FileOutputStream(selFile);
                    ExampleFileFilter ff = (ExampleFileFilter)fc_save.getFileFilter();
                    if (ff.equals(utf16be)) { encoding="UnicodeBig"; } else
                    if (ff.equals(utf16le)) { encoding="UnicodeLittle"; } else
                    if (ff.equals(utf8)) { encoding="UTF-8"; } else
                    if (ff.equals(iso8859) || ff.equals(vcs)) { encoding="ISO8859_1"; } else
                    if (ff.equals(html)) { encoding="UTF-8"; htmltag=true; }

                    BufferedWriter myOutput = new BufferedWriter(new OutputStreamWriter(fos,encoding));
                    if (htmltag) {
                        // check, if HTML-Header/Footer is there
                        boolean add = false;
                        if (!historyTextArea.getText().startsWith("<html>"))
                        {
                           // html header already there really add?
                           add = showQuestion(this, "",rb.getString("Edit.HTMLTemplateNotFound"));
                        }
                        if (add) {
                           historyTextArea.insert(HTML_HEADER, 0);
                           historyTextArea.append(HTML_FOOTER);
                        }
                    }

                    myOutput.write(historyTextArea.getText());
                    myOutput.close();
                    fos.close();

                    messageOk(rb.getString("Message.GUI.FileWritten"));
                } catch (Exception e) {
                    messageLightError(e.getMessage());
                    JOptionPane.showMessageDialog(this,e.getMessage(),"Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }//GEN-LAST:event_saveButtonActionPerformed

    public static boolean showQuestion(Component parentComponent, String title, String message) {
        boolean temp=false;
        int result = JOptionPane.showConfirmDialog(
        parentComponent, // parentComponent
        message, // message
        title, // title
        JOptionPane.YES_NO_CANCEL_OPTION, // optionType
        JOptionPane.QUESTION_MESSAGE); // messageType

        switch(result) {
            case JOptionPane.CLOSED_OPTION:
                temp=false; break;
            case JOptionPane.YES_OPTION:
                temp=true; break;
            case JOptionPane.NO_OPTION:
                temp=false; break;
            case JOptionPane.CANCEL_OPTION:
                temp=false; break;
        }
        return temp;
    }

  private void recordMenuItemMakeListActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemMakeListActionPerformed
      // Add your handling code here:
      noticesButtonActionPerformed(evt);
  }//GEN-LAST:event_recordMenuItemMakeListActionPerformed

  private void historyPopupMenuItemMakeListActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPopupMenuItemMakeListActionPerformed
      // Add your handling code here:
      noticesButtonActionPerformed(evt);
  }//GEN-LAST:event_historyPopupMenuItemMakeListActionPerformed

  private void historyPopupMenuenuItemFormatActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPopupMenuenuItemFormatActionPerformed
      // Add your handling code here:
      recordMenuItemFormatActionPerformed(evt);
  }//GEN-LAST:event_historyPopupMenuenuItemFormatActionPerformed

  private void recordMenuItemFormatActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemFormatActionPerformed
      // Add your handling code here:

      String input = userProps.getProperty("notice_form",DEFAULT_NOTICE_FORM);

      ArrayList al = new ArrayList();
      int i=0;
      while (systemProps.getProperty("format_"+i) != null) {
          al.add(systemProps.getProperty("format_"+i,DEFAULT_NOTICE_FORM));
          i++;
      }
      i=0;
      while (userProps.getProperty("format_"+i) != null) {
          al.add(userProps.getProperty("format_"+i,DEFAULT_NOTICE_FORM));
          i++;
      }

      String entered = (String)JOptionPane.showInputDialog(this,rb.getString("GUI.Menu.Records.Format"),rb.getString("GUI.Menu.Records.Format"), JOptionPane.QUESTION_MESSAGE, null, al.toArray(), input);
      if (entered != null) {
          userProps.setProperty("notice_form",entered);
          saveUserProperties();
      }
  }//GEN-LAST:event_recordMenuItemFormatActionPerformed

    private void noticesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noticesButtonActionPerformed
        // Add your handling code here:
        multipleNoticeCluster.reset();

        // if category "Calendar (date)" is selected, then init list values with today
        if (clusterObject instanceof ClusterCalendar) {
            multipleNoticeCluster.setStartValue(new BigDecimal(ClusterCalendar.getInternationalTodayValue()));
            multipleNoticeCluster.setEndValue(multipleNoticeCluster.getStartValue());
        }

        // if category "Calendar (year)" is selected, then init list values with current year
        // and make the "show list format" visible
        if (clusterObject instanceof ClusterCalendarYear) {
            multipleNoticeCluster.setStartValue(new BigDecimal(clusterObject.getInitialValue()));
            multipleNoticeCluster.setEndValue(multipleNoticeCluster.getStartValue());
            multipleNoticeCluster.setListFormatVisible(true);
    }

        if (clusterObject instanceof ClusterTimezones) {
            multipleNoticeCluster.setStartValue(new BigDecimal(ClusterTimezones.getInternationalTimeValue()));
            multipleNoticeCluster.setEndValue(multipleNoticeCluster.getStartValue());
        }

        new MultipleNoticeDialog(this,true,multipleNoticeCluster,rb);

        if (
          multipleNoticeCluster.isSuccess()
          &&
          (multipleNoticeCluster.getStartValue().compareTo(multipleNoticeCluster.getEndValue()) <= 0)
          &&
          (
            ((multipleNoticeCluster.getStartValue().compareTo(multipleNoticeCluster.getEndValue()) < 0) &&
            (multipleNoticeCluster.getStepValue().compareTo(new BigDecimal(BigInteger.ZERO)) > 0))
            ||
            (multipleNoticeCluster.getStartValue().compareTo(multipleNoticeCluster.getEndValue()) == 0)
          )
        ) {
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            boolean specialFormat = multipleNoticeCluster.getListFormat() != null;
            int headerPos=historyTextArea.getCaretPosition();
            int count=0;
            if (multipleNoticeCluster.getTargets()==MultipleNoticeCluster.ALL_TARGETS) {
                dontPanic();
                setState(Frame.ICONIFIED);
                int save = targetBox.getSelectedIndex();
                for (int i=0; i < targetBox.getItemCount(); i++) {
                    targetBox.setSelectedIndex(i);
                    count=processList(multipleNoticeCluster);
                    if (!specialFormat && count > 1) historyTextArea.append("\n");
                }
                targetBox.setSelectedIndex(save);
                setState(Frame.NORMAL);
            } else
            if (multipleNoticeCluster.getTargets()==MultipleNoticeCluster.MULTIPLE_TARGETS) {
                int save = targetBox.getSelectedIndex();
                // which flag?
                String flag = ((NcRecord)targetBox.getSelectedItem()).getIcon();
                for (int i=0; i < targetBox.getItemCount(); i++) {

                    if ( ((NcRecord)targetBox.getItemAt(i)).getIcon().equals(flag))
                    {
                      targetBox.setSelectedIndex(i);
                      count=processList(multipleNoticeCluster);
                      if (!specialFormat && count > 1) historyTextArea.append("\n");
                    }
                }
                targetBox.setSelectedIndex(save);
            } else {
                processList(multipleNoticeCluster);
            }

            // add both header and footer if a special output format was requested
            if (specialFormat) {
                // header
                historyTextArea.insert(multipleNoticeCluster.getListFormat().getHeader(), headerPos);
                // footer
                historyTextArea.append(multipleNoticeCluster.getListFormat().getFooter());
            }
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

        }
        // else do nothing

    }//GEN-LAST:event_noticesButtonActionPerformed

    /**
     * return how many lines were processed
     */
    private int processList(MultipleNoticeCluster mnc) {
        int i=0;
        BigDecimal runValue = mnc.getStartValue();
        //end=end.add(mnc.getStepValue());
        boolean first = true;
        try {
            if (mnc.getStartValue().equals(mnc.getEndValue())) {
                setValue(runValue);
                i++;
                performNotice();
            } else
                // nur weitermachen, wenn alles ok war bei der letzten Konvertierung
                while ( runValue.add(mnc.getStepValue()).compareTo(mnc.getEndValue()) <= 0) {
                    if (first) {
                        setValue(runValue);
                        first=false;
                    } else
                        addValue(mnc.getStepValue());

                    if (!historyNewButton.isEnabled()) break;

                    i++;
                    performNotice();
                    runValue = new BigDecimal(clusterObject.getTransferValue(sourceBox.getSelectedIndex()));
                }

        } catch (Exception e) {
            messageError(e.getMessage());
            // In case of an error
            // e.g. RomanNumber 4999+1=5000, 5000+1=5001 => error
            // an invalid number is set internally in clusterObject, probably
            // therefore we set it again
            clusterObject.setInput(sourceTextArea.getText());
        }
        return i;
    }


    private void maximumRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_maximumRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        luxuryFrame(-1);
    }//GEN-LAST:event_maximumRadioButtonMenuItemItemStateChanged

    private void reloadMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_reloadMenuItemActionPerformed
        // Add your handling code here:
        categoryBoxItemStateChanged();
    }//GEN-LAST:event_reloadMenuItemActionPerformed

    private void reloadUnitsItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_reloadUnitsItemActionPerformed
        // Add your handling code here:
        categoryBoxItemStateChanged();
    }//GEN-LAST:event_reloadUnitsItemActionPerformed

    private void countUnitsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_countUnitsMenuItemActionPerformed
        // Add your handling code here:
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            String filename = "/data/lists/categories.list";
            InputStream is = getClass().getResourceAsStream(filename);
            BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
            InputStream is2 = null;
            BufferedReader br2 = null;

            int categories=0;
            int units=0;

            StringTokenizer st = null;
            String thisLine;
            while ((thisLine = br.readLine()) != null) {
                if (!thisLine.startsWith("#") && !thisLine.equals("")) {

                    st = new StringTokenizer(thisLine,":");
                    String code = st.nextToken();

                    categories++;
                    if (code.equals(ClusterBases35.CODE)) units+=ClusterBases35.INTERNAL; else
                    if (code.equals(ClusterAngle.CODE)) units+=ClusterAngle.INTERNAL; else
                    if (code.equals(ClusterRoman.CODE)) units+=ClusterRoman.INTERNAL; else
                    if (code.equals(ClusterBases35.CODE)) units+=ClusterBases35.INTERNAL; else
                    if (code.equals(ClusterSpokenNumbers.CODE)) units+=ClusterSpokenNumbers.INTERNAL; else
                    if (code.equals(ClusterTemperature.CODE)) units+=ClusterTemperature.INTERNAL; else
                    if (code.equals(ClusterUnicode.CODE)) units+=ClusterUnicode.INTERNAL; else
                    if (code.equals(ClusterColor.CODE)) units+=ClusterColor.INTERNAL; else
                    if (code.equals(ClusterCalendar.CODE)) units+=ClusterCalendar.INTERNAL; else
                    if (code.equals(ClusterCalendarYear.CODE)) units+=ClusterCalendarYear.INTERNAL; else
                    if (code.equals(ClusterAlphaPhone.CODE)) units+=ClusterAlphaPhone.INTERNAL; else
                    if (code.equals(ClusterDialCode.CODE)) units+=ClusterDialCode.INTERNAL; else
                    if (code.equals(ClusterTimezones.CODE)) units+=ClusterTimezones.INTERNAL;
                    else {
                        is2 = getClass().getResourceAsStream("/data/units/"+code+".list");
                        if (is2!=null) {
                            br2 = new BufferedReader(new InputStreamReader(is2,"ISO-8859-1"));
                            while ((thisLine = br2.readLine()) != null) {
                                if (!thisLine.startsWith("#") && !thisLine.equals("")) units++;
                            }
                            br2.close();
                        } else {
                            System.err.println(code+".list not found.");
                        }
                    }
                }
            }
            br.close();
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            JOptionPane.showMessageDialog(this,GeneralString.message(rb.getString("Moduls.Count"),units,categories),"Info",JOptionPane.INFORMATION_MESSAGE);
        } catch (Exception e) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            JOptionPane.showMessageDialog(this,e.toString(),"Error",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_countUnitsMenuItemActionPerformed

    private void helpIndexMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpIndexMenuItemActionPerformed
        // Add your handling code here:
        if (!helpGenerated) {
            try {
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                generatingHelp();
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                helpGenerated=true;
                helpIndexMenuItem.addActionListener(new CSH.DisplayHelpFromSource(mainHB));
                helpIndexMenuItem.doClick();
            } catch (Exception ee) {
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                error(ee.toString());
            }
        }
    }//GEN-LAST:event_helpIndexMenuItemActionPerformed

    private void ptRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ptRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("pt","BR","en");
        ptRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_ptRadioButtonMenuItemActionPerformed

    private void frRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_frRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("fr","FR","en");
        frRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_frRadioButtonMenuItemActionPerformed

    private void deRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("de","DE","de");
        deRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_deRadioButtonMenuItemActionPerformed

    private void enRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_enRadioButtonMenuItemActionPerformed
        // Add your handling code here:
        xRadioButtonMenuItem("en","GB","en");
        enRadioButtonMenuItem.setSelected(true);
    }//GEN-LAST:event_enRadioButtonMenuItemActionPerformed

    private void updateExchangeRatesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateExchangeRatesMenuItemActionPerformed
        // Add your handling code here:

        ExchangeRates exchangeRates = new ExchangeRates();
        exchangeRates.setPreferredFilter(systemProps.getProperty("rates_preferred",""));
        exchangeRates.setPreferredDirectory(systemProps.getProperty("rates_preferred_dir",null));
        new ExchangeRatesUpdateDialog(this,rb,exchangeRates);

        if (exchangeRates.success()) {
            try {
                exchangeRates.writeToDisk();
                systemProps.setProperty("rates_date",exchangeRates.getDate());
                systemProps.setProperty("rates_logic",exchangeRates.getLogic() ? "true":"false");
                systemProps.setProperty("rates_preferred",exchangeRates.getPreferredFilter());
                systemProps.setProperty("rates_preferred_dir",exchangeRates.getPreferredDirectory());
                saveSystemProperties();
            } catch (Exception e){ System.err.println(e); }

            // modify the exchange rates category record
            SectionRecord sr = (SectionRecord)categoriesHashtable.get("exchange_rates");

            sr.setString(GeneralString.replaceString(exchange_rates_description,"$RATES_DATE", exchangeRates.getDate()));
            sr.setLogic(exchangeRates.getLogic());

            // reset the filter, if exchange rates are not loaded
            if (!groupsHashtable.containsKey("exchange_rates")) {
               clickOnFilterCategoryIndicator();
            }
            categoryBox.setSelectedItem(categoriesHashtable.get("exchange_rates"));
       //     categoryBox.getModel().setSelectedItem(sr);

            transferValue="1";
            categoryBoxItemStateChanged();
        }
        messageOk(GeneralString.message(rb.getString("GUI.Rates.count"),exchangeRates.count()),true);
    }//GEN-LAST:event_updateExchangeRatesMenuItemActionPerformed

    private void saveSystemProperties() {
        if (debug) System.out.println("Saving system properties...");
        try {
            FileOutputStream out = new FileOutputStream("../data/"+FILE_SYSTEM_PROPERTIES);
            systemProps.store(out, "/* properties updated */");
            if (debug) System.out.println("OK.");
        } catch (Exception e) {
            try {
                FileOutputStream out = new FileOutputStream("../../data/"+FILE_SYSTEM_PROPERTIES);
                systemProps.store(out, "/* properties updated */");
                if (debug) System.out.println("OK.");
            } catch (Exception e2) {
                System.err.println(e2.toString());
            }
        }
    }

    private void saveUserProperties() {
        if (debug) System.out.print("Saving user properties...");
        try {
            FileOutputStream out = new FileOutputStream(System.getProperty("user.home")+File.separator+FILE_USER_PROPERTIES);
            userProps.store(out,"/* properties updated */");
            if (debug) System.out.println("OK.");
        } catch (Exception e3) {
            System.err.println(e3.toString());
        }
    }

    private void xRadioButtonMenuItem(String language, String country, String help) {
        //if (!Locale.getDefault().getLanguage().equals(language)) {
            // don't panic message
            dontPanic();
            Locale.setDefault(new Locale(language,country));
            if (GeneralProgram.isSupportFor("1.4"))
               JColorChooser.setDefaultLocale(Locale.getDefault());
            if (!jnlp)
            {
              userProps.setProperty("language",language);
              //userProps.setProperty("iso3166",language);
              userProps.setProperty("country",country);
              userProps.setProperty("help",help);

            }
            restart();
        //}
    }

    private void setBoundsForUserProps() {
        userProps.setProperty("width",Integer.toString(getWidth()));
        userProps.setProperty("heigth",Integer.toString(getHeight()));
        userProps.setProperty("locationX",Integer.toString(getX()));
        userProps.setProperty("locationY",Integer.toString(getY()));
    }

    // restart the application
    private void restart() {
        if (!jnlp) {
          setBoundsForUserProps();
          saveUserProperties();
        }
        helpGenerated=false;
        init(false);
    }

    private void dontPanic() {
        JOptionPane.showMessageDialog(this,rb.getString("Message.GUI.DontPanic"),rb.getString("GUI.Menu.Main.Restart"),JOptionPane.INFORMATION_MESSAGE,  new javax.swing.ImageIcon(getClass().getResource("/core/images/main/hint48.png")));
    }

    private void macRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_macRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(GeneralGUI.LAF_MAC);
          userProps.setProperty("lnf","mac");
          checkForFrameDecoration();
        }
    }//GEN-LAST:event_macRadioButtonMenuItemItemStateChanged

    private void windowsRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_windowsRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(GeneralGUI.LAF_WINDOWS);
          userProps.setProperty("lnf","windows");
          checkForFrameDecoration();
        }
    }//GEN-LAST:event_windowsRadioButtonMenuItemItemStateChanged

    public void checkForFrameDecoration() {
        if (!first && userProps.getProperty("decoration","native").equals("java")) {
            JOptionPane.showMessageDialog(this, rb.getString("GUI.Options.FrameDecorationMessage"), "", JOptionPane.WARNING_MESSAGE);
        }
    }
    private void motifRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_motifRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setLaf(GeneralGUI.LAF_MOTIF);
          userProps.setProperty("lnf","motif");
          checkForFrameDecoration();
        }
    }//GEN-LAST:event_motifRadioButtonMenuItemItemStateChanged

    private void swingRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_swingRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
        if (evt.getStateChange()==ItemEvent.SELECTED) {
          setMetalTheme(defaultMetalTheme);
          userProps.setProperty("lnf","metal_default");
        }
    }//GEN-LAST:event_swingRadioButtonMenuItemItemStateChanged

    private void miserlyRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_miserlyRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
       if (evt.getStateChange()==ItemEvent.SELECTED) luxuryFrame(3);
    }//GEN-LAST:event_miserlyRadioButtonMenuItemItemStateChanged

    private void economicalRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_economicalRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
       if (evt.getStateChange()==ItemEvent.SELECTED) luxuryFrame(2);
    }//GEN-LAST:event_economicalRadioButtonMenuItemItemStateChanged

    private void standardRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_standardRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
      if (evt.getStateChange()==ItemEvent.SELECTED) luxuryFrame(1);
    }//GEN-LAST:event_standardRadioButtonMenuItemItemStateChanged

    private void luxuryRadioButtonMenuItemItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_luxuryRadioButtonMenuItemItemStateChanged
        // Add your handling code here:
      if (evt.getStateChange()==ItemEvent.SELECTED) luxuryFrame(0);
    }//GEN-LAST:event_luxuryRadioButtonMenuItemItemStateChanged

    private void KunststoffCustomMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_KunststoffCustomMenuItemActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_KunststoffCustomMenuItemActionPerformed

  private void targetMenuItemFiguresActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemFiguresActionPerformed
      // Add your handling code here:
      precisionTextField.requestFocus();
  }//GEN-LAST:event_targetMenuItemFiguresActionPerformed

  private void historyPopupMenuItemNewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPopupMenuItemNewActionPerformed
      // Add your handling code here:
      historyNewButtonActionPerformed(evt);
  }//GEN-LAST:event_historyPopupMenuItemNewActionPerformed

  private void recordMenuItemNewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemNewActionPerformed
      // Add your handling code here:
      historyNewButtonActionPerformed(evt);
  }//GEN-LAST:event_recordMenuItemNewActionPerformed

  private void historyNewButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyNewButtonActionPerformed
      // Add your handling code here:
      performNotice();
      sourceTextArea.requestFocus();
  }//GEN-LAST:event_historyNewButtonActionPerformed

  private void recordMenuItemPasteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemPasteActionPerformed
      // Add your handling code here:
      historyPasteButtonActionPerformed(evt);
  }//GEN-LAST:event_recordMenuItemPasteActionPerformed

  private void recordMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemCopyActionPerformed
      // Add your handling code here:
      historyCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_recordMenuItemCopyActionPerformed

  private void sourceMenuItemPasteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemPasteActionPerformed
      // Add your handling code here:
      sourcePasteButtonActionPerformed(evt);
  }//GEN-LAST:event_sourceMenuItemPasteActionPerformed

  private void sourceMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemCopyActionPerformed
      // Add your handling code here:
      sourceCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_sourceMenuItemCopyActionPerformed

  private void sourceMenuItemPlusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemPlusActionPerformed
      // Add your handling code here:
      sourcePlusButtonActionPerformed(evt);
  }//GEN-LAST:event_sourceMenuItemPlusActionPerformed

  private void sourceMenuItemMinusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemMinusActionPerformed
      // Add your handling code here:
      sourceMinusButtonActionPerformed(evt);
  }//GEN-LAST:event_sourceMenuItemMinusActionPerformed

  private void programMenuItemSwapActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_programMenuItemSwapActionPerformed
      // Add your handling code here:
      swapButtonActionPerformed(evt);
  }//GEN-LAST:event_programMenuItemSwapActionPerformed

  private void recordMenuItemClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordMenuItemClearActionPerformed
      // Add your handling code here:
      historyClearButtonActionPerformed(evt);
  }//GEN-LAST:event_recordMenuItemClearActionPerformed

  private void targetMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetMenuItemCopyActionPerformed
      // Add your handling code here:
      targetCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_targetMenuItemCopyActionPerformed

  private void sourceMenuItemClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMenuItemClearActionPerformed
      // Add your handling code here:
      sourceClearButtonActionPerformed(evt);
  }//GEN-LAST:event_sourceMenuItemClearActionPerformed

  private void targetCopyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetCopyButtonActionPerformed
      // Add your handling code here:
      textAreaCopy(targetTextArea);
  }//GEN-LAST:event_targetCopyButtonActionPerformed

  private void historyMenuItemPasteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemPasteActionPerformed
      // Add your handling code here:
      historyPasteButtonActionPerformed(evt);
  }//GEN-LAST:event_historyMenuItemPasteActionPerformed

  private void historyMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemCopyActionPerformed
      // Add your handling code here:
      historyCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_historyMenuItemCopyActionPerformed

  private void historyAreaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_historyAreaMouseClicked
      // Add your handling code here:
      checkPopupMenu(evt, historyPopupMenu);
  }//GEN-LAST:event_historyAreaMouseClicked

  private void historyAreaMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_historyAreaMouseReleased
      // Add your handling code here:
      checkPopupMenu(evt, historyPopupMenu);
  }//GEN-LAST:event_historyAreaMouseReleased

  private void historyAreaMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_historyAreaMousePressed
      // Add your handling code here:
      checkPopupMenu(evt, historyPopupMenu);
  }//GEN-LAST:event_historyAreaMousePressed

  private void historyMenuItemClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemClearActionPerformed
      // Add your handling code here:
      historyClearButtonActionPerformed(evt);
  }//GEN-LAST:event_historyMenuItemClearActionPerformed

  private void sourcePopupMenuItemMinusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemMinusActionPerformed
      // Add your handling code here:
      sourceMinusButtonActionPerformed(evt);
  }//GEN-LAST:event_sourcePopupMenuItemMinusActionPerformed

  private void sourcePopupMenuItemPlusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemPlusActionPerformed
      // Add your handling code here:
      sourcePlusButtonActionPerformed(evt);
  }//GEN-LAST:event_sourcePopupMenuItemPlusActionPerformed

  private void sourcePasteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePasteButtonActionPerformed
      // Add your handling code here:
      textAreaPaste(sourceTextArea);
  }//GEN-LAST:event_sourcePasteButtonActionPerformed

  private void sourceCopyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceCopyButtonActionPerformed
      // Add your handling code here:
      textAreaCopy(sourceTextArea);
  }//GEN-LAST:event_sourceCopyButtonActionPerformed

  private void sourceClearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceClearButtonActionPerformed
      // Add your handling code here:
      textAreaCopy(sourceTextArea);
      textAreaClear(sourceTextArea);
  }//GEN-LAST:event_sourceClearButtonActionPerformed

  private void targetTextAreaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_targetTextAreaMouseClicked
      // Add your handling code here:
      checkPopupMenu(evt, targetPopupMenu);
  }//GEN-LAST:event_targetTextAreaMouseClicked

  private void targetTextAreaMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_targetTextAreaMouseReleased
      // Add your handling code here:
      checkPopupMenu(evt, targetPopupMenu);
  }//GEN-LAST:event_targetTextAreaMouseReleased

  private void targetTextAreaMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_targetTextAreaMousePressed
      // Add your handling code here:
      checkPopupMenu(evt, targetPopupMenu);
  }//GEN-LAST:event_targetTextAreaMousePressed

  private void targetPopupMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetPopupMenuItemCopyActionPerformed
      // Add your handling code here:
      targetCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_targetPopupMenuItemCopyActionPerformed

  private void sourcePopupMenuItemPasteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemPasteActionPerformed
      // Add your handling code here:
      sourcePasteButtonActionPerformed(evt);
  }//GEN-LAST:event_sourcePopupMenuItemPasteActionPerformed

  private void sourcePopupMenuItemClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemClearActionPerformed
      // Add your handling code here:
      sourceClearButtonActionPerformed(evt);
  }//GEN-LAST:event_sourcePopupMenuItemClearActionPerformed

  private void sourcePopupMenuItemCopyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePopupMenuItemCopyActionPerformed
      // Add your handling code here:
      sourceCopyButtonActionPerformed(evt);
  }//GEN-LAST:event_sourcePopupMenuItemCopyActionPerformed

  private void sourceTextAreaMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sourceTextAreaMouseReleased
      // Add your handling code here:
      checkPopupMenu(evt, sourcePopupMenu);
  }//GEN-LAST:event_sourceTextAreaMouseReleased

  private void sourceTextAreaMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sourceTextAreaMousePressed
      // Add your handling code here:
      checkPopupMenu(evt, sourcePopupMenu);
  }//GEN-LAST:event_sourceTextAreaMousePressed

  private void sourceTextAreaMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sourceTextAreaMouseClicked
      // Add your handling code here:
      checkPopupMenu(evt, sourcePopupMenu);
  }//GEN-LAST:event_sourceTextAreaMouseClicked

  private void sourceNextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceNextButtonActionPerformed
      // Add your handling code here:

      int now = sourceBox.getSelectedIndex();
      int all = sourceBox.getItemCount();
      if (now+1 < all) sourceBox.setSelectedIndex(now+1); else sourceBox.setSelectedIndex(0);

  }//GEN-LAST:event_sourceNextButtonActionPerformed

  private void restartItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_restartItemActionPerformed
      // Add your handling code here:
      dontPanic();
      restart();
  }//GEN-LAST:event_restartItemActionPerformed


  private void exitItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitItemActionPerformed
      // Add your handling code here:
      quit();
  }//GEN-LAST:event_exitItemActionPerformed


  private void helpMenuItemAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpMenuItemAboutActionPerformed
      // Add your handling code here:
      showAboutDialog();
  }//GEN-LAST:event_helpMenuItemAboutActionPerformed


  private void sourceBackButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceBackButtonActionPerformed
      // Add your handling code here:
      int now = sourceBox.getSelectedIndex();
      int all = sourceBox.getItemCount();
      if (now > 0) sourceBox.setSelectedIndex(now-1); else sourceBox.setSelectedIndex(all-1);
  }//GEN-LAST:event_sourceBackButtonActionPerformed

  private void historyPasteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyPasteButtonActionPerformed
      // Add your handling code here:
      textAreaPaste(historyTextArea);
  }//GEN-LAST:event_historyPasteButtonActionPerformed


  private void precisionTextFieldKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_precisionTextFieldKeyReleased
      // Add your handling code here:
      int code = evt.getKeyCode();
      boolean b = true;
      for (int i=0; i < GeneralGUI.noTextChangeKeyEvents.length; i++) {
          if (code == GeneralGUI.noTextChangeKeyEvents[i]) { b=false; break; }
      }
      if (b) {
          try {
              if (precisionTextField.getText().length() > 4)
                  throw new Exception();
              int i = Integer.parseInt(precisionTextField.getText());
              if (i > 1000) throw new Exception();
              clusterObject.setPrecision(i);
              precisionerror=false;
              process();
          } catch (Exception e) {
              messageError(rb.getString(ERROR_GUI_PRECISION));
              precisionerror=true;
          }
      }
  }//GEN-LAST:event_precisionTextFieldKeyReleased


  private void historyCopyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyCopyButtonActionPerformed
      // Add your handling code here:
      textAreaCopy(historyTextArea);
  }//GEN-LAST:event_historyCopyButtonActionPerformed


  private void historyClearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyClearButtonActionPerformed
      // Add your handling code here:
      textAreaCopy(historyTextArea);
      textAreaClear(historyTextArea);
  }//GEN-LAST:event_historyClearButtonActionPerformed


  private void sourceTextAreaKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_sourceTextAreaKeyReleased
      // Add your handling code here:
      int code = evt.getKeyCode();
      boolean b = true;
      for (int i=0; i < GeneralGUI.noTextChangeKeyEvents.length; i++) {
          if (code == GeneralGUI.noTextChangeKeyEvents[i]) { b=false; break; }
      }
      if (code==KeyEvent.VK_ENTER) {
          sourceTextArea.setText(sourceTextArea.getText().replace('\n',' ').replace('\r',' ').trim());
      }
      if (b) process();
  }//GEN-LAST:event_sourceTextAreaKeyReleased


  private void sourceMinusButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourceMinusButtonActionPerformed
      // Add your handling code here:
      try {
          addValue(MINUS_ONE);
      } catch (Exception e) {
          clusterObject.setInput(sourceTextArea.getText());
          messageError(e.getMessage());
      }
      sourceTextArea.requestFocus();
  }//GEN-LAST:event_sourceMinusButtonActionPerformed


  private void sourcePlusButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sourcePlusButtonActionPerformed
      // Add your handling code here:
      try {
          addValue(PLUS_ONE);
      } catch (Exception e) {
          clusterObject.setInput(sourceTextArea.getText());
          messageError(e.getMessage());
      }
      sourceTextArea.requestFocus();
  }//GEN-LAST:event_sourcePlusButtonActionPerformed


  private void backButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backButtonActionPerformed
      // Add your handling code here:
      int now = targetBox.getSelectedIndex();
      int all = targetBox.getItemCount();
      if (now > 0) targetBox.setSelectedIndex(now-1); else targetBox.setSelectedIndex(all-1);
  }//GEN-LAST:event_backButtonActionPerformed


  private void nextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextButtonActionPerformed
      // Add your handling code here:
      int now = targetBox.getSelectedIndex();
      int all = targetBox.getItemCount();
      if (now+1 < all) targetBox.setSelectedIndex(now+1); else targetBox.setSelectedIndex(0);
  }//GEN-LAST:event_nextButtonActionPerformed


  private void swapButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_swapButtonActionPerformed
      // Add your handling code here:
      // source should be target, target should be source

      if (!(clusterObject instanceof ClusterSpokenNumbers)) {

      if (!clusterObject.acceptEmptyStrings())
          sourceTextArea.setText(targetTextArea.getText());

      int itemp=sourceBox.getSelectedIndex();
      sourceBox.setSelectedIndex(targetBox.getSelectedIndex());
      targetBox.setSelectedIndex(itemp);
      process();
      sourceTextArea.setCaretPosition(0);
      targetTextArea.setCaretPosition(0);
      sourceTextArea.requestFocus();

      }  else {
         try {
             clusterObject.play(targetBox.getSelectedIndex());
         } catch (Exception e) {
             messageLightError(e.getMessage());
         }
      }

  }//GEN-LAST:event_swapButtonActionPerformed

   public void exitForm(java.awt.event.WindowEvent evt) {
      quit();
   }

  /** Exit the Application */
    private void quit() {
        int result=JOptionPane.YES_OPTION;
        if (userProps.getProperty("askBeforeExit","true").equals("true"))
            result = JOptionPane.showConfirmDialog(this,rb.getString("Message.GUI.ReallyQuit"),rb.getString("GUI.Menu.Main.Exit"), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, new javax.swing.ImageIcon(getClass().getResource("/core/images/main/quit.png"))  );
        if (result == JOptionPane.YES_OPTION) {
          setBoundsForUserProps();
          saveUserProperties();
          if (userProps.getProperty("exitAnimation","false").equals("true")) {
              doExitAnimation();
          }
          hide();
          this.dispose();  // for compatibility with the Java Shell
          System.exit(0);
        }
    }

    // the method doExitAnimation() is based on the work of Hughes Monget,
    // he is the author of JPassGen - a really cool product by the way ;-)
    // see http://jpassgen.sourceforge.net/
    private void doExitAnimation() {

        Point oldP = getLocation();
        Dimension dim = getSize();

        oldP.x += dim.width  / 2;
        oldP.y += dim.height / 2;

        int loop = 1;
        if (dim.width > dim.height)
            loop = dim.width / 32; else
                loop = dim.height / 32;

        int dx = dim.width  / loop;
        int dy = dim.height / loop;

        for (int ii = 0 ; ii < loop ; ii++) {
            dim.width  -= dx;
            dim.height -= dy;
            setSize(dim);
            Dimension newDim = getSize();
            Point newP = new Point();
            newP.x = oldP.x - newDim.width  / 2;
            newP.y = oldP.y - newDim.height / 2;
            setLocation(newP);
            try {
                Thread.currentThread().sleep(100 / loop);
            } catch (InterruptedException e) { ; }
        }
    }

    public ResourceBundle getResourceBundle() {
        return rb;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    private java.awt.CardLayout cardLayout;
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu LaFMenu;
    private javax.swing.JLabel STLabel;
    private javax.swing.JMenuItem addFavoriteMenuItem;
    private javax.swing.JRadioButtonMenuItem aquaRadioButtonMenuItem;
    private javax.swing.JButton backButton;
    private javax.swing.JPanel bottomPanel;
    private javax.swing.JPanel calPanel;
    private javax.swing.JMenuItem cancelFavoritesMenuItem;
    private javax.swing.JMenuItem cancelGroupsMenuItem;
    private javax.swing.JPanel cardPanel;
    private javax.swing.JMenu categoriesMenu;
    private javax.swing.JComboBox categoryBox;
    private javax.swing.JPanel colPanel;
    private javax.swing.JButton colorButton;
    private javax.swing.JMenu configMenu;
    private javax.swing.JLabel conversionLabel;
    private javax.swing.JPanel converterPanel;
    private javax.swing.JMenuItem countUnitsMenuItem;
    private javax.swing.JRadioButtonMenuItem deRadioButtonMenuItem;
    private javax.swing.JMenuItem deactivateFilterCategoriesMenuItem;
    private javax.swing.JRadioButtonMenuItem economicalRadioButtonMenuItem;
    private javax.swing.JMenuItem editCategoriesMenuItem;
    private javax.swing.JMenuItem editUnitsMenuItem;
    private javax.swing.JRadioButtonMenuItem elRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem enRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem esRadioButtonMenuItem;
    private javax.swing.JMenuItem exitItem;
    private javax.swing.JMenu favoritesMenu;
    private javax.swing.JLabel filterCatIndicatorLabel;
    private javax.swing.JLabel filterLabel;
    private javax.swing.JButton findSourceButton;
    private javax.swing.JMenuItem findSourceUnitMenuItem;
    private javax.swing.JButton findTargetButton;
    private javax.swing.JRadioButtonMenuItem frRadioButtonMenuItem;
    private javax.swing.ButtonGroup frameButtonGroup;
    private javax.swing.JMenu frameMenu;
    private javax.swing.JMenu groupsMenu;
    private javax.swing.JRadioButtonMenuItem gtkRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem heRadioButtonMenuItem;
    private javax.swing.JPanel headerPanel;
    private javax.swing.JMenuItem helpIndexMenuItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem helpMenuItemAbout;
    private javax.swing.JPanel historyButtonPanel;
    private javax.swing.JButton historyClearButton;
    private javax.swing.JButton historyCopyButton;
    private javax.swing.JMenuItem historyMenuFontSize;
    private javax.swing.JMenuItem historyMenuItemClear;
    private javax.swing.JMenuItem historyMenuItemCopy;
    private javax.swing.JMenuItem historyMenuItemLoad;
    private javax.swing.JMenuItem historyMenuItemPaste;
    private javax.swing.JMenuItem historyMenuItemRedo;
    private javax.swing.JMenuItem historyMenuItemSave;
    private javax.swing.JMenuItem historyMenuItemUndo;
    private javax.swing.JButton historyNewButton;
    private javax.swing.JScrollPane historyPane;
    private javax.swing.JButton historyPasteButton;
    private javax.swing.JPopupMenu historyPopupMenu;
    private javax.swing.JMenuItem historyPopupMenuItemHTML;
    private javax.swing.JMenuItem historyPopupMenuItemMakeList;
    private javax.swing.JMenuItem historyPopupMenuItemNew;
    private javax.swing.JMenuItem historyPopupMenuenuItemFormat;
    private javax.swing.JTextArea historyTextArea;
    private javax.swing.JMenuItem homepageMenuItem;
    private javax.swing.JMenuItem htmlMenuItem;
    private javax.swing.JButton jButton1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator11;
    private javax.swing.JSeparator jSeparator12;
    private javax.swing.JSeparator jSeparator13;
    private javax.swing.JSeparator jSeparator14;
    private javax.swing.JSeparator jSeparator15;
    private javax.swing.JSeparator jSeparator16;
    private javax.swing.JSeparator jSeparator17;
    private javax.swing.JSeparator jSeparator18;
    private javax.swing.JSeparator jSeparator19;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator20;
    private javax.swing.JSeparator jSeparator21;
    private javax.swing.JSeparator jSeparator22;
    private javax.swing.JSeparator jSeparator23;
    private javax.swing.JSeparator jSeparator24;
    private javax.swing.JSeparator jSeparator25;
    private javax.swing.JSeparator jSeparator26;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JSeparator jSeparator9;
    private javax.swing.JMenu kunststoffMenu;
    private javax.swing.ButtonGroup lafButtonGroup;
    private javax.swing.ButtonGroup langButtonGroup;
    private javax.swing.JMenu languageMenu;
    private javax.swing.JMenuItem loadMenuItem;
    private javax.swing.JRadioButtonMenuItem luxuryRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem macRadioButtonMenuItem;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuItem manageFavoritesMenuItem;
    private javax.swing.JRadioButtonMenuItem maximumRadioButtonMenuItem;
    private javax.swing.JTextField messageTextField;
    private javax.swing.JMenu metouiaMenu;
    private javax.swing.JRadioButtonMenuItem miserlyRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem motifRadioButtonMenuItem;
    private javax.swing.JMenu navigationMenu;
    private javax.swing.JMenuBar ncMenuBar;
    private javax.swing.JButton nextButton;
    private javax.swing.JButton noticesButton;
    private javax.swing.JRadioButtonMenuItem oceanRadioButtonMenuItem;
    private javax.swing.JMenuItem plasticMenuItem;
    private javax.swing.JButton playButton;
    private javax.swing.JLabel precisionLabel;
    private javax.swing.JTextField precisionTextField;
    private javax.swing.JMenu programMenu;
    private javax.swing.JMenuItem programMenuItemSwap;
    private javax.swing.JMenuItem propsMenuItem;
    private javax.swing.JRadioButtonMenuItem ptRadioButtonMenuItem;
    private javax.swing.JMenu recordMenu;
    private javax.swing.JMenuItem recordMenuItemClear;
    private javax.swing.JMenuItem recordMenuItemCopy;
    private javax.swing.JMenuItem recordMenuItemFontsize;
    private javax.swing.JMenuItem recordMenuItemFormat;
    private javax.swing.JMenuItem recordMenuItemMakeList;
    private javax.swing.JMenuItem recordMenuItemNew;
    private javax.swing.JMenuItem recordMenuItemPaste;
    private javax.swing.JPanel recordPanel;
    private javax.swing.JMenuItem redoMenuItem;
    private javax.swing.JMenuItem reloadCategoriesMenuItem;
    private javax.swing.JMenuItem reloadUnitsItem;
    private javax.swing.JMenuItem restartItem;
    private javax.swing.JButton saveButton;
    private javax.swing.JMenuItem saveMenuItem;
    private javax.swing.JCheckBox sciCheckBox;
    private javax.swing.JPanel sciPanel;
    private javax.swing.JMenuItem searchCategoriesMenuItem;
    private javax.swing.JMenu searchMenu;
    private javax.swing.JMenuItem searchSourceMenuItem;
    private javax.swing.JMenuItem searchTargetMenuItem;
    private javax.swing.JPanel soundPanel;
    private javax.swing.JButton sourceBackButton;
    private javax.swing.JComboBox sourceBox;
    private javax.swing.JPanel sourceButtonPanel;
    private javax.swing.JButton sourceClearButton;
    private javax.swing.JButton sourceCopyButton;
    private javax.swing.JPanel sourceHeaderPanel;
    private javax.swing.JMenu sourceMenu;
    private javax.swing.JMenuItem sourceMenuItemCalendar;
    private javax.swing.JMenuItem sourceMenuItemClear;
    private javax.swing.JMenuItem sourceMenuItemColor;
    private javax.swing.JMenuItem sourceMenuItemCopy;
    private javax.swing.JMenuItem sourceMenuItemDefault;
    private javax.swing.JMenuItem sourceMenuItemDefaultValue;
    private javax.swing.JMenuItem sourceMenuItemMinus;
    private javax.swing.JMenuItem sourceMenuItemPaste;
    private javax.swing.JMenuItem sourceMenuItemPlus;
    private javax.swing.JMenuItem sourceMenuItemRedo;
    private javax.swing.JMenuItem sourceMenuItemUndo;
    private javax.swing.JButton sourceMinusButton;
    private javax.swing.JButton sourceNextButton;
    private javax.swing.JPanel sourcePanel;
    private javax.swing.JButton sourcePasteButton;
    private javax.swing.JButton sourcePlusButton;
    private javax.swing.JPopupMenu sourcePopupMenu;
    private javax.swing.JMenuItem sourcePopupMenuDefaultValue;
    private javax.swing.JMenuItem sourcePopupMenuItemClear;
    private javax.swing.JMenuItem sourcePopupMenuItemCopy;
    private javax.swing.JMenuItem sourcePopupMenuItemDefault;
    private javax.swing.JMenuItem sourcePopupMenuItemFindUnit;
    private javax.swing.JMenuItem sourcePopupMenuItemMinus;
    private javax.swing.JMenuItem sourcePopupMenuItemPaste;
    private javax.swing.JMenuItem sourcePopupMenuItemPlus;
    private javax.swing.JMenuItem sourcePopupMenuItemRedo;
    private javax.swing.JMenuItem sourcePopupMenuItemUndo;
    private javax.swing.JScrollPane sourceScrollPane;
    private javax.swing.JTextArea sourceTextArea;
    private javax.swing.JMenuItem spellMenuItem;
    private javax.swing.JMenuItem squarenessMenuItem;
    private javax.swing.JMenu standardMenu;
    private javax.swing.JRadioButtonMenuItem standardRadioButtonMenuItem;
    private javax.swing.JButton stopButton;
    private javax.swing.JMenuItem supportMenuItem;
    private javax.swing.JButton swapButton;
    private javax.swing.JRadioButtonMenuItem swingRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem systemRadioButtonMenuItem;
    private javax.swing.JMenuItem tablesMenuItem;
    private javax.swing.JComboBox targetBox;
    private javax.swing.JPanel targetButtonPanel;
    private javax.swing.JButton targetCopyButton;
    private javax.swing.JPanel targetHeaderPanel;
    private javax.swing.JMenu targetMenu;
    private javax.swing.JMenuItem targetMenuItemCopy;
    private javax.swing.JMenuItem targetMenuItemDefault;
    private javax.swing.JMenuItem targetMenuItemFigures;
    private javax.swing.JMenuItem targetMenuItemFindUnit;
    private javax.swing.JMenuItem targetMenuItemPlay;
    private javax.swing.JMenuItem targetMenuItemStopPlaying;
    private javax.swing.JPanel targetPanel;
    private javax.swing.JPopupMenu targetPopupMenu;
    private javax.swing.JMenuItem targetPopupMenuItemCopy;
    private javax.swing.JMenuItem targetPopupMenuItemDefault;
    private javax.swing.JMenuItem targetPopupMenuItemFindUnit;
    private javax.swing.JMenuItem targetPopupMenuItemPlay;
    private javax.swing.JMenuItem targetPopupMenuItemStopPlaying;
    private javax.swing.JScrollPane targetScrollPane;
    private javax.swing.JTextArea targetTextArea;
    private javax.swing.JMenu themesMenu;
    private javax.swing.JMenuItem undoMenuItem;
    private javax.swing.JMenu unitsMenu;
    private javax.swing.JMenuItem updateExchangeRatesMenuItem;
    private javax.swing.JRadioButtonMenuItem usRadioButtonMenuItem;
    private javax.swing.JRadioButtonMenuItem windowsRadioButtonMenuItem;
    // End of variables declaration//GEN-END:variables

}
